<?php

class Clip {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [false, false, false, false];
		
	}
	
	//---------------------------------------------------------
	//  統計データ公開
	//---------------------------------------------------------
	
	public static function execute() {
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		////////////////////////////////////////////////////////////
		
		// 4回ループ
		for ($i = 1; $i < 5; ++$i) {
			
			// 引数が存在しない時は終了
			if (!isset($GLOBALS['args'][$i])) {
				return;
			}
			
		}
		
		// サイト番号を取得
		$site = (isset($GLOBALS['args'][5])) ? (int) $GLOBALS['args'][5] : 0;
		
		// DBのサフィックス（接尾辞）を設定
		$suffix = '_' . $site . '.sqlite';
		
		// メインDB名を設定
		$mainDB = 'access_main' . $suffix;
		
		////////////////////////////////////////////////////////////
		
		// DBクラスを読み込み
		include(FW_DIR . '/modules/core/DB.php');
		
		// インスタンスを生成
		$db = new DB(DATA_DIR);
		
		// DBに接続
		$db->connect($mainDB);
		
		// DBが存在しない時は終了
		if (!$db->exists($mainDB)) {
			return;
		}
		
		////////////////////////////////////////////////////////////
		
		// パラメータを取得
		$param = $GLOBALS['args'][1];
		
		// 年を取得
		$y = $GLOBALS['args'][2];
		
		// 月を取得（intにキャスト）
		$m = (int) $GLOBALS['args'][3];
		
		// 表示件数を取得
		$limit = $GLOBALS['args'][4];
		
		// 月を2桁に補正
		if ($m < 10) {
			$m = '0' . $m;
		}
		
		////////////////////////////////////////////////////////////
		
		// 月間DB名を設定
		$atDB ='access_' . $y . '_' . $m . $suffix;
		
		// DBが存在しない時は終了
		if (!$db->exists($atDB)) {
			return;
		}
		
		// 月間DBに追加接続
		$db->attach($atDB);
		
		// テンプレートを分割
		[$head, $body, $foot] = $tmpl->read('clip.htm');
		
		// 改行を削除
		$head = preg_replace("/\r|\n|\t/", '', $head);
		$body = preg_replace("/\r|\n|\t/", '', $body);
		$foot = preg_replace("/\r|\n|\t/", '', $foot);
		
		////////////////////////////////////////////////////////////
		
		// JavaScript用ヘッダーを出力
		header("Content-type: application/x-javascript; charset=UTF-8");
		
		// テーブル名を取得
		$paramTable = $GLOBALS['config']['table']['param'];
		
		// クエリーを作成
		$selectSumQuery = "select sum(count) as sum from $paramTable where type = '$param';";
		
		// クエリーを実行
		$sum = $db->queryFetch($selectSumQuery, 'sum');
		
		// データが無い時は終了
		if (!$sum) {
			return;
		}
		
		// SQL
		$selectQuery = '';
		
		// ページ、リンク元、リンク先の時
		if ($param === 'page' || $param === 'referrer' || $param === 'click') {
			
			// テーブル名を取得
			$pageTable = $GLOBALS['config']['table'][$param];
			
			// クエリーを作成
			$selectQuery = "select url, substr(title, 0, 24) as value, count from $paramTable, $pageTable where type = '$param' and value = id order by count desc limit $limit;";
			
		}
		
		// ページ、リンク元、リンク先以外の時
		else {
			
			// クエリーを作成
			$selectQuery = "select value, count from $paramTable where type = '$param' order by count desc limit $limit;";
			
		}
		
		// クエリーを実行
		$results = $db->query($selectQuery);
		
		// JavaScriptヘッダーを出力
		echo "document.write('" . $head;
		
		////////////////////////////////////////////////////////////
		
		// カウンターを初期化
		$i = 1;
		
		// データ取得ループ
		while ($rec = $db->fetch($results)) {
			
			// 色分け用class属性を設定
			$vars['style'] = ($i % 2 === 0) ? '' : 'background: none;';
			
			// パーセンテージを算出
			$per = number_format($rec['count'] / $sum * 100, 1);
			
			// 0.1未満の時は「0.1」をセット
			if ($per === 0) {
				$per = 0.1;
			}
			
			// パーセンテージを設定
			$vars['per'] = $per. '%';
			
			// 順位を設定
			$vars['rank'] = $i;
			
			// ページ、リンク元、リンク先の時
			if ($param === 'page' || $param === 'referrer' || $param === 'click') {
				$vars['value'] = '<a href="' . $rec['url'] . '" target="_blank">' . $rec['value'] . '</a>';
			}
			
			// ページ、リンク元、リンク先以外の時
			else {
				$vars['value'] = $rec['value'];
			}
			
			// カウント数を設定
			$vars['count'] = $rec['count'];
			
			// メインテンプレートを出力
			$tmpl->show($body, $vars);
			
			// カウンターを1増加
			++$i;
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 年、月、種別をテンプレート変数に設定
		$footVars['y'] = $y;
		$footVars['m'] = $m;
		$footVars['param'] = $param;
		
		// JavaScriptフッターを出力
		echo $tmpl->show($foot, $footVars) . "');";
		
	}
	
}

