<?php

class Graph {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  月間パラメータ統計生成
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		////////////////////////////////////////////////////////////
		
		// 表示タイプを取得
		$graph = $GLOBALS['args']['graph'];
		
		// 年を取得
		$y = $GLOBALS['args']['y'];
		
		// 月を取得
		$m = $GLOBALS['args']['m'];
		
		// 日を取得
		$d = $GLOBALS['args']['d'];
		
		// 折れ線グラフ使用フラグを取得
		$allowLineChart = $GLOBALS['config']['content']['allowLineChart'];
		
		////////////////////////////////////////////////////////////
		
		// UU数配列
		$uu = [];
		
		// PV数配列
		$pv = [];
		
		// ループの開始値
		$begin = 0;
		
		// ループの終了値
		$end = 23;
		
		// 折れ線グラフの開始幅
		$left = 0;
		
		// 折れ線グラフの移動幅
		$length = 0;
		
		// 月初の曜日
		$wdayCode = 0;
		
		// 日別、時間別ログの件数
		$count = 1;
		
		////////////////////////////////////////////////////////////
		
		// 月の日数を取得
		$monthDays = Header::monthDays($y, $m + 0);
		
		// テンプレートを分割
		[$head, $thead, $graphTd, $tr, $dateTd, $foot] = $tmpl->read('graph.htm');
		
		////////////////////////////////////////////////////////////
		
		// 端末種別を取得
		[$device, $os] = Controller::device();
		
		// スマホの時
		if ($device === 1) {
			$width = ($os === 1) ? 354 : 349;
		}
		
		// タブレットの時
		elseif ($device === 2) {
			$width = 744;
		}
		
		// PCの時
		else {
			$width = 672;
		}
		
		////////////////////////////////////////////////////////////
		
		// DB名を設定
		$atDB = 'access_' . $y . '_' . $m . $suffix;
		
		// 月別の時
		if ($graph === 'monthly') {
			
			// ループの開始値を設定
			$begin = 1;
			
			// ループの終了値を設定
			$end = 12;
			
			// グラフ生成用データを取得
			[$uu, $pv] = self::monthly($db, $y, $suffix);
			
			// スマホの時
			if ($device === 1) {
				
				// iOSの時
				if ($os === 1) {
					$length = 29.7;
					$left = 13;
				}
				
				// Androidの時
				else {
					$length = 28.5;
					$left = 12;
				}
				
			}
			
			// タブレットの時
			elseif ($device === 2) {
				$length = 63.3;
				$left = 26;
			}
			
			// PCの時
			else {
				$length = 56.1;
				$left = 28;
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// DBが存在する時
		elseif ($db->exists($atDB)) {
			
			// DBに接続
			$db->attach($atDB);
			
			// テーブル名を取得
			$countTable = $GLOBALS['config']['table']['count'];
			
			// クエリーを作成
			$query = "select count(*) as count from $countTable;";
			
			// ログの件数を取得
			$count = $db->queryFetch($query, 'count');
			
			// グラフ生成用データを取得
			[$uu, $pv] = self::$graph($db, $y, $m, $d, $countTable);
			
			////////////////////////////////////////////////////////////
			
			// 時間別の時
			if ($graph === 'timely') {
				
				// スマホの時
				if ($device === 1) {
					
					// iOSの時
					if ($os === 1) {
						$length = 14.7;
						$left = 8;
					}
					
					// Androidの時
					else {
						$length = 14.1;
						$left = 7;
					}
					
				}
				
				// タブレットの時
				elseif ($device === 2) {
					$length = 31.2;
					$left = 15;
				}
				
				// PCの時
				else {
					$length = 28;
					$left = 15;
				}
				
			}
			
			////////////////////////////////////////////////////////////
			
			// 日別の時
			else {
				
				// スマホの時
				if ($device === 1) {
					
					// iOSの時
					if ($os === 1) {
						$length = 11.4;
						$left = 6;
					}
					
					// Androidの時
					else {
						$length = 10.9;
						$left = 6;
					}
					
				}
				
				// タブレットの時
				elseif ($device === 2) {
					$length = 24.1;
					$left = 10;
				}
				
				// PCの時
				else {
					$length = 21.5;
					$left = 16;
				}
				
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 日別の時
		if ($graph === 'daily') {
			
			// 月初の曜日を取得
			$wdayCode = self::wdayCode($y, $m, 1);
			
			// ループの開始値を設定
			$begin = 1;
			
			// ループの終了値を設定
			$end = 31;
			
		}
		
		// 終了値を調整
		++$end;
		
		////////////////////////////////////////////////////////////
		
		// PV数の最大値を算出
		$maxCount = (count($pv)) ? max($pv) : 0;
		
		// 最大値,半分数,基準ピクセルを取得
		[$max, $px] = self::maxPx($maxCount, 180);
		
		// 平均値算出用の分母を取得
		$limit = ($graph === 'daily') ? $count : count($uu);
		
		// 各種UUを初期化
		$vars['sumUU'] = 0;
		$vars['aveUU'] = 0;
		$vars['maxUU'] = 0;
		
		// 各種PVを初期化
		$vars['sumPV'] = 0;
		$vars['avePV'] = 0;
		$vars['maxPV'] = 0;
		
		// UUデータが存在する時は算出
		if ($uu) {
			$vars['sumUU'] = array_sum($uu);
			$vars['aveUU'] = round($vars['sumUU'] / $limit);
			$vars['maxUU'] = max($uu);
		}
		
		// PVデータが存在する時は算出
		if ($pv) {
			$vars['sumPV'] = array_sum($pv);
			$vars['avePV'] = round($vars['sumPV'] / $limit);
			$vars['maxPV'] = max($pv);
		}
		
		////////////////////////////////////////////////////////////
		
		// URLパスを設定
		$vars['path'] = PATH;
		
		// 表示タイプを設定
		$vars['graph'] = $graph;
		
		// ヘッダーテンプレートを出力
		$tmpl->show($head . $thead, $vars);
		
		////////////////////////////////////////////////////////////
		
		// テンプレート変数を初期化
		$vars = [];
		
		// 折れ線グラフの時
		if ($allowLineChart) {
			$vars['graphHtml'] = self::line($width, $px, $uu, $pv, $begin, $end, $length, $left);
		}
		
		// 棒グラフの時
		else {
			$vars['graphHtml'] = self::bar($width, $px, $uu, $pv, $begin, $end, $tmpl, $graphTd, $dateTd);
		}
		
		// 中間<tr>テンプレートを出力
		$tmpl->show($tr);
		
		// 基準<tr>テンプレートを出力
		self::bottomTr($tmpl, $graph, $y, $m, $begin, $end, $wdayCode, $monthDays, $dateTd);
		
		// フッターテンプレートを出力
		$tmpl->show($foot, $vars);
		
	}
	
	//---------------------------------------------------------
	//  時間別アクセス推移
	//---------------------------------------------------------
	
	private static function timely($db, $y, $m, $d, $countTable) {
		
		// 変数を初期化
		$uu = [];
		$pv = [];
		
		// クエリーを作成
		$query = "select * from $countTable where date = '$y-$m-$d';";
		
		// クエリーを実行
		$results = $db->query($query);
		
		// データ取得ループ
		while ($rec = $db->fetch($results, true)) {
			
			// アクセス数合計ループ
			for ($i = 0; $i < 24; ++$i) {
				
				// カラム名を設定
				$uuCol = ($i < 10) ? 'h0' . $i . '_uu' : 'h' . $i . '_uu';
				$pvCol = ($i < 10) ? 'h0' . $i . '_pv' : 'h' . $i . '_pv';
				
				// ユニーク数にプラス
				$uu[$i] = (isset($uu[$i])) ? $uu[$i] + $rec[$uuCol] : $rec[$uuCol];
				
				// PV数にプラス
				$pv[$i] = (isset($pv[$i])) ? $pv[$i] + $rec[$pvCol] : $rec[$pvCol];
				
			}
			
		}
		
		// グラフ生成用データを返す
		return [$uu, $pv];
		
	}
	
	//---------------------------------------------------------
	//  日別アクセス推移
	//---------------------------------------------------------
	
	private static function daily($db, $y, $m, $d, $countTable) {
		
		// 配列を初期化
		$uu = [];
		$pv = [];
		
		// クエリーを作成
		$query = "select date, wday, sum_uu, sum_pv from $countTable;";
		
		// クエリーを実行
		$results = $db->query($query);
		
		// データ取得ループ
		while ($rec = $db->fetch($results)) {
			
			// 年月日に日付を分割
			[$y, $m, $d] = explode('-', $rec['date']);
			
			// 日をキーに設定
			$key = (int) $d;
			
			// UU配列に設定
			$uu[$key] = $rec['sum_uu'];
			
			// PV配列に設定
			$pv[$key] = $rec['sum_pv'];
			
		}
		
		// グラフ生成用データを返す
		return [$uu, $pv];
		
	}
	
	//---------------------------------------------------------
	//  月別アクセス推移
	//---------------------------------------------------------
	
	private static function monthly($db, $y, $suffix) {
		
		// 配列を初期化
		$uu = [];
		$pv = [];
		
		// 詳細ログテーブル名を設定
		$countTable = $GLOBALS['config']['table']['count'];
		
		// 月ループ
		for ($i = 1; $i < 13; ++$i) {
			
			// 月を2桁に調整
			$m = ($i < 10) ? '0' . $i : $i;
			
			// DB名を設定
			$atDB = 'access_' . $y . '_' . $m . $suffix;
			
			// DBが存在しない時は次へ
			if (!$db->exists($atDB)) {
				continue;
			}
			
			// DBに追加接続
			$db->attach($atDB);
			
			// クエリーを作成
			$query = "select sum(sum_uu) as sum_uu, sum(sum_pv) as sum_pv from $countTable;";
			
			// クエリーを実行
			$rec = $db->queryFetch($query);
			
			// 月間ログが存在する時
			if ($rec) {
				
				// UU配列に設定
				$uu[$i] = $rec['sum_uu'];
				
				// PV配列に設定
				$pv[$i] = $rec['sum_pv'];
				
			}
			
			// DBの追加接続を解除
			$db->detach();
			
		}
		
		// グラフ生成用データを返す
		return [$uu, $pv];
		
	}
	
	//---------------------------------------------------------
	//  基準<tr>生成
	//---------------------------------------------------------
	
	private static function bottomTr($tmpl, $graph, $y, $m, $begin, $end, $wdayCode, $monthDays, $dateTd) {
		
		// 曜日カウント変数
		$j = $wdayCode;
		
		// 基準<td>出力ループ
		for ($i = $begin; $i < $end; ++$i) {
			
			// 曜日カウントが7の時は0にリセット
			if ($j === 7) {
				$j = 0;
			}
			
			// クラス属性を初期化
			$vars['class'] = '';
			
			// 左端<td>の時はid属性を設定
			$vars['id'] = ($i === $begin) ? ' id="td0"' : '';
			
			// 月別表示の時
			if ($graph === 'monthly') {
				
				// 月数を2桁に補正
				$m = ($i < 10) ? '0' . $i : $i;
				
				// 月リンクを整形
				$vars['date'] = '<a href="' . PATH . '/graph/daily/' . $y . '/' . $m . '/">' . $i . '</a>';
				
			}
			
			// 日別表示の時
			elseif ($graph === 'daily') {
				
				// 日曜日の時
				if ($j === 0) {
					$vars['class'] = ' class="sun"';
				}
				
				// 土曜日の時
				elseif ($j === 6) {
					$vars['class'] = ' class="sat"';
				}
				
				// 日付が最大値を超えた時は空欄をセット
				if ($i > $monthDays) {
					$vars['date'] = '';
				}
				
				// データが存在する時
				else {
					
					// 日を2桁に調整
					$d = ($i < 10) ? '0' . $i : $i;
					
					// 日リンクを整形
					$vars['date'] = '<a href="' . PATH . '/graph/timely/' . $y . '/' . $m . '/' . $d . '/">' . $i . '</a>';
					
				}
				
			}
			
			// 時間別表示の時
			else {
				$vars['date'] = $i;
			}
			
			// 基準<td>を出力
			$tmpl->show($dateTd, $vars);
			
			// 曜日変数をプラス
			++$j;
			
		}
		
	}
	
	//---------------------------------------------------------
	//  折れ線グラフ出力
	//---------------------------------------------------------
	
	private static function line($width, $px, $uu, $pv, $begin, $end, $length, $left) {
		
		// キャンバスサイズを設定
		$height = 205;
		
		// 空セルを出力
		echo '<td></td>';
		
		// クラスを読み込み
		include(WORK_DIR . '/modules/Canvas.php');
		
		// グラフデータを返す
		return Canvas::graph($width, $height, $length, $left, $px, $uu, $pv, $begin, $end);
		
	}
	
	//---------------------------------------------------------
	//  画像グラフ出力
	//---------------------------------------------------------
	
	private static function bar($width, $px, $uu, $pv, $begin, $end, $tmpl, $graphTd, $dateTd) {
		
		// グラフ画像ディレクトリ名を取得
		$tdBarColor = $GLOBALS['config']['content']['tdBarColor'];
		
		// 画像パスを設定
		$vars['uuSrc'] = PATH_DIR . '/templates/img/graph/' . $tdBarColor . '/uu.png';
		$vars['pvSrc'] = PATH_DIR . '/templates/img/graph/' . $tdBarColor . '/pv.png';
		
		// グラフ<td>出力ループ
		for ($i = $begin; $i < $end; ++$i) {
			
			// データが存在しない時
			if (!isset($uu[$i]) || !$uu[$i]) {
				
				// 空データを設定
				$vars['id'] = '';
				$vars['date'] = '';
				$vars['class'] = '';
				
				// 空のテンプレートを出力
				$tmpl->show($dateTd, $vars);
				
				// 次へ
				continue;
				
			}
			
			// アクセス数を変数にセット
			$vars['uu'] = $uu[$i];
			$vars['pv'] = $pv[$i];
			
			// グラフ画像の高さを変数にセット
			$vars['uuHeight'] = round($vars['uu'] * $px);
			$vars['pvHeight'] = round($vars['pv'] * $px);
			
			// <td>テンプレートを出力
			$tmpl->show($graphTd, $vars);
			
		}
		
		// 空白を返す
		return;
		
	}
	
	//----------------------------------------
	//  基準 最大・半分・px 値算出
	//----------------------------------------
	
	private static function maxPx($maxCount, $maxPx) {
		
		// 最大値の桁数を取得
		$figure = strlen($maxCount);
		
		// 上1桁目を切り出し
		$lastFigure = substr($maxCount, 0, 1);
		
		// 基準最大値を算出
		$max = ($maxCount < 10) ? 10 : pow(10, $figure - 1) * ($lastFigure + 1);
		
		// 基準ピクセル数を算出
		$px = $maxPx / $max;
		
		// 最大値,ピクセル数を返す
		return [$max, $px];
		
	}
	
	//---------------------------------------------------------
	//  年月日の曜日を取得
	//---------------------------------------------------------
	
	private static function wdayCode($y, $m, $d) {
		
		// 日付をintにキャスト
		$y = (int) $y;
		$m = (int) $m;
		
		// 1月、2月の時は閏年用に調整
		if ($m === 1 || $m === 2) {
			
			// 年を1減少
			--$y;
			
			// 月を12増加
			$m += 12;
			
		}
		
		// 曜日コードを返す
		return ($y + intval($y / 4) - intval($y / 100) + intval($y / 400) + intval((13 * $m + 8) / 5) + $d) % 7;
		
	}
	
}

