<?php

class Index {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  メイン処理
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		// メインDB名を取得
		$mainDB = $GLOBALS['mainDB'];
		
		////////////////////////////////////////////////////////////
		
		// 表示する日別パラメータを取得
		$indexParams = $GLOBALS['config']['content']['indexParams'];
		
		// 日別パラメータの表示上限を取得
		$indexViewLimit = $GLOBALS['config']['content']['indexViewLimit'];
		
		// グラフ種別を取得
		$allowLineChart = $GLOBALS['config']['content']['allowLineChart'];
		
		////////////////////////////////////////////////////////////
		
		// テンプレート変数を初期化
		$vars = [];
		
		// 変数を初期化
		$tdMaxPV = 0;
		$ydMaxPV = 0;
		$tdCounts = [];
		$ydCounts = [];
		
		// 年月日時を取得
		$y = $GLOBALS['args']['y'];
		$m = $GLOBALS['args']['m'];
		$d = $GLOBALS['args']['d'];
		$h = $GLOBALS['args']['h'];
		
		// 前日の年月日を取得
		[$ydY, $ydM, $ydD] = Controller::dates(-1);
		
		// フラグを初期化
		$existsTdCounts = false;
		
		////////////////////////////////////////////////////////////
		
		// メインDBが存在する時
		if ($db->exists($mainDB)) {
			
			// テーブル名を取得
			$totalTable = $GLOBALS['config']['table']['total'];
			
			// クエリーを作成
			$selectQuery = "select total_uu, total_pv from $totalTable;";
			
			// クエリーを実行
			$result = $db->queryFetch($selectQuery);
			
			// カウント数を取得
			$vars['totalUU'] = $result['total_uu'];
			$vars['totalPV'] = $result['total_pv'];
			
		}
		
		// メインDBが存在しない時
		else {
			
			// カウント数を変数に設定
			$vars['totalUU'] = 0;
			$vars['totalPV'] = 0;
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 月間DB名を設定（today）
		$tdDB = 'access_' . $y . '_' . $m . $suffix;
		
		// 月間DBが存在する時
		if ($db->exists($tdDB)) {
			
			// DBに接続
			$db->attach($tdDB, 'td');
			
			// テーブル名を取得
			$countTable = $GLOBALS['config']['table']['count'];
			
			// クエリーを作成
			$selectTdQuery = "select * from $countTable where date = '$y-$m-$d';";
			
			// クエリーを実行
			$tdCounts = $db->queryFetch($selectTdQuery);
			
			// データが存在する時
			if ($tdCounts) {
				
				// テンプレート変数にカウントを設定
				$vars['tdUU'] = $tdCounts['sum_uu'];
				$vars['tdPV'] = $tdCounts['sum_pv'];
				
				// フラグをtrueに設定
				$existsTdCounts = true;
				
			}
			
			// データが存在しない時
			else {
				
				// テンプレート変数にカウントを設定
				$vars['tdUU'] = 0;
				$vars['tdPV'] = 0;
				
				// 空のデータ配列を取得
				$tdCounts = self::createCounts();
				
			}
			
			// 不要な要素は0を設定
			$tdCounts['date'] = 0;
			$tdCounts['wday'] = 0;
			$tdCounts['sum_uu'] = 0;
			$tdCounts['sum_pv'] = 0;
			
			// 最大値を取得
			$tdMaxPV = max($tdCounts);
			
			////////////////////////////////////////////////////////////
			
			// DB名を設定（yesterday）
			$ydDB = 'access_' . $ydY . '_' . $ydM . $suffix;
			
			// DBが存在する時
			if ($db->exists($ydDB)) {
				
				// スキーマを設定
				$schema = '';
				
				// 昨日と今日でDBが違う時
				if ($tdDB !== $ydDB) {
					
					// スキーマを設定
					$schema = 'yd.';
					
					// DBに追加接続
					$db->attach($ydDB, 'yd');
					
				}
				
				// クエリーを作成
				$selectYdQuery = "select * from $schema$countTable where date = '$ydY-$ydM-$ydD';";
				
				// クエリーを実行
				$ydCounts = $db->queryFetch($selectYdQuery);
				
				// 追加接続を切断
				if ($schema) {
					$db->detach('yd');
				}
				
			}
			
			// DBが存在しない時
			else {
				$ydCounts = '';
			}
			
			// データが存在する時
			if ($ydCounts) {
				
				// テンプレート変数にカウントを設定
				$vars['ydUU'] = $ydCounts['sum_uu'];
				$vars['ydPV'] = $ydCounts['sum_pv'];
				
			}
			
			// データが存在しない時
			else {
				
				// テンプレート変数にカウントを設定
				$vars['ydUU'] = 0;
				$vars['ydPV'] = 0;
				
				// 空のデータ配列を取得
				$ydCounts = self::createCounts();
				
			}
			
			// 不要な要素は0を設定
			$ydCounts['date'] = 0;
			$ydCounts['wday'] = 0;
			$ydCounts['sum_uu'] = 0;
			$ydCounts['sum_pv'] = 0;
			
			// 終了値を算出
			$end = $h + 1;
			
			// 現在時刻までの昨日データは「0」を設定
			for ($i = 0; $i < $end; ++$i) {
				
				// カラム名を設定
				$pvName = ($i < 10) ? 'h0' . $i . '_pv' : 'h' . $i . '_pv';
				
				// 0を設定
				$ydCounts[$pvName] = 0;
				
			}
			
			// 最大値を取得
			$ydMaxPV = max($ydCounts);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// カウントデータが存在しない時
		else {
			
			// カウント数を変数に設定
			$vars['ydUU'] = 0;
			$vars['tdUU'] = 0;
			$vars['ydPV'] = 0;
			$vars['tdPV'] = 0;
			
			// カウント配列を取得
			$tdCounts = self::createCounts();
			$ydCounts = self::createCounts();
			
		}
		
		////////////////////////////////////////////////////////////
		
		// メインテンプレートを読み込み
		[$head, $body, $foot] = $tmpl->read('index.htm');
		
		// 大きい方を基準最大値に設定
		$maxCount = ($tdMaxPV > $ydMaxPV) ? $tdMaxPV : $ydMaxPV;
		
		// 最大値,半分数,基準ピクセルを取得
		$px = self::maxFigure($maxCount, 180);
		
		// ヘッダーテンプレートを出力
		$tmpl->show($head, $vars);
		
		////////////////////////////////////////////////////////////
		
		// テンプレート変数を初期化
		$footVars = [];
		
		// 折れ線グラフの時
		if ($allowLineChart) {
			$footVars['graphHtml'] = self::lineGraph($px, $tdCounts, $ydCounts, $h);
		}
		
		// 棒グラフの時
		else {
			$footVars['graphHtml'] = self::barGraph($px, $tdCounts, $ydCounts, $h, $tmpl, $body);
		}
		
		// フッターテンプレートを出力
		$tmpl->show($foot, $footVars);
		
		// 今日のデータがない時は終了
		if (!$existsTdCounts || !$indexParams) {
			return;
		}
		
		////////////////////////////////////////////////////////////
		
		// SQL用年月日を設定
		$GLOBALS['args']['date'] = $y . '-' . $m . '-' . $d;
		
		// リンク用年月日を設定
		$GLOBALS['args']['datePath'] = $y . '/' . $m . '/' . $d;
		
		// パラメータ配列を設定
		$params = explode(',', str_replace(' ', '', $indexParams));
		
		// クラスを読み込み
		include(WORK_DIR . '/actions/ParamDaily.php');
		
		// 日別パラメータを表示
		ParamDaily::view($db, $tmpl, $params, $indexViewLimit, true);
		
	}
	
	//---------------------------------------------------------
	//  折れ線グラフ出力
	//---------------------------------------------------------
	
	private static function lineGraph($px, $tdCounts, $ydCounts, $h) {
		
		// 端末種別を取得
		[$device, $os] = Controller::device();
		
		// キャンバスサイズを設定
		$height = 205;
		
		// スマホの時
		if ($device === 1) {
			
			// iOSの時
			if ($os === 1) {
				$width = 354;
				$length = 14.7;
				$left = 8;
			}
			
			// Androidの時
			else {
				$width = 349;
				$length = 14.1;
				$left = 7;
			}
			
		}
		
		// タブレットの時
		elseif ($device === 2) {
			$width = 744;
			$length = 31.2;
			$left = 15;
		}
		
		// PCの時
		else {
			$width = 672;
			$length = 28;
			$left = 15;
		}
		
		// 空セルを出力
		echo '<td></td>';
		
		// クラスを読み込み
		include(WORK_DIR . '/modules/Canvas.php');
		
		// グラフデータを返す
		return Canvas::index($width, $height, $length, $left, $px, $tdCounts, $ydCounts, $h);
		
	}
	
	//---------------------------------------------------------
	//  棒グラフ出力
	//---------------------------------------------------------
	
	private static function barGraph($px, $tdCounts, $ydCounts, $h, $tmpl, $body) {
		
		// 画像グラフの色を取得
		$tdBarColor = $GLOBALS['config']['content']['tdBarColor'];
		$ydBarColor = $GLOBALS['config']['content']['ydBarColor'];
		
		// テンプレート変数を初期化
		$vars = [];
		
		// インデントタブを設定
		$tab = "\n\t\t\t\t\t\t\t\t";
		
		// 時間別グラフ出力ループ
		for ($i = 0; $i < 24; ++$i) {
			
			// 桁補正
			$zero = ($i < 10) ? '0' : '';
			
			// カラム名を設定
			$uuCol = 'h' . $zero . $i . '_uu';
			$pvCol = 'h' . $zero . $i . '_pv';
			
			// 現在時間以下の時
			if ($i <= $h) {
				
				// 今日グラフ画像を設定
				$vars['uuSrc'] = 'templates/img/graph/' . $tdBarColor . '/uu.png';
				$vars['pvSrc'] = 'templates/img/graph/' . $tdBarColor . '/pv.png';
				
				// 今日のカウントを設定
				$vars['uu'] = $tdCounts[$uuCol];
				$vars['pv'] = $tdCounts[$pvCol];
				
			}
			
			// 現在時間以上の時
			else {
				
				// 昨日グラフ画像を設定
				$vars['uuSrc'] = 'templates/img/graph/' . $ydBarColor . '/uu.png';
				$vars['pvSrc'] = 'templates/img/graph/' . $ydBarColor . '/pv.png';
				
				// 今日のカウントを設定
				$vars['uu'] = $ydCounts[$uuCol];
				$vars['pv'] = $ydCounts[$pvCol];
				
			}
			
			// グラフ画像の高さを取得
			$vars['uuHeight'] = round($vars['uu'] * $px);
			$vars['pvHeight'] = round($vars['pv'] * $px);
			
			// PVカウントが0の時
			if (!$vars['pv']) {
				
				// タブを出力
				echo $tab , '<td></td>';
				
				// 次へ
				continue;
				
			}
			
			// メインテンプレートを出力
			$tmpl->show($body, $vars);
			
		}
		
		// 空白を返す
		return;
		
	}
	
	//---------------------------------------------------------
	//  基準 最大・半分・px 値算出
	//---------------------------------------------------------
	
	private static function maxFigure($maxCount, $maxPx) {
		
		// 最大値の桁数を取得
		$figure = strlen($maxCount);
		
		// 上1桁目を切り出し
		$lastFigure = (int) substr($maxCount, 0, 1);
		
		// 基準最大値を算出
		$max = ($maxCount < 10) ? 10 : pow(10, $figure - 1) * ($lastFigure + 1);
		
		// 基準ピクセル数を算出
		$px = $maxPx / $max;
		
		// 最大値,半数値,ピクセル数を返す
		return $px;
		
	}
	
	//---------------------------------------------------------
	//  カウント配列生成
	//---------------------------------------------------------
	
	private static function createCounts() {
		
		// 配列を初期化
		$counts = [];
		
		// 時間別グラフ出力ループ
		for ($i = 0; $i < 24; ++$i) {
			
			// 桁補正
			$zero = ($i < 10) ? '0' : '';
			
			// カラム名を設定
			$uuCol = 'h' . $zero . $i . '_uu';
			$pvCol = 'h' . $zero . $i . '_pv';
			
			// 配列に0を設定
			$counts[$uuCol] = 0;
			$counts[$pvCol] = 0;
			
		}
		
		// 配列を返す
		return $counts;
		
	}
	
}

