<?php

class Jump {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [false, false, false, false];
		
	}
	
	//---------------------------------------------------------
	//  URLジャンプ
	//---------------------------------------------------------
	
	public static function execute() {
		
		// リファラを取得
		$referrer = preg_replace("/.*\/jump\//", "", $_SERVER['REQUEST_URI']);
		
		// リファラが存在しない時は空欄を返す
		if (!$referrer) {
			return;
		}
		
		// 文字コードをUTF-8に変換
		$referrer = mb_convert_encoding($referrer, 'UTF-8', 'auto');
		
		// リファラをURLと検索語に分割
		$urls = explode('::', $referrer);
		
		// URLを取得
		$url = $urls[0];
		
		// 検索語が無い時
		if (!isset($urls[1])) {
			
			// URL引数が存在する時
			if (preg_match('/([^\?]+\?)(.*)/', $url, $m)) {
				$url = $m[1] . self::utfDecode($m[2]);
			}
			
			// ジャンプ用ヘッダーを出力
			header("Refresh: 0; URL=$url");
			
			// 終了
			return;
			
		}
		
		// リンクテキストを整形
		$text = $url . ' ' . urldecode($urls[1]);
		
		// 検索語を取得
		$words = self::utfEncode($urls[1]);
		
		// 実体参照文字列を変換
		$words = stripslashes($words);
		
		// 変数を初期化
		$encoding = '';
		
		// 文字コードがUTF-8のサイトの時
		if (preg_match("/(google|yahoo)/", $url)) {
			
		}
		
		// 文字コードがShift-JISのサイトの時
		elseif (preg_match("/(excite|biglobe|auone)/", $url)) {
			$encoding = 'SJIS';
		}
		
		// 文字コードがEUC-JPのサイトの時
		elseif (preg_match("/hatena|goo\.ne|livedoor|docomo/", $url)) {
			$encoding = 'EUC-JP';
		}
		
		// 未知の検索エンジンの時
		elseif (strpos($url, '?') !== false) {
			$encoding = self::getEncoding($url);
		}
		
		// 文字エンコーディングの指定が存在する時は変更
		if ($encoding) {
			$words = urldecode($words);
			$words = mb_convert_encoding($words, $encoding, 'auto');
			$words = urlencode($words);
		}
		
		// Googleの時1
		if (strpos($url, 'google.com') !== false) {
			$url = 'https://www.google.com/search?hl=ja&ie=UTF-8&q=';
		}
		
		// Googleの時2
		elseif (strpos($url, 'google.co.jp') !== false) {
			$url = 'https://www.google.co.jp/search?hl=ja&ie=UTF-8&q=';
		}
		
		// Yahoo!の時
		elseif (strpos($url, 'search.yahoo') !== false) {
			$url = 'https://search.yahoo.co.jp/search?p=';
		}
		
		// Bingの時
		elseif (strpos($url, 'bing.com') !== false) {
			$url = 'https://www.bing.com/search?q=';
		}
		
		// 百度の時
		elseif (strpos($url, 'baidu') !== false) {
			$url = 'https://www.baidu.jp/s?ie=utf-8&wd=';
		}
		
		// gooの時
		elseif (strpos($url, 'goo') !== false) {
			$url = 'https://search.goo.ne.jp/web.jsp?MT=';
		}
		
		// infoseekの時
		elseif (strpos($url, 'infoseek') !== false) {
			$url = 'https://search.www.infoseek.co.jp/Web?qt=';
		}
		
		// exciteの時
		elseif (strpos($url, 'excite') !== false) {
			$url = 'https://www.excite.co.jp/search.gw?lang=jp&search=';
		}
		
		// livedoorの時
		elseif (strpos($url, 'livedoor') !== false) {
			$url = 'https://search.livedoor.com/search/?q=';
		}
		
		// はてなの時
		elseif (strpos($url, 'hatena') !== false) {
			$url = 'https://search.hatena.ne.jp/websearch?word=';
		}
		
		// @niftyの時
		elseif (strpos($url, 'nifty') !== false) {
			$url = 'https://search.nifty.com/websearch/search?cflg=検索&q=' ;
		}
		
		// BIGLOBEの時
		elseif (strpos($url, 'biglobe') !== false) {
			$url = 'https://cgi.search.biglobe.ne.jp/cgi-bin/search2-b?q=';
		}
		
		// So-netの時
		elseif (strpos($url, 'so-net') !== false) {
			$url = 'https://www.so-net.ne.jp/search/web/?query=';
		}
		
		// docomoの時
		elseif (strpos($url, 'docomo') !== false) {
			$url = 'https://search.goo.ne.jp/web.jsp?MT=';
		}
		
		// auの時
		elseif (strpos($url, 'auone') !== false) {
			$url = 'https://search.auone.jp/?q=';
		}
		
		// Luna Searchの時
		elseif (strpos($url, 'luna.tv') !== false) {
			$url = 'https://s.luna.tv/search.aspx?q=';
		}
		
		// conduitの時
		elseif (strpos($url, 'conduit') !== false) {
			$url = 'https://search.conduit.com/Results.aspx?q=';
		}
		
		// babylonの時
		elseif (strpos($url, 'babylon') !== false) {
			$url = 'https://search.babylon.com/?q=';
		}
		
		// URLに検索語を追記
		$url .= $words;
		
		// ジャンプ用ヘッダーを出力
		header("Refresh: 0; URL=$url");
		
	}
	
	//---------------------------------------------------------
	//  文字エンコーディングを取得
	//---------------------------------------------------------
	
	private static function getEncoding($url) {
		
		// 基本情報を設定
		$stream['http'] = ['method' => 'GET', 'header' => "User-Agent: Lunalys\r\n"];
		
		// HTTPS用設定
		$stream['ssl'] = ['verify_peer' => false, 'verify_peer_name' => false];
		
		// ストリームコンテキストを生成
		$context = stream_context_create($stream);
		
		// HTMLを読み込み
		$html = @file_get_contents($url, false, $context);
		
		// UTF-8|EUC-JP|Shift_JISの時
		if (preg_match('/(UTF-8|EUC-JP|Shift_JIS)/i', $html, $m)) {
			$encoding = $m[1];
		}
		
		// その他の時
		else {
			$encoding = '';
		}
		
		// 文字エンコーディングを返す
		return $encoding;
		
	}
	
	//---------------------------------------------------------
	//  UTF-8デコード
	//---------------------------------------------------------
	
	private static function utfDecode($words) {
		
		// URLエンコード
		$words = urlencode($words);
		
		// 特殊記号をを元に戻す
		$words = str_replace("%3D", '=', $words);
		$words = str_replace("%26", '&', $words);
		//$words = str_replace("%3A", ':', $words);
		//$words = str_replace("%2F", '/', $words);
		//$words = str_replace("%20", ' ', $words);
		
		// 変換後の文字列を返す
		return $words;
		
	}
	
	//---------------------------------------------------------
	//  UTF-8エンコード
	//---------------------------------------------------------
	
	private static function utfEncode($words) {
		
		// 特殊記号を変換
		$words = str_replace("'" , '%22', $words);
		$words = str_replace("#" , '%23', $words);
		$words = str_replace(":" , '%3A', $words);
		$words = str_replace(";" , '%3B', $words);
		$words = str_replace("\*", '%2A', $words);
		$words = str_replace("\+", '%2B', $words);
		$words = str_replace("\-", '%2D', $words);
		
		// 変換後の文字列を返す
		return $words;
		
	}
	
}

