<?php

class LogData {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  管理メニュー表示
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// テーブル名を取得
		$totalTable = $GLOBALS['config']['table']['total'];
		
		// メインDB名を取得
		$mainDB = $GLOBALS['mainDB'];
		
		////////////////////////////////////////////////////////////
		
		// DBが存在しない時は終了
		if (!$db->exists($mainDB)) {
			return;
		}
		
		////////////////////////////////////////////////////////////
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		// URLパスを設定
		$vars['path'] = PATH;
		
		// テンプレートを取得
		[$sectionHead, $head, $body, $foot, $sectionFoot] = $tmpl->read('log_data.htm');
		
		// リクエストメソッドがPOSTの時は累計アクセス数を変更
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			self::update($db);
		}
		
		////////////////////////////////////////////////////////////
		
		// クエリーを作成
		$query = "select * from $totalTable;";
		
		// クエリーを実行
		$rec = $db->queryFetch($query);
		
		// カウントデータが存在する時
		if ($rec) {
			$vars['totalUU'] = $rec['total_uu'];
			$vars['totalPV'] = $rec['total_pv'];
		}
		
		// カウントデータが存在しない時
		else {
			$vars['totalUU'] = 0;
			$vars['totalPV'] = 0;
		}
		
		////////////////////////////////////////////////////////////
		
		// 累計カウント修正、メインテーブルリストを出力
		$tmpl->show($sectionHead, $vars);
		
		// 月間テーブルリストを出力
		self::showMonthlyTable($tmpl, $suffix, $head, $body, $foot);
		
		// フッターを出力
		$tmpl->show($sectionFoot);
		
	}
	
	//---------------------------------------------------------
	//  テーブル名表示
	//---------------------------------------------------------
	
	private static function showMonthlyTable($tmpl, $suffix, $head, $body, $foot) {
		
		// データフォルダのファイル名を取得
		$files = glob(DATA_DIR . '/access*.sqlite');
		
		// ファイル名配列を降順ソート
		rsort($files);
		
		// テーブル名を配列で設定
		$tables = ['count', 'param', 'daily', 'visit'];
		
		// URLパスを設定
		$vars['path'] = PATH;
		
		// 検索DB名を設定
		$reg = 'access_(\d{4}_\d{2})' . $suffix;
		
		// ファイル名を取得するループ
		foreach ($files as $file) {
			
			// マッチしない時は次へ
			if (!preg_match("/$reg/", $file, $m)) {
				continue;
			}
			
			// マッチした文字列を年月に分割
			[$vars['y'], $vars['m']] = explode('_', $m[1]);
			
			// ヘッダーテンプレートを出力
			$tmpl->show($head, $vars);
			
			// テーブル名出力ループ
			foreach ($tables as $table) {
				
				// テーブル名を設定
				$vars['table'] = $table;
				
				// メインテンプレートを出力
				$tmpl->show($body, $vars);
				
			}
			
			// フッターテンプレートを出力
			$tmpl->show($foot);
			
		}
		
	}
	
	//---------------------------------------------------------
	//  累計アクセス数の変更
	//---------------------------------------------------------
	
	private static function update($db) {
		
		// テーブル名を取得
		$totalTable = $GLOBALS['config']['table']['total'];
		
		// アクセス数を取得（intにキャスト）
		$totalUU = ($GLOBALS['args']['total_uu']) ? (int) $GLOBALS['args']['total_uu'] : 0;
		$totalPV = ($GLOBALS['args']['total_pv']) ? (int) $GLOBALS['args']['total_pv'] : 0;
		
		// クエリーを作成
		$query = "update $totalTable set total_uu = $totalUU, total_pv = $totalPV;";
		
		// クエリーを実行
		$db->query($query);
		
	}
	
}

