<?php

class Overlay {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Connect', false];
		
	}
	
	//---------------------------------------------------------
	//  ページオーバーレイ生成
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		////////////////////////////////////////////////////////////
		
		// 共通処理クラスを読み込み
		include(WORK_DIR . '/filters/Header.php');
		
		// インスタンスを生成
		$header = new Header();
		
		// 年月移動リンクを取得
		$nav = $header->createYMLink('overlay', 'url');
		
		// 年月を取得
		$y = $GLOBALS['args']['y'];
		$m = $GLOBALS['args']['m'];
		
		// DB名を設定
		$atDB = 'access_' . $y . '_' . $m . $suffix;
		
		// 戻るリンクを設定
		$no_data = '<a href="javascript:history.back();">no data</a>';
		
		// DBが存在しない時は終了
		if (!$db->exists($atDB)) {
			exit($no_data);
		}
		
		// DBに接続
		else {
			$db->attach($atDB);
		}
		
		// テーブル名を取得
		$pageTable = $GLOBALS['config']['table']['page'];
		$visitTable = $GLOBALS['config']['table']['visit'];
		$paramTable = $GLOBALS['config']['table']['param'];
		
		// リンクを整形
		$link = PATH . '/overlay/' . $y . '/' . $m;
		
		////////////////////////////////////////////////////////////
		
		// 引数が存在する時
		if (isset($GLOBALS['args'][3])) {
			$url = preg_replace("/.*\/(http.+)/A", "$1", $_SERVER['REQUEST_URI']);
		}
		
		// 引数が存在しない時
		else {
			
			// クエリーを作成
			$query = "select url from $paramTable, $pageTable where type = 'page' and value = id order by count desc limit 1;";
			
			// クエリーを実行
			$url = $db->queryFetch($query, 'url');
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 変数を初期化
		$x = [];
		$jsData = '';
		$pageId = 1;
		
		// グローバル変数に設定
		$GLOBALS['args']['ii'] = 0;
		$GLOBALS['args']['url'] = $url;
		$GLOBALS['args']['link'] = $link;
		
		// リンクを入れ替え
		$nav = str_replace('url/', $url, $nav);
		
		////////////////////////////////////////////////////////////
		
		// クエリーを作成
		$query = "select * from $pageTable;";
		
		// クエリーを実行
		$results = $db->query($query);
		
		// オプションフィルタリングループ
		while ($rec = $db->fetch($results)) {
			
			// データを取得
			$id = $rec['id'];
			
			// 特定文字列が含まれる時は除外
			if (preg_match("/[<>'\"\(\)\,*]/", $rec['url'])) {
				continue;
			}
			
			// URLが一致する時
			elseif ($url === $rec['url']) {
				
				// JavaScriptを整形
				$jsData .= "olUrls['" . $rec['url'] . "'] = 'self';\n";
				
				// p_noを別名で保持
				$pageId = $id;
				
			}
			
			// URLが一致しない時
			else {
				
				// URLを配列に設定
				$urls[$id] = $rec['url'];
				
				// 「0」を配列に設定
				$x[$id] = 0;
				
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// クエリーを作成
		$query = "select route from $visitTable where route like '$pageId-%' or route like '%-$pageId-%';";
		
		// クエリーを実行
		$results = $db->query($query);
		
		// トータル数を初期化
		$allCnt = 0;
		
		// レコードデータ表示ループ
		while ($rec = $db->fetch($results)) {
			
			// page_noが一致しない時は次へ
			if (!preg_match("/$pageId(-)([^-]*)/", $rec['route'], $m)) {
				continue;
			}
			
			// 一致移行のp_noを取得
			$z = $m[2];
			
			// カウント数を増加
			$x[$z] = (isset($x[$z])) ? $x[$z] + 1 : 1;
			
			// トータル数を増加
			++$allCnt;
			
		}
		
		// カウント数配列ループ
		foreach ($x as $key => $val) {
			
			// URLが存在する時
			if (isset($urls[$key])) {
				
				// URLを取得
				$xurl = $urls[$key];
				
				// パーセンテージを算出
				$xper = ($allCnt) ? round(($val / $allCnt) * 100) : 0;
				
				// JavaScriptを整形
				$jsData .= "olUrls['$xurl'] = '$xper%';\n";
				
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// ポップアップ用CSSを設定
		$css = <<<_HTML_
<style>

.overlay_per,
#content_date {
	
	position: absolute;
	
	font: normal normal 12px/120% "メイリオ", "Meiryo", "ＭＳ Ｐゴシック", sans-serif !important;
	
	border-radius: 4px;
	
	box-shadow: 3px 3px 4px rgba(0, 0, 0, 0.4);
	
}

.overlay_per {
	
	display: inline;
	
	width: 30px;
	height: 15px;
	
	padding-right: 3px;
	
	border: 1px solid darkgray;
	
	text-align: right !important;
	
	text-decoration: none !important;
	
	color: #666666 !important;
	
	background-color: #f8f8f8 !important;
	
}

#content_date {
	
	top: 5px;
	right: 15px;
	
	padding: 0 5px;
	
	background-color: #ffffff !important;
	
}
</style>
</head>
_HTML_;
		
		////////////////////////////////////////////////////////////
		
		// ポップアップ用JavaScriptを設定
		$js = <<<_HTML_
<script>

// リンクの総数を取得
var length = document.links.length;

// イベント種別を設定
var eventType = (window.ontouchstart === undefined) ? 'mousedown' : 'touchstart';

// 配列を初期化
var olUrls = new Array();

// パーセンテージ配列
$jsData

// リンクの数だけループ
for (i = 0; i < length; i++) {
	
	// 要素を取得
	var node = document.links[i];
	
	// クリックイベントを設定
	//node.onclick = clickLink;
	
	// クリックイベントを設定
	if (window.addEventListener) {
		node.addEventListener(eventType, clickLinkOverlay, false);
	}
	
	// 要素に<span>を追記
	if (olUrls[node] && olUrls[node] != 'self') {
		node.innerHTML += '<span class="overlay_per">' + olUrls[node] + '</span>';
	}
	
}

// リンクタップ時のイベント
function clickLinkOverlay() {
	
	// Lunalys用のURLでページ遷移
	location.href = '$link/' + this.href;
	
	// falseを返す
	return false;
	
}

</script>$nav
</body>
_HTML_;
		
		////////////////////////////////////////////////////////////
		
		// 基準パスをグローバル変数に設定
		$GLOBALS['args']['baseUrl'] = self::urlPath($url);
		
		// プロトコルをグローバル変数に設定
		$GLOBALS['args']['protocol'] = (preg_match('/https:/A', $url)) ? 'https' : 'http';
		
		// 基本情報を設定
		$stream['http'] = ['method' => 'GET', 'header' => "User-Agent: Lunalys\r\n"];
		
		// HTTPS用設定
		$stream['ssl'] = ['verify_peer' => false, 'verify_peer_name' => false];
		
		// ストリームコンテキストを生成
		$context = stream_context_create($stream);
		
		// ファイルデータを読み込み
		$html = @file_get_contents($url, false, $context);
		
		// データが取得出来なかった時は終了
		if (!$html) {
			exit($no_data);
		}
		
		// URLを絶対パスに変換
		$html = preg_replace_callback('/(href|src)(=")([^>]*\.)(.{1,4})("[^>]*>)/i', 'path', $html);
		
		// <head>にCSSを挿入
		$html = preg_replace('/<\/head>/i', $css, $html);
		
		// <body>にJavaScriptを挿入
		$html = preg_replace('/<\/body>/i', $js, $html);
		
		// EUC-JP|Shift_JISの時は文字コードヘッダーを送信
		if (preg_match('/charset=\"?(EUC-JP|Shift_JIS)/i', $html, $h)) {
			header("Content-type: text/html; charset=$h[1]");
		}
		
		// ページデータを出力
		echo $html;
		
	}
	
	//---------------------------------------------------------
	//  URL省略/整形
	//---------------------------------------------------------
	
	public static function urlPath($url) {
		
		// 「http://」「https://」を削除
		$url = preg_replace('/(http|https):\/\//', '', $url);
		
		// 末尾のファイル名は削除
		$url = preg_replace('/\/[^\/]+\..+/','/', $url);
		
		// URLを返す
		return $url;
		
	}
	
}
	
	//---------------------------------------------------------
	//  絶対パス算出
	//---------------------------------------------------------
	
	function path($value) {
		
		// 基準パスを取得
		$baseUrl = $GLOBALS['args']['baseUrl'];
		
		// プロトコルを取得
		$protocol = $GLOBALS['args']['protocol'];
		
		// 絶対パスを取得
		$value[3] = filePath($baseUrl, $value[3], $protocol);
		
		// 先頭の要素を削除
		array_shift($value);
		
		// 配列を繋いで返す
		return implode('', $value);
		
	}
	
	//---------------------------------------------------------
	//  絶対パス算出（詳細）
	//---------------------------------------------------------
	
	function filePath($baseUrl, $file, $protocol) {
		
		// 「./」を削除
		$file = preg_replace('/^\.\//', '', $file);
		
		// 絶対パスの時
		if (preg_match('/^(http|https):/', $file)) {
			return $file;
		}
		
		// ファイル名のみの時
		elseif (preg_match('/^\w/', $file)) {
			return $protocol . '://' . $baseUrl . $file;
		}
		
		// 「/」から始まるURLの時
		elseif (preg_match('/^\//', $file)) {
			
			// URLを解析
			$urlData = parse_url($protocol . '://' . $baseUrl);
			
			// ホスト名を取得
			$host = $urlData['host'];
			
			// 絶対パスを返す
			return $protocol . '://' . $host . $file;
			
		}
		
		// 基準URLを「/」で分割
		$path = explode('/', $baseUrl);
		
		// 対象パスを「../」で分割
		$filePath = explode('../', $file);
		
		// 戻る階層数を取得
		$backCnt = count($filePath);
		
		// 先頭の要素を削除
		$lastPath = array_pop($filePath);
		
		// 戻る階層数だけループ
		for ($i = 0; $i < $backCnt; $i++) {
			array_pop($path);
		}
		
		// 絶対パスを整形
		$file = $protocol . '://' . implode('/', $path) . '/' . $lastPath;
		
		// 絶対パスを返す
		return $file;
		
	}
	
