<?php

class ParamDaily {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  日別統計表示
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		////////////////////////////////////////////////////////////
		
		// 年月日を取得
		$y = $GLOBALS['args']['y'];
		$m = $GLOBALS['args']['m'];
		$d = $GLOBALS['args']['d'];
		
		// DB名を設定
		$atDB = 'access_' . $y . '_' . $m . $suffix;
		
		// DBが存在しない時は終了
		if (!$db->exists($atDB)) {
			return;
		}
		
		// DBに追加接続
		$db->attach($atDB);
		
		////////////////////////////////////////////////////////////
		
		// SQL用年月日を設定
		$GLOBALS['args']['date'] = $y . '-' . $m . '-' . $d;
		
		// リンク用年月日を設定
		$GLOBALS['args']['datePath'] = $y . '/' . $m . '/' . $d;
		
		////////////////////////////////////////////////////////////
		
		// 対象パラメータを取得
		$param = (isset($GLOBALS['args'][1])) ? $GLOBALS['args'][1] : 'list';
		
		// 配列にセット
		$params = [$param];
		
		// limitオプションを設定
		$viewLimit = '';
		
		// 項目一覧の時
		if ($param === 'list') {
			self::showList($tmpl);
		}
		
		// 項目一覧以外の時
		else {
			self::view($db, $tmpl, $params, $viewLimit, false);
		}
		
	}
	
	//---------------------------------------------------------
	//  項目一覧を表示
	//---------------------------------------------------------
	
	private static function showList($tmpl) {
		
		// テンプレートを取得
		$html = $tmpl->read('param_daily_list.htm', false);
		
		// URLパスを設定
		$vars['path'] = PATH;
		
		// 日付パスを設定
		$vars['datePath'] = $GLOBALS['args']['datePath'];
		
		// テンプレートを出力
		$tmpl->show($html, $vars);
		
	}
	
	//---------------------------------------------------------
	//  View表示
	//---------------------------------------------------------
	
	public static function view($db, $tmpl, $params, $viewLimit, $isIndex) {
		
		// 最大表示文字数
		$maxLen = ($isIndex) ? 38 : 80;
		
		// 最大表示文字数を定数に設定
		define('MAX_LEN', $maxLen);
		
		// ジャンプ先定数を設定
		define('JUMP_PATH', PATH . '/jump/');
		
		// 年月日を取得（yyyy-mm-dd）
		$date = $GLOBALS['args']['date'];
		
		// 年月日を取得（yyyy/mm/dd）
		$datePath = $GLOBALS['args']['datePath'];
		
		// テーブル名を取得
		$dailyTable = $GLOBALS['config']['table']['daily'];
		
		// Indexの時
		if ($isIndex) {
			
			// limitオプションを設定
			$limit = 'limit ' . $viewLimit;
			
			// 表示件数制限をintにキャスト
			$viewLimit = (int) $viewLimit;
			
			// テンプレートを分割
			[$wrapHead, $head, $body, $foot, $wrapFoot] = $tmpl->read('param_index.htm');
			
			// テンプレートを出力
			$tmpl->show($wrapHead);
			
		}
		
		// PramDailyの時
		else {
			
			// limitオプションを設定
			$limit = '';
			
			// 表示件数制限を設定
			$viewLimit = 0;
			
			// テンプレートを分割
			[$head, $body, $foot] = $tmpl->read('param_daily.htm');
			
		}
		
		// ループカウンタ
		$p = 1;
		
		// パラメータ表示ループ
		foreach ($params as $param) {
			
			// 未設定のパラメータは何もしない
			if (!isset($GLOBALS['title'][$param])) {
				continue;
			}
			
			// 変数を初期化
			$replace = '';
			$column = '';
			$where = "type = '$param'";
			$from = '';
			$slash = '/';
			
			// テンプレート変数をセット
			$headVars['param'] = $param;
			$headVars['title'] = $GLOBALS['title'][$param];
			$headVars['table'] = ($p % 2 === 0) ? 'right_table' : 'left_table';
			
			// ヘッダーテンプレートを出力
			$tmpl->show($head, $headVars);
			
			// アッパーキャメルに変換
			$ucParam = Controller::toUpperCamel($param);
			
			// 置換メソッドがあるときはセット
			if (method_exists('ParamDaily', 'replace' . $ucParam)) {
				$replace = 'replace' . $ucParam;
			}
			
			////////////////////////////////////////////////////////////
			
			// ページ、リンク元、リンク先の時
			if ($param === 'page' || $param === 'click' || $param === 'referrer') {
				
				// テーブル名を取得
				$table = $GLOBALS['config']['table'][$param];
				
				// カラム名を設定
				$column = ', title, url';
				
				// where句を設定
				$where .= ' and value = id';
				
				// リンク元テーブル名を設定
				$from = ', ' . $table;
				
				// 変換メソッドを設定
				$replace = 'replacePage';
				
			}
			
			// 訪問回数の時
			elseif ($param === 'visit_count') {
				$replace = 'replaceVisit';
			}
			
			// UAの時
			elseif ($param === 'ua') {
				$where = "(type = 'ua' or type = 'mobile' or type = 'game')";
			}
			
			// OSの時
			elseif ($param === 'os') {
				$GLOBALS['os'] = parse_ini_file(WORK_DIR . '/templates/ini/os.ini');
			}
			
			// 端末機種の時
			elseif ($param === 'device') {
				$GLOBALS['device'] = parse_ini_file(WORK_DIR . '/templates/ini/device.ini');
			}
			
			////////////////////////////////////////////////////////////
			
			// クエリーを作成
			$query = "select value, count $column from $dailyTable $from where date = '$date' and $where order by count desc $limit;";
			
			// クエリーを実行
			$results = $db->query($query);
			
			// 簡易検索用リンクを設定
			$link = PATH . '/search/' . $param . '/' . $datePath;
			
			// 端末機種フラグ
			$isDevice = false;
			
			// 端末機種の時
			if ($param === 'device') {
				
				// 端末機種フラグを設定
				$isDevice = true;
				
				// データを合算
				[$optCounts, $allRows] = self::device($db, $results);
				
				// 表示件数を設定
				$limitCount = ($isIndex && $allRows > $viewLimit) ? $viewLimit : $allRows;
				
			}
			
			////////////////////////////////////////////////////////////
			
			// ログを表示するループ
			for ($i = 1; ; ++$i) {
				
				// 端末機種ではない時
				if (!$isDevice) {
					
					// データを取得
					$rec = $db->fetch($results);
					
					// データが無い時はループ終了
					if (!$rec) {
						break;
					}
					
				}
				
				// フィルターが存在する時
				else {
					
					// データが無い時はループ終了
					if ($i > $limitCount) {
						break;
					}
					
					// 配列を初期化
					$rec = [];
					
					// 配列のキーを取得
					$key = key($optCounts);
					
					// 項目名を取得
					$rec['value'] = $key;
					
					// カウント数を取得
					$rec['count'] = $optCounts[$key];
					
					// 配列のポインタを進める
					next($optCounts);
					
				}
				
				////////////////////////////////////////////////////////////
				
				// 簡易検索リンクを設定
				$vars['count'] = '<a href="' . $link . '/' . $rec['value'] . $slash . '">' . $rec['count'] . '</a>';
				
				// 表示文字列を置換
				if ($replace) {
					$vars['value'] = self::$replace($rec);
				} else {
					$vars['value'] = $rec['value'];
				}
				
				// 色分け用class属性を設定
				$vars['tr'] = ($i % 2 === 0) ? 2 : 1;
				
				// 順位を取得
				$vars['rank'] = $i;
				
				// メインテンプレートを出力
				$tmpl->show($body, $vars);
				
			}
			
			// フッターテンプレートを出力
			$tmpl->show($foot);
			
			// カウンタ増加
			++$p;
			
		}
		
		// Indexページの時はテンプレートを出力
		if ($isIndex) {
			$tmpl->show($wrapFoot);
		}
		
	}
	
	//---------------------------------------------------------
	//  端末機種のデータを合算
	//---------------------------------------------------------
	
	private static function device($db, $results) {
		
		// 合算後のデータ配列
		$optCounts = [];
		
		// データを読み込む
		while ($rec = $db->fetch($results)) {
			
			// 項目名を取得
			$value = $rec['value'];
			
			// カウント数を取得
			$count = $rec['count'];
			
			// モデル名を取得
			$optValue = (isset($GLOBALS['device'][$value])) ? $GLOBALS['device'][$value] : $value;
			
			// 配列のデータが存在する時はカウントを増加
			if (isset($optCounts[$optValue])) {
				$optCounts[$optValue] += $count;
			}
			
			// 配列のデータが存在しない時はカウントを配列に設定
			else {
				$optCounts[$optValue] = $count;
			}
			
		}
		
		// 降順にソート
		arsort($optCounts);
		
		// 総数を取得
		$allRows = count($optCounts);
		
		// データを返す
		return [$optCounts, $allRows];
		
	}
	
	//---------------------------------------------------------
	//  表示名の置換：ページ、リンク元、リンク先
	//---------------------------------------------------------
	
	private static function replacePage($vars) {
		
		// リンクを整形
		return '<a href="' . JUMP_PATH . $vars['url'] . '" class="out">' . mb_strimwidth($vars['title'], 0, MAX_LEN, '...', 'UTF-8') . '</a>';
		
	}
	
	//---------------------------------------------------------
	//  表示名の置換：OS
	//---------------------------------------------------------
	
	private static function replaceOS($vars) {
		
		// 値を別名で保持
		$value = $vars['value'];
		
		// 愛称が存在するときはUAに追記
		if (isset($GLOBALS['os'][$value])) {
			$value = $GLOBALS['os'][$value];
		}
		
		// UAを返す
		return $value;
		
	}
	
	//---------------------------------------------------------
	//  表示名の置換：UA
	//---------------------------------------------------------
	
	private static function replaceUA($vars) {
		
		// 文字数を制限
		return mb_strimwidth($vars['value'], 0, MAX_LEN, '...', 'UTF-8');
		
	}
	
	//---------------------------------------------------------
	//  表示名の置換：訪問回数
	//---------------------------------------------------------
	
	private static function replaceVisit($vars) {
		
		// 文字を整形
		return $vars['value'] . '回目';
		
	}
	
	//---------------------------------------------------------
	//  表示名の置換：検索語
	//---------------------------------------------------------
	
	private static function replaceSearchWords($vars) {
		
		// リンクを整形
		return '<a href="' . JUMP_PATH . 'https://www.google.com/' . '::' . preg_replace('/"/', "'", $vars['value']) . '" class="out">' . mb_strimwidth($vars['value'], 0, MAX_LEN, '...', 'UTF-8') . '</a>';
		
	}
	
	//---------------------------------------------------------
	//  表示名の置換：ドメイン
	//---------------------------------------------------------
	
	private static function replaceHostDomain($vars) {
		
		// unknownの時はそのまま返す
		if ($vars['value'] === 'unknown') {
			return $vars['value'];
		}
		
		// リンクを整形
		return '<a href="' . JUMP_PATH . 'https://www.' . $vars['value'] . '/" class="out">' . $vars['value'] . '</a>';
		
	}
	
}

