<?php

class PieChart {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  円グラフを表示
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		////////////////////////////////////////////////////////////
		
		// アクション名を取得
		$act = $GLOBALS['args']['act'];
		
		// 項目種別を取得
		$param = $GLOBALS['args'][1];
		
		// フィルター設定を取得
		$opt = $GLOBALS['args']['opt'];
		
		// ピックアップ項目を取得
		$pickup = $GLOBALS['args']['pickup'];
		
		// 年を取得
		$y = $GLOBALS['args']['y'];
		
		// 月を取得
		$m = $GLOBALS['args']['m'];
		
		////////////////////////////////////////////////////////////
		
		// ジャンプパスを定数に設定
		define('JUMP_PATH', PATH . '/jump/');
		
		// 最大表示文字数を定数に設定
		define('MAX_LEN', 45);
		
		////////////////////////////////////////////////////////////
		
		// DB名を設定
		$atDB = 'access_' . $y . '_' . $m . $suffix;
		
		// DBが存在しない時は終了
		if (!$db->exists($atDB)) {
			return;
		}
		
		////////////////////////////////////////////////////////////
		
		// テーブル名を取得
		$paramTable = $GLOBALS['config']['table']['param'];
		
		// クラスを読み込み
		include(WORK_DIR . '/actions/Param.php');
		
		// テンプレートを分割
		[$head, $body, $foot] = $tmpl->read('param_daily.htm');
		
		////////////////////////////////////////////////////////////
		
		// 変数を初期化
		$opt2 = '';
		$from = '';
		$where = "type = '$param'";
		$limit = '';
		$slash = '/';
		$uaType = '';
		$column1 = 'sum(count) as sum, count(distinct value) as rows';
		$column2 = '';
		$replace = '';
		
		// 簡易検索用リンクを設定
		$searchPath = PATH . '/search/' . $param . '/' . $y . '/' . $m . '/xx';
		
		////////////////////////////////////////////////////////////
		
		// 有無フィルターの時（リンク元）
		if ($opt === 'exists') {
			$replace = 'replaceOptExists';
		}
		
		// ドメインフィルターの時（リンク元）
		elseif ($opt === 'domain') {
			[$replace, $column2, $from, $where, $opt] = Param::domain($db);
		}
		
		// 検索フィルターの時（リンク元）
		elseif ($opt === 'search') {
			[$replace, $column2, $from, $where, $opt] = Param::search($db);
		}
		
		// ページ、リンク先、リンク元の時
		elseif ($param === 'page' || $param === 'click' || $param === 'referrer') {
			[$replace, $column2, $from, $where, $param, $opt, $slash] = Param::page($db, $param, $opt);
		}
		
		// 統合フィルターの時（OS、UA、端末機種）
		elseif ($opt === 'group') {
			[$replace, $where] = Param::group($act, $param, $where, $y, $m);
		}
		
		// 検索キーワードの時
		elseif ($param === 'search_word' || $param === 'search_words') {
			$replace = 'replaceSearchWord';
		}
		
		// 訪問回数の時
		elseif ($param === 'visit_count') {
			$replace = ($opt === 'repeat') ? 'replaceRepeat' : 'replaceVisit';
		}
		
		// OSの時
		elseif ($param === 'os') {
			[$replace, $where, $opt] = Param::os($opt, $pickup);
		}
		
		// UAの時
		elseif ($param === 'ua') {
			[$replace, $param, $where, $opt, $uaType, $pickup] = Param::ua($opt, $pickup);
		}
		
		// ホストドメインの時
		elseif ($param === 'host_domain') {
			[$replace, $where, $opt] = Param::hostDomain($opt);
		}
		
		// 端末機種の時
		elseif ($param === 'device') {
			
			// iniファイルを読み込み
			$replace = Param::device();
			
			// オプションを設定
			if (!$opt) {
				$opt = 'model';
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 表示件数を設定
		$paramLimit = 9;
		
		// limit句を設定
		if (!$opt) {
			$limit = "limit $paramLimit";
		}
		
		////////////////////////////////////////////////////////////
		
		// DBに追加接続
		$db->attach($atDB);
		
		// クエリーを作成
		$query3 = "select $column1 from $paramTable $from where $where;";
		
		// クエリーを実行
		$rec3 = $db->queryFetch($query3);
		
		// 総カウント数を取得
		$totalCount = $rec3['sum'];
		
		// 総件数を取得
		$allRows = $rec3['rows'];
		
		// クエリーを作成
		$query4 = "select value, count $column2 from $paramTable $from where $where order by count desc $limit;";
		
		// クエリーを実行
		$results4 = $db->query($query4);
		
		// フィルター設定が存在する時
		if ($opt) {
			
			// フィルタリングしたデータを取得
			[$optCounts, $totalCount, $allRows] = Param::optFilter($db, $param, $opt, $results4, $totalCount, $paramTable);
			
			// パスに追記
			$opt2 = '/';
			
		}
		
		// 携帯フィルターの時はフィルターに追記
		elseif ($uaType) {
			$opt2 = $uaType . '/';
		}
		
		////////////////////////////////////////////////////////////
		
		// 変数を初期化
		$pers = [];
		$items = [];
		
		// 最大表示件数を設定
		$limitCount = ($allRows > $paramLimit) ? $paramLimit : $allRows;
		
		// その他のパーセンテージを初期化
		$etcPer = 100;
		
		// ヘッダーテンプレートを追記
		$table = $head;
		
		////////////////////////////////////////////////////////////
		
		// レコードデータ表示ループ
		for ($i = 1; ; ++$i) {
			
			// フィルターが存在しない時
			if (!$opt) {
				
				// データを取得
				$vars = $db->fetch($results4);
				
				// データが無い時はループ終了
				if (!$vars) {
					break;
				}
				
			}
			
			// フィルターが存在する時
			else {
				
				// データが無い時はループ終了
				if ($i > $limitCount) {
					break;
				}
				
				// 配列のキーを取得
				$key = key($optCounts);
				
				// 項目名を取得
				$vars['value'] = $key;
				
				// カウント数を取得
				$vars['count'] = $optCounts[$key];
				
				// 配列のポインタを進める
				next($optCounts);
				
			}
			
			// 項目名を取得
			$value = $vars['value'];
			
			// 表示文字列を置換
			if ($replace) {
				$vars['value'] = Param::$replace($vars, $param);
			}
			
			// 色分け用class属性を設定
			$vars['tr'] = 'b' . $i;
			
			// パーセンテージを算出
			$vars['per'] = number_format($vars['count'] / $totalCount * 100, 1);
			
			// 0.1未満の時は「0.1」を設定
			if ($vars['per'] === 0) {
				$vars['per'] = 0.1;
			}
			
			// 簡易検索リンクを設定
			$vars['count'] = '<a href="' . $searchPath . '/' . $value . $slash . $opt . $opt2 . '">' . $vars['per'] . '%</a>';
			
			// 順位を設定
			$vars['rank'] = '<span class="c' . $i . '">■</span>';
			
			// その他からパーセンテージを減少
			$etcPer -= $vars['per'];
			
			// グラフ内表示用に整形
			$graphValue = strip_tags($vars['value']);
			$graphValue = preg_replace('/\(.+/', '', $graphValue);
			
			// 配列に格納
			$pers[] = $vars['per'];
			$items[] = "'" . $graphValue . "'";
			
			// メインテンプレートを出力
			$table .= $tmpl->res($body, $vars);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// その他を追加
		if ($etcPer >= 0.1) {
			
			// 配列に格納
			$pers[] = $etcPer;
			$items[] = "'その他'";
			
			// テンプレート変数を設定
			$etcVars['tr'] = 'b10';
			$etcVars['rank'] = '<span class="c10">■</span>';
			$etcVars['value'] = 'その他';
			$etcVars['count'] = number_format($etcPer, 1) . '%';
			
			// メインテンプレートを出力
			$table .= $tmpl->res($body, $etcVars);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// フッターテンプレートを追記
		$table .= $foot;
		
		// テンプレート変数に設定
		$footVars['pers'] = implode(',', $pers);
		$footVars['items'] = implode(',', $items);
		
		// テンプレートファイルを読み込み
		$tmpl_d = $tmpl->read('pie_chart.htm', false);
		
		// メインテンプレートを出力
		$tmpl->show($tmpl_d, $footVars);
		
		// 項目テーブルを出力
		echo $table;
		
	}
	
}

