<?php

class Preference {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, false, 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  バージョン情報を表示
	//---------------------------------------------------------
	
	public static function execute() {
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		////////////////////////////////////////////////////////////
		
		// config.iniのパスを設定
		$config = WORK_DIR . '/config.ini';
		
		// リクエストメソッドがPOSTの時
		if ($_SERVER['REQUEST_METHOD'] === 'POST') {
			
			// アップデートの時はデータ更新
			if (isset($GLOBALS['args']['submit'])) {
				self::update();
				self::save($config);
			}
			
			// ログアウトの時はCookieを削除
			else {
				self::logout();
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// テンプレートファイルを読み込み
		[$body, $error, $admin] = $tmpl->read('preference.htm');
		
		// ヘッダークラスを読み込み
		include(WORK_DIR . '/filters/Header.php');
		
		// ヘッダーを出力
		Header::execute();
		
		// 一般ユーザーの時は終了
		if (!$GLOBALS['isAdmin']) {
			
			// URLパスを設定
			$vars['path'] = PATH;
			
			// エラーテンプレートを出力
			$tmpl->show($admin, $vars);
			
			// 以下の処理を省略
			return;
			
		}
		
		// config.iniに書き込み出来ない時は
		elseif (!is_writable($config)) {
			
			// config.iniのパスを設定
			$vars['config'] = $config;
			
			// エラーテンプレートを出力
			$tmpl->show($error, $vars);
			
			// 以下の処理を省略
			return;
			
		}
		
		////////////////////////////////////////////////////////////
		// テンプレート変数を設定（text or number）
		
		// URLパスを設定
		$vars['path'] = PATH;
		
		// ログイン設定
		$vars['username'] = $GLOBALS['config']['login']['username'];
		$vars['password'] = $GLOBALS['config']['login']['password'];
		
		// データ取得時の設定
		$vars['visitBlankTime'] = $GLOBALS['config']['content']['visitBlankTime'];
		$vars['visitSaveLimit'] = $GLOBALS['config']['content']['visitSaveLimit'];
		$vars['routeSaveLimit'] = $GLOBALS['config']['content']['routeSaveLimit'];
		$vars['cutInternalUrl'] = $GLOBALS['config']['content']['cutInternalUrl'];
		$vars['cutExternalUrl'] = $GLOBALS['config']['content']['cutExternalUrl'];
		$vars['allowDomain'] = $GLOBALS['config']['content']['allowDomain'];
		
		// データ表示時の設定
		$vars['visitViewLimit'] = $GLOBALS['config']['content']['visitViewLimit'];
		$vars['paramViewLimit'] = $GLOBALS['config']['content']['paramViewLimit'];
		$vars['pvLineColor'] = $GLOBALS['config']['content']['pvLineColor'];
		$vars['uuLineColor'] = $GLOBALS['config']['content']['uuLineColor'];
		$vars['indexViewLimit'] = $GLOBALS['config']['content']['indexViewLimit'];
		$vars['indexParams'] = $GLOBALS['config']['content']['indexParams'];
		
		// 全体の設定
		$vars['gmtDiff'] = $GLOBALS['config']['framework']['gmtDiff'];
		$vars['timeZone'] = $GLOBALS['config']['framework']['timeZone'];
		
		////////////////////////////////////////////////////////////
		// テンプレート変数を設定（radio 2択）
		
		// キー配列を設定
		$selectKeys1 = [
			'allowCountAdmin',
			'allowCountAnotherUA',
			'allowAddSlash',
			'allowCutArgs',
			'allowDarkMode',
			'allowLineChart',
			'displayErrors',
			'logErrors'
		];
		
		// 配列を読み込むループ
		foreach ($selectKeys1 as $key) {
			
			// 設定から値を取得
			if (isset($GLOBALS['config']['content'][$key])) {
				$val = $GLOBALS['config']['content'][$key];
			} elseif (isset($GLOBALS['config']['framework'][$key])) {
				$val = $GLOBALS['config']['framework'][$key];
			} else {
				$val = 0;
			}
			
			// 変数を初期化
			$vars[$key . '0'] = '';
			$vars[$key . '1'] = '';
			
			// selectedを設定
			$vars[$key . $val] = ' checked';
			
		}
		
		////////////////////////////////////////////////////////////
		// テンプレート変数を設定（radio 6択）
		
		// キー配列を設定
		$selectKeys2 = [
			'tdBarColor',
			'ydBarColor'
		];
		
		// 配列を読み込むループ
		foreach ($selectKeys2 as $key) {
			
			// 設定から値を取得
			$val = $GLOBALS['config']['content'][$key];
			
			// 変数を初期化
			$vars[$key . 'r'] = '';
			$vars[$key . 'g'] = '';
			$vars[$key . 'b'] = '';
			$vars[$key . 'p'] = '';
			$vars[$key . 'k'] = '';
			$vars[$key . 's'] = '';
			
			// selectedを設定
			$vars[$key . $val] = ' checked';
			
		}
		
		////////////////////////////////////////////////////////////
		// テンプレート変数を設定（サイト設定）
		
		// 10回ループ
		for ($i = 1; $i < 11; ++$i) {
			
			// valueを設定
			$vars['site' . $i] = (isset($GLOBALS['config']['site'][$i])) ? $GLOBALS['config']['site'][$i] : '';
			
		}
		
		////////////////////////////////////////////////////////////
		// テンプレート変数を設定（一般ユーザーアカウント設定）
		
		// 5回ループ
		for ($j = 1; $j < 6; ++$j) {
			
			// 配列のキーを設定
			$key1 = 'username' . $j;
			$key2 = 'password' . $j;
			
			// valueを設定
			$vars[$key1] = (isset($GLOBALS['config']['account'][$key1])) ? $GLOBALS['config']['account'][$key1] : '';
			$vars[$key2] = (isset($GLOBALS['config']['account'][$key2])) ? $GLOBALS['config']['account'][$key2] : '';
			
		}
		
		////////////////////////////////////////////////////////////
		
		// メインテンプレートを出力
		$tmpl->show($body, $vars);
		
	}
	
	//---------------------------------------------------------
	//  POSTデータを$GLOBALSに反映
	//---------------------------------------------------------
	
	private static function update() {
		
		////////////////////////////////////////////////////////////
		// POSTデータを変数に設定（text or number）
		
		// ログイン設定
		$GLOBALS['config']['login']['username'] = $GLOBALS['args']['username'];
		$GLOBALS['config']['login']['password'] = $GLOBALS['args']['password'];
		
		// データ取得時の設定
		$GLOBALS['config']['content']['visitBlankTime'] = (int) $GLOBALS['args']['visitBlankTime'];
		$GLOBALS['config']['content']['visitSaveLimit'] = (int) $GLOBALS['args']['visitSaveLimit'];
		$GLOBALS['config']['content']['routeSaveLimit'] = (int) $GLOBALS['args']['routeSaveLimit'];
		$GLOBALS['config']['content']['cutInternalUrl'] = $GLOBALS['args']['cutInternalUrl'];
		$GLOBALS['config']['content']['cutExternalUrl'] = $GLOBALS['args']['cutExternalUrl'];
		$GLOBALS['config']['content']['allowDomain'] = $GLOBALS['args']['allowDomain'];
		
		// データ表示時の設定
		$GLOBALS['config']['content']['visitViewLimit'] = (int) $GLOBALS['args']['visitViewLimit'];
		$GLOBALS['config']['content']['paramViewLimit'] = (int) $GLOBALS['args']['paramViewLimit'];
		$GLOBALS['config']['content']['pvLineColor'] = $GLOBALS['args']['pvLineColor'];
		$GLOBALS['config']['content']['uuLineColor'] = $GLOBALS['args']['uuLineColor'];
		$GLOBALS['config']['content']['indexViewLimit'] = (int) $GLOBALS['args']['indexViewLimit'];
		$GLOBALS['config']['content']['indexParams'] = $GLOBALS['args']['indexParams'];
		
		// 全体の設定
		$GLOBALS['config']['framework']['gmtDiff'] = (int) $GLOBALS['args']['gmtDiff'];
		$GLOBALS['config']['framework']['timeZone'] = $GLOBALS['args']['timeZone'];
		
		////////////////////////////////////////////////////////////
		// POSTデータを変数に設定（radio）
		
		// データ取得時の設定
		$GLOBALS['config']['content']['allowCountAdmin'] = (int) $GLOBALS['args']['allowCountAdmin'];
		$GLOBALS['config']['content']['allowCountAnotherUA'] = (int) $GLOBALS['args']['allowCountAnotherUA'];
		$GLOBALS['config']['content']['allowAddSlash'] = (int) $GLOBALS['args']['allowAddSlash'];
		$GLOBALS['config']['content']['allowCutArgs'] = (int) $GLOBALS['args']['allowCutArgs'];
		
		// データ表示時の設定
		$GLOBALS['config']['content']['allowDarkMode'] = (int) $GLOBALS['args']['allowDarkMode'];
		$GLOBALS['config']['content']['allowLineChart'] = (int) $GLOBALS['args']['allowLineChart'];
		$GLOBALS['config']['content']['tdBarColor'] = $GLOBALS['args']['tdBarColor'];
		$GLOBALS['config']['content']['ydBarColor'] = $GLOBALS['args']['ydBarColor'];
		
		// 全体の設定
		$GLOBALS['config']['framework']['displayErrors'] = (int) $GLOBALS['args']['displayErrors'];
		$GLOBALS['config']['framework']['logErrors'] = (int) $GLOBALS['args']['logErrors'];
		
		////////////////////////////////////////////////////////////
		// 無効な値が入っていないかチェックして補正（int）
		
		// キー配列を設定
		$intKeys = [
			'visitBlankTime',
			'visitSaveLimit',
			'routeSaveLimit',
			'visitViewLimit',
			'paramViewLimit',
			'indexViewLimit'
		];
		
		// 配列を読み込むループ
		foreach ($intKeys as $key) {
			
			// 1未満の時は1に補正
			if ($GLOBALS['config']['content'][$key] < 1) {
				$GLOBALS['config']['content'][$key] = 1;
			}
			
		}
		
		////////////////////////////////////////////////////////////
		// 無効な値が入っていないかチェックして補正（RGB）
		
		// キー配列を設定
		$colorKeys = [
			'pvLineColor',
			'uuLineColor'
		];
		
		// 配列を読み込むループ
		foreach ($colorKeys as $key) {
			
			// 「,」で値を分割
			$rgb = explode(',', $GLOBALS['config']['content'][$key]);
			
			// 3以外の要素数の時
			if (count($rgb) !== 3) {
				
				// 初期値を設定
				$GLOBALS['config']['content'][$key] = ($key === 'pvLineColor') ? '165, 133, 229' : '136, 136, 204';
				
				// 次へ
				continue;
				
			}
			
			// 配列を読み込むループ
			foreach ($rgb as $i => $val) {
				
				// intにキャスト
				$rgb[$i] = (int) $val;
				
				// 0未満の時は0に補正
				if ($rgb[$i] < 0) {
					$rgb[$i] = 0;
				}
				
				// 256以上の時は255に補正
				elseif ($rgb[$i] > 255) {
					$rgb[$i] = 255;
				}
				
			}
			
			// 配列をつないで値を再設定
			$GLOBALS['config']['content'][$key] = $rgb[0] . ', ' . $rgb[1] . ', ' . $rgb[2];
			
		}
		
		////////////////////////////////////////////////////////////
		// サイト設定の配列を補正
		
		// 10回ループ
		for ($i = 1; $i < 11; ++$i) {
			
			// サイト名を取得
			$siteName = $GLOBALS['args']['site' . $i];
			
			// 空欄の時
			if (empty($siteName)) {
				
				// 配列から除外
				if (isset($GLOBALS['config']['site'][$i])) {
					unset($GLOBALS['config']['site'][$i]);
				}
				
			}
			
			// 空欄でない時は値を設定
			else {
				$GLOBALS['config']['site'][$i] = $siteName;
			}
			
		}
		
		////////////////////////////////////////////////////////////
		// 一般ユーザーアカウント設定を補正
		
		// アカウント設定が存在するかフラグ
		$existsAccount = false;
		
		// 5回ループ
		for ($j = 1; $j < 6; ++$j) {
			
			// 配列のキーを設定
			$key1 = 'username' . $j;
			$key2 = 'password' . $j;
			
			// ユーザー名、パスワードのどちらかが空の時
			if (empty($GLOBALS['args'][$key1]) || empty($GLOBALS['args'][$key2])) {
				
				// 配列から除外（ユーザー名）
				if (isset($GLOBALS['config']['account'][$key1])) {
					unset($GLOBALS['config']['account'][$key1]);
				}
				
				// 配列から除外（パスワード）
				if (isset($GLOBALS['config']['account'][$key2])) {
					unset($GLOBALS['config']['account'][$key2]);
				}
				
			}
			
			// ユーザー名、パスワード両方ある時
			else {
				$GLOBALS['config']['account'][$key1] = $GLOBALS['args'][$key1];
				$GLOBALS['config']['account'][$key2] = $GLOBALS['args'][$key2];
				$existsAccount = true;
			}
			
		}
		
		// 1つもデータがない時はセクションを配列から除外
		if (!$existsAccount && isset($GLOBALS['config']['account'])) {
			unset($GLOBALS['config']['account']);
		}
		
		////////////////////////////////////////////////////////////
		// 管理者Cookieを再発行
		
		// ユーザーIDを取得
		$username = md5($GLOBALS['args']['username']);
		
		// パスワードを取得
		$password = md5($GLOBALS['args']['password']);
		
		// Cookieの有効期限を設定
		$time = $_SERVER['REQUEST_TIME'] + (60 * 60 * 24 * 365);
		
		// 現在のホストドメインを取得
		$host = $_SERVER['SERVER_NAME'];
		
		// Cookieを発行
		setcookie('lunalysLogin', "username=$username&password=$password", $time, '/', $host);
		
	}
	
	//---------------------------------------------------------
	//  変更データをconfig.iniに保存
	//---------------------------------------------------------
	
	private static function save($config) {
		
		// 出力用データ
		$data = '';
		
		// 配列を読み込むループ（セクション）
		foreach ($GLOBALS['config'] as $section => $params) {
			
			// セクションを追記
			$data .= "\n" . '[' . $section . ']' . "\n";
			
			// 配列を読み込むループ（パラメータ）
			foreach ($params as $key => $val) {
				
				// パラメータを追記
				$data .= $key . ' = ';
				$data .= (is_numeric($val)) ? $val : '"' . $val . '"';
				$data .= "\n";
				
			}
			
		}
		
		// ファイルクラスを取得
		$file = Controller::load('file');
		
		// iniファイルに書き込みし成否を返す
		return $file->write($config, $data);
		
	}
	
	//---------------------------------------------------------
	//  Cookieを削除してログアウト
	//---------------------------------------------------------
	
	private static function logout() {
		
		// ユーザーIDを取得
		$username = '';
		
		// パスワードを取得
		$password = '';
		
		// Cookieの有効期限を設定
		$time = 0;
		
		// 現在のホストドメインを取得
		$host = $_SERVER['SERVER_NAME'];
		
		// Cookieを発行
		setcookie('lunalysLogin', "username=$username&password=$password", $time, '/', $host);
		
		// ログインフォームを表示
		Login::showForm();
		
		// 以後の処理は行わず終了
		exit;
		
	}
	
}

