<?php

class Route {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  アクセスルート表示（Ajaxでは無い時）
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		////////////////////////////////////////////////////////////
		
		// ジャンプパスを定義
		define('JUMP_PATH', PATH . '/jump/');
		
		// ヘッダー、メイン、フッター テンプレートを取得
		[$head, $body, $foot] = $tmpl->read('route.htm');
		
		// ページ番号テーブル名を定義
		$pageTable = $GLOBALS['config']['table']['page'];
		
		// ルートを取得
		$route = $GLOBALS['args'][1];
		
		// 接続時間を取得
		$stayTime = (isset($GLOBALS['args'][2])) ? $GLOBALS['args'][2] : '';
		
		// クリックルートを取得
		$clickRoute = (isset($GLOBALS['args'][3])) ? $GLOBALS['args'][3] : '';
		
		////////////////////////////////////////////////////////////
		
		// アクセスルートを表示
		self::view($db, $tmpl, $route, $stayTime, $pageTable, $head, $body, $foot);
		
		// クリックルートが存在しない時は終了
		if (!$clickRoute) {
			return;
		}
		
		// 変数を初期化
		$times = [];
		
		// テーブル名を取得
		$clickTable = $GLOBALS['config']['table']['click'];
		
		// 滞在時間
		$stayTime = '';
		
		// <hr>を出力
		echo "\n\t\t\t\t" , '<hr>' , "\n";
		
		// クリックルートを表示
		self::view($db, $tmpl, $clickRoute, $stayTime, $clickTable, $head, $body, $foot);
		
	}
	
	//---------------------------------------------------------
	//  ルート表示
	//---------------------------------------------------------
	
	private static function view($db, $tmpl, $route, $stayTime, $pageTable, $head, $body, $foot) {
		
		// 変数を初期化
		$vars = [];
		$times = [];
		$pages = [];
		
		// ルートを配列に分割
		$ids = explode('-', $route);
		
		// カウントを初期化
		$count = count($ids);
		
		// 滞在時間が存在する時 
		if ($stayTime) {
			$times = explode('-', $stayTime . '-');
		}
		
		// アクセスルートをp_noに分割
		$where = 'where id = ' . implode(' or id = ', $ids);
		
		// クエリーを作成
		$query = "select * from $pageTable $where;";
		
		// クエリーを実行
		$results = $db->query($query);
		
		////////////////////////////////////////////////////////////
		
		// データ取得ループ
		while ($rec = $db->fetch($results)) {
			
			// ページIDを取得
			$id = $rec['id'];
			
			// URLを配列に格納
			$pages[$id]['url'] = $rec['url'];
			
			// タイトルを配列に格納
			$pages[$id]['title'] = $rec['title'];
			
		}
		
		////////////////////////////////////////////////////////////
		
		// ヘッダーテンプレートを出力
		$tmpl->show($head);
		
		// ページタイトル表示ループ
		for ($i = 0; $i < $count; ++$i) {
			
			// ページ番号を取得
			$id = $ids[$i];
			
			// タイトルが存在する時
			if (isset($pages[$id]['url'])) {
				
				// 滞在時間を取得
				$time = (isset($times[$i])) ? ' &nbsp;' . $times[$i] : '';
				
				// テンプレート変数にタイトルを設定
				$vars['title'] = '<a href="' . JUMP_PATH . $pages[$id]['url'] . '" class="out">';
				$vars['title'] .= $pages[$id]['title'] . '</a>' . $time;
				
			}
			
			// タイトルが存在しない時
			else {
				$vars['title'] = 'undefined';
			}
			
			// メインテンプレートを出力
			$tmpl->show($body, $vars);
			
		}
		
		// フッターテンプレートを出力
		$tmpl->show($foot);
		
	}
	
}

