<?php

class Visit {
	
	//---------------------------------------------------------
	//  コントロール設定
	//---------------------------------------------------------
	
	public static function control() {
		
		// コントロール設定を返す
		return [true, false, 'Header', 'Footer'];
		
	}
	
	//---------------------------------------------------------
	//  詳細ログ表示
	//---------------------------------------------------------
	
	public static function execute() {
		
		// DBクラスを取得
		$db = $GLOBALS['object']['db'];
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// DBのサフィックスを取得
		$suffix = $GLOBALS['suffix'];
		
		////////////////////////////////////////////////////////////
		
		// オプションを取得
		$opt = $GLOBALS['args']['opt'];
		
		// 年月日を取得
		$y = $GLOBALS['args']['y'];
		$m = $GLOBALS['args']['m'];
		$d = $GLOBALS['args']['d'];
		
		// 最大表示件数を取得
		$visitViewLimit = $GLOBALS['config']['content']['visitViewLimit'];
		
		////////////////////////////////////////////////////////////
		
		// DB名を設定
		$atDB = 'access_' . $y . '_' . $m . $suffix;
		
		// DBが存在しない時は終了
		if (!$db->exists($atDB)) {
			return;
		}
		
		// DBに接続
		$db->attach($atDB);
		
		// テーブル名を取得
		$visitTable = $GLOBALS['config']['table']['visit'];
		
		// 対象日付を設定
		$date = $y . '-' . $m . '-' . $d;
		
		// 変数を初期化
		$where = '';
		$limit = '';
		$param = '';
		$begin = 0;
		
		////////////////////////////////////////////////////////////
		
		// 引数の数を取得
		$argsCount = count($GLOBALS['args']);
		
		// 引数が存在しない時
		if ($argsCount === 2) {
			$limit = "limit $visitViewLimit";
		}
		
		// 引数が存在する時
		else {
			[$opt, $begin, $where, $limit] = self::filter($opt, $visitViewLimit);
		}
		
		// ロボットフィルターではない時
		if ($opt !== 'robot') {
			$where .= " and ua_type != 'Robot'";
		}
		
		////////////////////////////////////////////////////////////
		
		// メイン、ナビリンク、フッター テンプレートを取得
		[$body, $navLink, $foot] = $tmpl->read('visit.htm');
		
		// クエリーを作成
		$selectVisitQuery = "select * from $visitTable where date = '$date' $where order by last_time desc $limit;";
		
		// クエリーを実行
		$results = $db->query($selectVisitQuery);
		
		// クエリーを作成
		$selectCountQuery = "select count(ip_address) as count from $visitTable where date = '$date' $where;";
		
		// クエリーを実行
		$count = $db->queryFetch($selectCountQuery, 'count');
		
		// 開始カウント数を設定
		$seq = $count - $begin;
		
		// レスポンスを出力
		self::view($db, $tmpl, $body, $results, $seq, $opt);
		
		////////////////////////////////////////////////////////////
		
		// URLパスを設定
		$footVars['path'] = PATH;
		
		// フッターテンプレートを出力
		$tmpl->show($foot, $footVars);
		
		// 表示制限が存在する時
		if ($limit and $count > $visitViewLimit) {
			
			// リンクを設定
			$link = 'href="' . PATH . '/visit/' . $y . '/' . $m . '/' . $d . '/';
			
			// ナビリンクデータを取得
			$navVars = self::naviLink($link, $count, $begin, $visitViewLimit);
			
			// ナビリンクを出力
			$GLOBALS['args']['contentAll'] = $tmpl->res($navLink, $navVars);
			
		}
		
	}
	
	//---------------------------------------------------------
	//  メイン出力
	//---------------------------------------------------------
	
	public static function view($db, $tmpl, $body, $results, $seq, $opt) {
		
		// グローバル変数
		global $osTitles;
		global $refTitles;
		global $pageTitles;
		global $deviceTitles;
		
		// 定数が定義されていない時
		if (!defined('JUMP_PATH')) {
			
			// ジャンプパスを設定
			define('JUMP_PATH', PATH . '/jump/');
			
			// リファラタイトルを取得
			$refTitles = self::refTitles($db);
			
			// ページタイトルを取得
			$pageTitles = self::pageTitles($db);
			
			// 端末機種リストを取得
			$deviceTitles = parse_ini_file(WORK_DIR . '/templates/ini/device.ini');
			
			// OSリストを取得
			$osTitles = parse_ini_file(WORK_DIR . '/templates/ini/os.ini');
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 詳細ログを表示するループ
		while ($rec = $db->fetch($results)) {
			
			// カウント数を設定
			$vars['seq'] = $seq;
			
			////////////////////////////////////////////////////////////
			
			// リンク元が存在する時
			if ($rec['referrer']) {
				
				// リンク元を取得
				$referrer = $rec['referrer'];
				
				// 検索語を取得
				$searchWords = $rec['search_words'];
				
				// リンク元データが存在しない時は「Undefined」を表示
				if (!isset($refTitles[$referrer])) {
					$vars['referrer'] = 'Undefined';
				}
				
				// リンク元データが存在する時
				else {
					
					// リンク元データを取得
					$refUrl = $refTitles[$referrer]['url'];
					$refTitle = $refTitles[$referrer]['title'];
					
					// 検索語が存在する時
					if ($searchWords) {
						
						// 検索語を取得
						$refTitle .= ' / ' . $searchWords;
						
						// 「"」を「'」に変換
						$refUrl .= '::' . str_replace('"', "'", $searchWords);
						
					}
					
					// リンクを整形
					$vars['referrer'] = '<a href="' . JUMP_PATH . $refUrl . '" class="out">' . $refTitle . '</a>';
					
				}
				
			}
			
			// リンク元が存在しない時
			else {
				$vars['referrer'] = '';
			}
			
			////////////////////////////////////////////////////////////
			
			// ルートを取得
			$route = $rec['route'];
			
			// クリックルートを取得
			$clickRoute = $rec['click_route'];
			
			// ページルートを-で分割
			$pageIds = explode('-', $route);
			
			// PV数を算出
			$pv = count($pageIds);
			
			// 初回ページのNoを取得
			$pageId = $pageIds[0];
			
			// 初回ページのタイトルを取得
			$pageTitle = (isset($pageTitles[$pageId])) ? $pageTitles[$pageId] : 'Undefined';
			
			// ページルートが複数の時はPV数を追記
			if ($pv > 1) {
				$pageTitle .= ' : ' . $pv;
			}
			
			// クリックルートが存在する時は追記
			if ($clickRoute) {
				$pageTitle .= ' + ' . count(explode('-', $clickRoute));
			}
			
			// 滞在時間を取得
			$stayTime = (empty($rec['stay_time'])) ? '-' : $rec['stay_time'];
			
			// リンクを設定
			$vars['route'] = '<a href="' . PATH . '/route/' . $route . '/' . $stayTime . '/' . $clickRoute . '/" class="pop">' . $pageTitle . '</a>';
			
			////////////////////////////////////////////////////////////
			
			// ローカル変数に値を保持
			$os = $rec['os'];
			$ua = $rec['ua'];
			$uaType = $rec['ua_type'];
			$device = $rec['device'];
			
			////////////////////////////////////////////////////////////
			
			// ディスプレイサイズ補完
			if (!$rec['display_size']) {
				$vars['displaySize'] = '------------';
			} else {
				$vars['displaySize'] = $rec['display_size'];
			}
			
			////////////////////////////////////////////////////////////
			
			// 表示領域サイズ補完
			if (!$rec['client_size']) {
				$vars['clientSize'] = '------------';
			} else {
				$vars['clientSize'] = $rec['client_size'];
			}
			
			////////////////////////////////////////////////////////////
			
			// OS補完
			if (!$os) {
				$vars['os'] = '------------';
			} elseif ($uaType === 'PC' && isset($osTitles[$os])) {// OS X の時は愛称を補記
				$vars['os'] = $osTitles[$os];
			} else {
				$vars['os'] = $os;
			}
			
			////////////////////////////////////////////////////////////
			
			// UA補完
			if (!$ua) {
				$vars['ua'] = '------------';
			} elseif ($uaType === 'Robot') {// ロボットの時はUAの文字数を制限
				$vars['ua'] = self::replaceRobot($ua);
			} else {
				$vars['ua'] = $ua;
			}
			
			////////////////////////////////////////////////////////////
			
			// UA種別保管
			if ($device) {// 端末機種がある時はUA種別を上書き
				$vars['uaType'] = (isset($deviceTitles[$device])) ? $deviceTitles[$device] : $device;
			} else {
				$vars['uaType'] = $uaType;
			}
			
			////////////////////////////////////////////////////////////
			
			// UA-CH
			if (isset($rec['ch_ua'])) {
				$vars['chUA'] = $rec['ch_ua'];
			} else {
				$vars['chUA'] = '';
			}
			
			////////////////////////////////////////////////////////////
			
			// 通信キャリア
			$vars['carrier'] = $rec['carrier'];
			
			// 都道府県
			$vars['city'] = $rec['city'];
			
			// 日付
			$vars['date'] = $rec['date'];
			
			// UA全文
			$vars['uaFull'] = $rec['ua_full'];
			
			// 最終訪問時間
			$vars['lastTime'] = $rec['last_time'];
			
			// IPアドレス
			$vars['ipAddress'] = $rec['ip_address'];
			
			// リモートホスト
			$vars['remoteHost'] = $rec['remote_host'];
			
			// 訪問者ID
			$vars['visitorId'] = '<a href="' . PATH . '/search/visitor_id/' . $rec['visitor_id'] . '/">' . $rec['visitor_id'] . ' : ' . $rec['visit_count'] . '</a>';
			
			////////////////////////////////////////////////////////////
			
			// メインテンプレートを出力
			$tmpl->show($body, $vars);
			
			// カウント数を1減少
			--$seq;
			
		}
		
	}
	
	//---------------------------------------------------------
	//  リンク元タイトル取得
	//---------------------------------------------------------
	
	private static function refTitles($db) {
		
		// リンク元テーブル名を設定
		$refTable = $GLOBALS['config']['table']['referrer'];
		
		// クエリーを作成
		$query = "select * from $refTable;";
		
		// クエリーを実行
		$results = $db->query($query);
		
		// 変数を初期化
		$refTitles = [];
		
		// データレコード取得ループ
		while ($rec = $db->fetch($results)) {
			
			// IDを取得
			$id = $rec['id'];
			
			// URLとタイトルを配列に設定
			$refTitles[$id]['url'] = $rec['url'];
			$refTitles[$id]['title'] = $rec['title'];
			
		}
		
		// リンク元タイトルを返す
		return $refTitles;
		
	}
	
	//---------------------------------------------------------
	//  ページタイトル取得
	//---------------------------------------------------------
	
	private static function pageTitles($db) {
		
		// ページテーブル名を設定
		$pageTable = $GLOBALS['config']['table']['page'];
		
		// クエリーを作成
		$query = "select id, title from $pageTable;";
		
		// クエリーを実行
		$results = $db->query($query);
		
		// 変数を初期化
		$pageTitles = [];
		
		// 不明ページは Unknown を設定
		$pageTitles[0] = 'Unknown';
		
		// データレコード取得ループ
		while ($rec = $db->fetch($results)) {
			
			// IDを取得
			$id = $rec['id'];
			
			// タイトルを配列に設定
			$pageTitles[$id] = $rec['title'];
			
		}
		
		// ページタイトルを返す
		return $pageTitles;
		
	}
	
	//---------------------------------------------------------
	//  フッター出力
	//---------------------------------------------------------
	
	private static function naviLink($link, $allRows, $begin, $visitViewLimit) {
		
		// リミット数を変数に設定
		$vars['visitViewLimit'] = $visitViewLimit;
		
		// リンクを変数に設定
		$vars['allLink'] = $link . 'all/"';
		
		// 開始番号を取得
		$prevId = $begin - $visitViewLimit;
		
		// 開始番号が0未満の時
		if ($prevId < 0) {
			$prevId = 0;
		}
		
		// 次の開始番号を取得
		$nextId = $begin + $visitViewLimit;
		
		// 開始番号が総数以上の時
		if ($nextId > $allRows) {
			$nextId = $allRows;
		}
		
		// 前何件へのリンクを設定
		$vars['prevLink'] = ($begin !== 0) ? $link . $prevId . '/"' : 'class="no_link"';
		
		// 次何件へのリンクを設定
		$vars['nextLink'] = ($nextId !== $allRows) ? $link . $nextId . '/"' : 'class="no_link"';
		
		// リンクデータを返す
		return $vars;
		
	}
	
	//---------------------------------------------------------
	//  オプションフィルタリング
	//---------------------------------------------------------
	
	private static function filter($opt, $visitViewLimit) {
		
		// 変数を初期化
		$where = '';
		$limit = '';
		$begin = 0;
		
		// オプションが存在しない時
		if (!$opt) {
			$limit = "limit $visitViewLimit";
		}
		
		// リンク元フィルターの時
		elseif ($opt === 'referrer') {
			$where = "and referrer != ''";
		}
		
		// PCフィルターの時
		elseif ($opt === 'pc') {
			$where = "and ua_type = 'PC'";
		}
		
		// ロボットフィルターの時
		elseif ($opt === 'robot') {
			$where = "and ua_type = 'Robot'";
		}
		
		// 携帯フィルターの時
		elseif ($opt === 'mobile') {
			$where = "and ua_type = 'Mobile'";
		}
		
		// 全件表示の時
		elseif ($opt === 'all') {
			$opt = '';
		}
		
		// 通常表示の時
		else {
			
			// 変数を別名で保持
			$limit = $visitViewLimit;
			
			// 開始番号を取得
			$begin = (int) $opt;
			
			// 開始番号が0未満の時
			if ($begin < 0) {
				$begin = 0;
			}
			
			// Limit句を設定
			$limit = "limit $begin, $limit";
			
			// オプションを初期化
			$opt = '';
			
		}
		
		// 各種データを返す
		return [$opt, $begin, $where, $limit];
		
	}
	
	//---------------------------------------------------------
	//  リンクの設定
	//---------------------------------------------------------
	
	private static function replaceRobot($ua) {
		
		// Mozilla互換文字列を削除
		$ua = preg_replace('/(Mozilla\/\d\.\d )/', '', $ua);
		
		// UA文字列を抽出
		if (preg_match('/compatible; ([^\;]+)/', $ua, $m)) {
			$ua = $m[1];
		} elseif (preg_match('/\(([^\;+]+)( \d{1,2}\.\d)\)/', $ua, $m)) {
			$ua = $m[1] . $m[2];
		} elseif (preg_match('/\([^)]+\)([^(]+)/', $ua, $m)) {
			$ua = $m[1];
		} elseif (preg_match('/([^(]+)/A', $ua, $m)) {
			$ua = $m[1];
		} elseif (preg_match('/\(([^\;+]+)/A', $ua, $m)) {
			$ua = $m[1];
		}
		
		// バージョン表記を整形する
		$ua = preg_replace('/\/v{0,1}(\d)/', " $1", $ua);
		
		// 無関係の文字列があれば削除
		$ua = preg_replace('/ ?[\+\,\;\)].*?$| .+@.+|Gecko [\d]+ /', '', $ua);
		
		// 文字数を制限
		$ua = mb_strimwidth($ua, 0, 36, '...', 'UTF-8');
		
		// URLが含まれている時はリンクを張る
		if (preg_match("/(http:[^\);,&]*)/", $ua, $m)) {
			$ua = '<a href="' . JUMP_PATH . $m[1] . '" class="out">' . $ua . '</a>';
		}
		
		// UAを返す
		return $ua;
		
	}
	
}

