<?php

class Header {
	
	//---------------------------------------------------------
	//  初期処理
	//---------------------------------------------------------
	
	public static function execute() {
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// アクション名を取得
		$act = $GLOBALS['args']['act'];
		
		// メニューiniファイルを解析
		$GLOBALS['menu'] = parse_ini_file(WORK_DIR . '/templates/ini/menu.ini', true);
		
		// タイトルiniファイルを解析
		$GLOBALS['title'] = parse_ini_file(WORK_DIR . '/templates/ini/title.ini');
		
		////////////////////////////////////////////////////////////
		
		// URLパスを設定
		$vars['path'] = PATH;
		
		// URLパスディレクトリを設定
		$vars['pathDir'] = PATH_DIR;
		
		// CSSにアクション名を設定
		$vars['act'] = $act;
		
		// タイトルを取得
		$vars['title'] = $GLOBALS['title'][$act];
		
		// 日付リンク
		$vars['contentDate'] = '';
		
		// サイト切り替え
		$vars['select'] = '';
		
		// ダークモード
		$vars['link'] = '';
		
		////////////////////////////////////////////////////////////
		
		// 解析トップの時
		if ($act === 'index') {
			
			// 日付を取得
			$vars['contentDate'] = self::index();
			
			// CSSにgraphを設定
			$vars['act'] = 'graph';
			
		}
		
		// 詳細検索の時
		elseif ($act === 'search') {
			
			// CSSにvisitを設定
			$vars['act'] = 'visit';
			
		}
		
		// 日付及びナビリンクが必要な時
		elseif (preg_match('/param|pie_chart|transition|graph|page_data|search_form|shortcut|visit/', $act)) {
			
			// メソッド名を設定
			$method = ($act === 'pie_chart' || $act === 'transition') ? 'param' : Controller::toLowerCamel($act);
			
			// 可変メソッドを実行
			[$vars['title'], $vars['contentDate']] = self::$method($vars['title']);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 詳細検索 or ページ管理以外の時は日付リンクに<section>を追記
		if ($act !== 'search' && $act !== 'page_data') {
			$vars['contentDate'] .= "\n\t\t\t" . '<section id="content_main">';
		}
		
		////////////////////////////////////////////////////////////
		
		// ダークモードのCSS読み込みを追記
		if ($GLOBALS['config']['content']['allowDarkMode']) {
			$vars['link'] = "\n\t\t" . '<link rel="stylesheet" href="' . PATH_DIR . '/templates/css/dark.css">';
		}
		
		////////////////////////////////////////////////////////////
		
		// サイト番号をCookieから取得
		$site = (isset($_COOKIE['site'])) ? (int) $_COOKIE['site'] : 1;
		
		// 複数サイトで利用する時
		if (count($GLOBALS['config']['site']) >= 2) {
			
			// タブを設定
			$tab1 = "\n\t\t\t\t";
			$tab2 = $tab1 . "\t";
			
			// select要素開始
			$vars['select'] = $tab1 . '<select id="site_select" onchange="switchSite(this.options[this.selectedIndex].value);">';
			
			// iniデータを読み込むループ
			foreach ($GLOBALS['config']['site'] as $key => $val) {
				
				// selected属性を設定
				$selected = ($key === $site) ? ' selected' : '';
				
				// option要素を追記
				$vars['select'] .= $tab2 . '<option value="' . $key . '"' . $selected . '>' . $val . '</option>';
				
			}
			
			// select要素終了
			$vars['select'] .= $tab1 . '</select>';
			
		}
		
		// DBのサフィックスを設定（接尾辞）
		$GLOBALS['suffix'] = '_' . $site . '.sqlite';
		
		// メインDB名を設定
		$GLOBALS['mainDB'] = 'access_main' . $GLOBALS['suffix'];
		
		////////////////////////////////////////////////////////////
		
		// 旧式のファイルが存在する時（将来的に廃止予定）
		if (file_exists(DATA_DIR . '/access_main.db')) {
			
			// リネームクラスを読み込み
			include(WORK_DIR . '/modules/Rename.php');
			
			// リネームを実行
			Rename::execute(DATA_DIR);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// DBクラスを読み込み
		include(FW_DIR . '/modules/core/DB.php');
		
		// インスタンスを生成
		$GLOBALS['object']['db'] = new DB(DATA_DIR);
		
		// DBに接続
		$GLOBALS['object']['db']->connect($GLOBALS['mainDB']);
		
		////////////////////////////////////////////////////////////
		
		// テンプレートファイルを読み込み
		$header = $tmpl->read('header.htm', false);
		
		// ヘッダーテンプレートを出力
		$tmpl->show($header, $vars);
		
	}
	
	//---------------------------------------------------------
	//  解析トップの時
	//---------------------------------------------------------
	
	private static function index() {
		
		// 今日の年月日曜時を取得
		[$y, $m, $d, $hh] = Controller::dates();
		
		// intにキャスト
		$h = (int) $hh;
		
		// グローバル変数に保持
		$GLOBALS['args']['y'] = $y;
		$GLOBALS['args']['m'] = $m;
		$GLOBALS['args']['d'] = $d;
		$GLOBALS['args']['h'] = $h;
		
		// インデントタブを設定
		$tab1 = "\n\t\t\t";
		$tab2 = $tab1 . "\t";
		
		// 今日の日付を整形
		$contentDate = $tab1 . '<div id="content_date">' . $tab2 . '&lt; <time datetime="' . $y . '-' . $m . '-' . $d . '">' . $y . '/' . $m . '/' . $d . '</time> &gt;' . $tab1 . '</div>';
		
		// 今日の日付を返す
		return $contentDate;
		
	}
	
	//---------------------------------------------------------
	//  アクセス詳細の時
	//---------------------------------------------------------
	
	private static function visit($title) {
		
		// アクション名を設定
		$act = 'visit';
		
		// オプション設定を取得
		$opt = (isset($GLOBALS['args'][4])) ? $GLOBALS['args'][4] : '';
		
		// インデントタブを設定
		$tab = "\n\t\t\t";
		
		// <nav>を設定
		$contentDate = $tab . '<nav id="content_nav">';
		
		// 日付リンクを取得
		$contentDate .= self::createYMDLink($act);
		
		// フィルターリンクを取得
		$contentDate .= self::createFilterLink($act);
		
		// </nav>を設定
		$contentDate .= $tab . '</nav>';
		
		// オプション設定が存在する時はタイトルに追記
		if ($opt && $opt !== 'all' && !is_numeric($opt)) {
			$title .= '<span id="filter"> : ' . $GLOBALS['menu'][$act][$opt] . '</span>';
		}
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  日間パラメータ統計の時
	//---------------------------------------------------------
	
	private static function paramDaily($title) {
		
		// アクション名を設定
		$act = 'param_daily';
		
		// パラメータを取得
		$param = $GLOBALS['args'][1];
		
		// インデントタブを設定
		$tab = "\n\t\t\t";
		
		// <nav>を設定
		$contentDate = $tab . '<nav id="content_nav">';
		
		// 日付リンクを取得
		$contentDate .= self::createYMDLink($act);
		
		// フィルターリンクを取得
		$contentDate .= self::createFilterLink($act);
		
		// </nav>を設定
		$contentDate .= $tab . '</nav>';
		
		// タイトルに追記
		$title .= '<span id="filter"> : ' . $GLOBALS['title'][$param] . '</span>';
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  詳細ログ条件検索の時
	//---------------------------------------------------------
	
	private static function searchForm($title) {
		
		// アクション名を設定
		$act = 'search_form';
		
		// 日付リンクを取得
		$contentDate = self::createYMLink($act);
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  月間パラメータ統計の時
	//---------------------------------------------------------
	
	private static function param($date) {
		
		// アクション名を設定
		$act = $GLOBALS['args'][0];
		
		// 対象パラメータを取得
		$param = $GLOBALS['args'][1];
		
		// オプション設定を取得
		$opt = (isset($GLOBALS['args'][4])) ? $GLOBALS['args'][4] : '';
		
		// ピックアップ項目を取得
		$pickup = (isset($GLOBALS['args'][5])) ? $GLOBALS['args'][5] : '';
		
		// 全件表示フラグを取得
		$all = (isset($GLOBALS['args'][6])) ? $GLOBALS['args'][6] : '';
		
		// 変数に設定
		$GLOBALS['args']['all'] = $all;
		$GLOBALS['args']['opt'] = $opt;
		$GLOBALS['args']['pickup'] = $pickup;
		
		// タイトルを取得
		$title = $GLOBALS['title'][$param];
		
		// オプション設定が存在する時はタイトルに追記
		if ($opt && $opt !== 'none') {
			$title .= '<span id="filter"> : ';
			$title .= ($pickup && $pickup !== 'all') ? $pickup : $GLOBALS['menu'][$param][$opt];
			$title .= '</span>';
		}
		
		////////////////////////////////////////////////////////////
		
		// ピックアップの時はオプション設定に項目を追記
		if ($pickup) {
			$opt .= '/' . $pickup;
		}
		
		// インデントタブを設定
		$tab = "\n\t\t\t";
		
		// <nav>を設定
		$contentDate = $tab . '<nav id="content_nav">';
		
		// 日付リンクを取得
		$contentDate .= self::createYMLink($act, $opt);
		
		// フィルターリンクを取得
		$contentDate .= self::createFilterLink($act, $param);
		
		// </nav>を設定
		$contentDate .= $tab . '</nav>';
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  月間アクセス推移の時
	//---------------------------------------------------------
	
	private static function graph($date) {
		
		// アクション名を設定
		$act = 'graph';
		
		// 対象データを取得
		$graph = $GLOBALS['args'][1];
		
		// 日を取得
		$d = (isset($GLOBALS['args'][4])) ? $GLOBALS['args'][4] : '';
		
		// 今日の日時を取得
		if (!$d) {
			[$y, $m, $d] = Controller::dates();
		}
		
		// グローバル変数に設定
		$GLOBALS['args']['d'] = $d;
		$GLOBALS['args']['graph'] = $graph;
		
		// タイトルを取得
		$title = $GLOBALS['title'][$graph];
		
		// インデントタブを設定
		$tab = "\n\t\t\t";
		
		// <nav>を設定
		$contentDate = $tab . '<nav id="content_nav">';
		
		////////////////////////////////////////////////////////////
		
		// 月間グラフの時は日付リンク（年）を取得
		if ($graph == 'monthly') {
			$contentDate .= self::createYLink($act);
		}
		
		// 時間グラフの時は日付リンク（年月日）を取得
		elseif ($graph == 'timely') {
			$contentDate .= self::createYMDLink($act);
		}
		
		// 日間グラフの時は日付リンク（年月）を取得
		else {
			$contentDate .= self::createYMLink($act);
		}
		
		////////////////////////////////////////////////////////////
		
		// フィルターリンクを取得
		$contentDate .= self::createFilterLink($act);
		
		// </nav>を設定
		$contentDate .= $tab . '</nav>';
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  ページ番号管理
	//---------------------------------------------------------
	
	private static function pageData($title) {
		
		// オプションを取得
		$opt = (isset($GLOBALS['args'][1])) ? $GLOBALS['args'][1] : '';
		
		// リンク元、リンク先の時はタイトルに追記
		if ($opt === 'referrer' || $opt === 'click') {
			$title .= '<span id="filter"> : ' . $GLOBALS['menu']['page_data'][$opt] . '</span>';
		}
		
		// <nav>を設定
		$contentDate = '';
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  ショートカットの時
	//---------------------------------------------------------
	
	private static function shortcut($date) {
		
		// アクション名を設定
		$act = 'shortcut';
		
		// 表示種別（モード）を取得
		$type = $GLOBALS['args'][1];
		
		// パラメータ種別を取得
		$param = $GLOBALS['args'][2];
		
		// タイトルを取得
		$title = (preg_match('/param$|graph|pie_chart|transition/', $type)) ? $GLOBALS['title'][$param] : $GLOBALS['title'][$type];
		
		// 日付リンクを取得
		$contentDate = (preg_match('/visit|daily/', $type)) ? self::createYMLink($act) : self::createYLink($act);
		
		// テンプレート変数を返す
		return [$title, $contentDate];
		
	}
	
	//---------------------------------------------------------
	//  日付リンク生成（年月日）
	//---------------------------------------------------------
	
	private static function createYMDLink($act) {
		
		// 年引数のインデックスを設定
		$yIdx = 2;
		
		// オプション引数のインデックスを設定
		$optIdx = 5;
		
		// リンク用パスを設定
		$linkPath = $act;
		
		// 年月日を初期化
		$y = '';
		$m = '';
		$d = '';
		
		// 詳細ログの時
		if ($act === 'visit') {
			
			// インデックスを1減少
			--$yIdx;
			--$optIdx;
			
		}
		
		// グラフ、日間統計の時
		elseif ($act === 'graph' || $act === 'param_daily') {
			
			// リンク用パスに追記
			$linkPath .= '/' . $GLOBALS['args'][1];
			
		}
		
		// オプションを取得
		$opt = (isset($GLOBALS['args'][$optIdx])) ? $GLOBALS['args'][$optIdx] : '';
		
		// 引数が存在する時
		if (isset($GLOBALS['args'][$yIdx])) {
			
			// インデックスを1増加
			$mIdx = $yIdx + 1;
			$dIdx = $mIdx + 1;
			
			// 指定年月日を取得
			$y = $GLOBALS['args'][$yIdx];
			$m = $GLOBALS['args'][$mIdx];
			$d = (isset($GLOBALS['args'][$dIdx])) ? $GLOBALS['args'][$dIdx] : '01';
			
		}
		
		// 引数が存在しない時
		else {
			
			// 現在の年月日を取得
			[$y, $m, $d] = Controller::dates();
			
		}
		
		// 年月日をグローバル変数に設定
		$GLOBALS['args']['y'] = $y;
		$GLOBALS['args']['m'] = $m;
		$GLOBALS['args']['d'] = $d;
		
		// オプションをグローバル変数に設定
		$GLOBALS['args']['opt'] = $opt;
		
		////////////////////////////////////////////////////////////
		
		// 年月
		$ym = $y . '/' . $m;
		
		// 年月日
		$ymd = $ym . '/' . $d;
		
		// インデントタブ
		$tab1 = "\n\t\t\t\t";
		$tab2 = $tab1 . "\t";
		
		// 前日リンクを取得
		$prevLink = '<a href="' . PATH . '/' . $linkPath . '/' . self::createLinkPath('prev', $y, $m, $d, $opt) . '">&lt;</a>';
		
		// 次日リンクを取得
		$nextLink = '<a href="' . PATH . '/' . $linkPath . '/' . self::createLinkPath('next', $y, $m, $d, $opt) . '">&gt;</a>';
		
		// 日付ジャンプリンクを設定
		$jumpLink = '<a href="' . PATH . '/shortcut/' . $linkPath . '/' . $ym . '/" title="' . $ym . '">' . $ymd . '</a>';
		
		// 先月/次月へのリンクを整形
		$link = $tab2 . $prevLink . $tab2 . $jumpLink . $tab2 . $nextLink . $tab1;
		
		// 先月/次月へのリンクを付け足す
		$nav = $tab1 . '<div id="content_date">' . $link . '</div>';
		
		// メニューリンクを返す
		return $nav;
		
	}
	
	//---------------------------------------------------------
	//  日付リンク生成（年月）
	//---------------------------------------------------------
	
	public static function createYMLink($act, $opt = '') {
		
		// 変数を初期化
		$yIdx = 1;
		$nav = '';
		$y = '';
		$m = '';
		
		// リンク用パス
		$linkPath = $act;
		
		////////////////////////////////////////////////////////////
		
		// パラメータ統計 or グラフ or ショートカットの時
		if (preg_match('/param|pie_chart|transition|graph|shortcut/', $act)) {
			
			// インデックスを1増加
			$yIdx++;
			
			// リンク用パスに追記
			$linkPath .= '/' . $GLOBALS['args'][1];
			
		}
		
		// ショートカットの時
		if (isset($GLOBALS['args'][1]) && $GLOBALS['args'][1] === 'param_daily') {
			
			// インデックスを1増加
			$yIdx++;
			
			// リンク用パスに追記
			$linkPath .= '/' . $GLOBALS['args'][2];
			
		}
		
		// 年月の指定が存在する時
		if (isset($GLOBALS['args'][$yIdx])) {
			
			// 引数のインデックスを1増加
			$mIdx = $yIdx + 1;
			
			// 年を取得
			$y = $GLOBALS['args'][$yIdx];
			
			// 月を取得
			$m = $GLOBALS['args'][$mIdx];
			
		}
		
		// 年月の指定が存在しない時
		else {
			
			// 現在の年月を取得
			[$y, $m] = Controller::dates();
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 前月の年月を算出
		$ly = $y;
		$lm = $m - 1;
		
		// 前月が12月の時は前年を算出
		if ($lm === 0) {
			$lm = 12;
			--$ly;
		}
		
		// 月が2桁になるよう補正
		elseif ($lm < 10) {
			$lm = 0 . $lm;
		}
		
		// 次月の年月を算出
		$ny = $y;
		$nm = $m + 1;
		
		// 次月が1月の時は来年を算出
		if ($nm === 13) {
			$nm = '01';
			++$ny;
		}
		
		// 月が2桁になるよう補正
		elseif ($nm < 10) {
			$nm = 0 . $nm;
		}
		
		// グローバル変数にデータを設定
		$GLOBALS['args']['y'] = $y;
		$GLOBALS['args']['m'] = $m;
		$GLOBALS['args']['ly'] = $ly;
		$GLOBALS['args']['lm'] = $lm;
		
		// オプションが存在する時は/を追記
		if ($opt) {
			$opt .= '/';
		}
		
		////////////////////////////////////////////////////////////
		
		// 年月
		$ym = $y . '/' . $m;
		$lym = $ly . '/' . $lm;
		$nym = $ny . '/' . $nm;
		
		// インデントタブを設定
		$tab1 = "\n\t\t\t\t";
		$tab2 = $tab1 . "\t";
		
		// 月間パラメータ統計の時
		if (preg_match('/param|pie_chart|transition/A', $act)) {
			
			// 一時配列を初期化
			$links = [];
			
			// 配列にリンクを格納
			foreach ($GLOBALS['menu']['param'] as $key => $val) {
				$links[] = '<a id="mode_' . $key . '" href="' . PATH . '/' . $key . '/' . $GLOBALS['args'][1] . '/' . $ym . '/' . $opt . '">' . $val . '</a>';
			}
			
			// メニューを整形
			$nav = $tab1 . '<div id="content_mode">' . $tab2 . implode("$tab2", $links) . $tab1 . '</div>';
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 前月へのリンクを整形
		$prevLink = '<a href="' . PATH . '/' . $linkPath . '/' . $lym . '/' . $opt . '" title="' . $lym . '">&lt;</a>';
		
		// 次月へのリンクを整形
		$nextLink = '<a href="' . PATH . '/' . $linkPath . '/' . $nym . '/' . $opt . '" title="' . $nym . '">&gt;</a>';
		
		// ジャンプリンクを設定
		$jumpLink = (!preg_match('/^shortcut/', $act)) ? '<a href="' . PATH . '/shortcut/' . $linkPath . '/' . $ym . '/' . $opt . '" title="' . $y . '">' . $ym . '</a>' : $ym;
		
		// 先月/次月へのリンクを整形
		$link = $tab2 . $prevLink . $tab2 . $jumpLink . $tab2 . $nextLink . $tab1;
		
		// 先月/次月へのリンクを付け足す
		$nav .= $tab1 . '<div id="content_date">' . $link . '</div>';
		
		// メニューリンクを返す
		return $nav;
		
	}
	
	//---------------------------------------------------------
	//  日付リンク生成（年）
	//---------------------------------------------------------
	
	private static function createYLink($act) {
		
		// 変数を初期化
		$yIdx = 1;
		$y = '';
		$m = '';
		
		// リンク用パス
		$linkPath = $act;
		
		// アクション名を設定
		$acts = 'param|graph|pie_chart|transition';
		
		////////////////////////////////////////////////////////////
		
		// パラメータ統計 or グラフの時
		if (preg_match("/$acts|shortcut/", $act)) {
			
			// インデックスを1増加
			++$yIdx;
			
			// リンク用パスに追記
			$linkPath .= '/' . $GLOBALS['args'][1];
			
		}
		
		// ショートカットの時
		if (preg_match("/$acts/", $GLOBALS['args'][1])) {
			
			// インデックスを1増加
			++$yIdx;
			
			// リンク用パスに追記
			$linkPath .= '/' . $GLOBALS['args'][2];
			
		}
		
		// 年月の指定が存在する時
		if (isset($GLOBALS['args'][$yIdx])) {
			
			// 引数のインデックスを1増加
			$mIdx = $yIdx + 1;
			
			// 年を取得
			$y = $GLOBALS['args'][$yIdx];
			
			// 月を取得
			$m = $GLOBALS['args'][$mIdx];
			
		}
		
		// 年月の指定が存在しない時
		else {
			
			// 現在の年月を取得
			[$y, $m] = Controller::dates();
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 前年を算出
		$ly = $y - 1;
		
		// 翌年を算出
		$ny = $y + 1;
		
		// グローバル変数にデータを設定
		$GLOBALS['args']['y'] = $y;
		$GLOBALS['args']['m'] = $m;
		
		////////////////////////////////////////////////////////////
		
		// 年月
		$ym = $y . '/' . $m;
		$lym = $ly . '/' . $m;
		$nym = $ny . '/' . $m;
		
		// インデントタブを設定
		$tab1 = "\n\t\t\t\t";
		$tab2 = $tab1 . "\t";
		
		// 前月へのリンクを整形
		$prevLink = '<a href="' . PATH . '/' . $linkPath . '/' . $lym . '/" title="' . $ly . '">&lt;</a>';
		
		// 次月へのリンクを整形
		$nextLink = '<a href="' . PATH . '/' . $linkPath . '/' . $nym . '/" title="' . $ny . '">&gt;</a>';
		
		// ジャンプリンクを設定
		$jumpLink = (!preg_match('/^shortcut/', $act)) ? '<a href="' . PATH . '/shortcut/' . $linkPath . '/' . $ym . '/" title="' . $y . '">' . $y . '</a>' : $y;
		
		// 先月/次月へのリンクを整形
		$link = $tab2 . $prevLink . $tab2 . $jumpLink . $tab2 . $nextLink . $tab1;
		
		// 先月/次月へのリンクを付け足す
		$nav = $tab1 . '<div id="content_date">' . $link . '</div>';
		
		// メニューリンクを返す
		return $nav;
		
	}
	
	//---------------------------------------------------------
	//  フィルターリンク生成
	//---------------------------------------------------------
	
	public static function createFilterLink($act, $param = '') {
		
		// 空欄の時はアクション名を設定
		if (!$param) {
			$param = $act;
		}
		
		// メニュー配列を取得
		$menu = $GLOBALS['menu'][$param];
		
		// 変数を初期化
		$ymd = '';
		$opt = '';
		$end = '';
		
		// 年月日を取得
		$y = (isset($GLOBALS['args']['y'])) ? $GLOBALS['args']['y'] : '';
		$m = (isset($GLOBALS['args']['m'])) ? $GLOBALS['args']['m'] : '';
		$d = (isset($GLOBALS['args']['d'])) ? $GLOBALS['args']['d'] : '';
		
		// パラメータ統計の時
		if ($act === 'param' || $act === 'pie_chart' || $act === 'transition') {
			
			// 検索語の時
			if ($param === 'search_word' || $param === 'search_words') {
				$ymd = $y . '/' . $m . '/';
			}
			
			// 検索語以外の時
			else {
				$ymd = $GLOBALS['args'][1] . '/' . $y . '/' . $m . '/';
			}
			
		}
		
		// グラフの時
		elseif ($act === 'graph') {
			$opt = '/' . $y . '/' . $m . '/' . $d;
		}
		
		// 詳細の時
		elseif ($act === 'visit') {
			$ymd = $y . '/' . $m . '/' . $d . '/';
		}
		
		// 日間詳細の時
		elseif ($act === 'param_daily') {
			$opt = '/' . $y . '/' . $m . '/' . $d;
		}
		
		// ページ情報の時
		elseif ($act === 'page_data' and isset($GLOBALS['args'][2]) and $GLOBALS['args'][2] === 'csv') {
			$opt = '/csv';
		}
		
		// エラー＆ページ情報ではない時
		elseif ($act !== 'page_data') {
			$ymd = $y . '/' . $m . '/';
		}
		
		// 一時配列を初期化
		$links = [];
		
		// iniファイル読み込みループ
		foreach ($menu as $key => $val) {
			
			// 基本の時はkeyを空欄にする
			if ($key === 'base') {
				$key = '';
				$end = '';
			}
			
			// 検索語の時は変則対応
			elseif ($key === 'word' || $key === 'words') {
				$act = 'param/search_' . $key;
				$key = '';
				$end = '';
			}
			
			// オプションが存在する時は「/」を追記
			else {
				$end = '/';
			}
			
			// 配列にメニューリンクを入れる
			$links[] = '<a href="' . PATH . '/' . $act . '/' . $ymd . $key . $opt . $end . '">' . $val . '</a>';
			
		}
		
		// インデントタブを設定
		$tab1 = "\n\t\t\t\t";
		$tab2 = $tab1 . "\t";
		
		// メニューを整形
		$nav = $tab1 . '<div id="content_filter">' . $tab2 . implode("$tab2", $links) . $tab1 . '</div>';
		
		// メニューリンクを返す
		return $nav;
		
	}
	
	//---------------------------------------------------------
	//  前日翌日リンク生成
	//---------------------------------------------------------
	
	private static function createLinkPath($type, $y, $m, $d, $opt) {
		
		// 日付をintにキャスト
		$m = (int) $m;
		$d = (int) $d;
		
		// 前日の時
		if ($type === 'prev') {
			$plusM = -1;
			$checkD = 1;
			$checkM = 1;
			$changeM = 12;
		}
		
		// 翌日の時
		else {
			$plusM = 1;
			$checkD = self::monthDays($y, $m);
			$checkM = 12;
			$changeM = 1;
		}
		
		// 日チェック
		if ($d == $checkD) {
			
			// 月チェック
			if ($m == $checkM) {
				$y += $plusM;
				$m = $changeM;
			}
			
			// 月増減
			else {
				$m += $plusM;
			}
			
			// 日を設定
			$d = ($type === 'prev') ? self::monthDays($y, $m) : 1;
			
		}
		
		// 日増減
		else {
			$d += $plusM;
		}
		
		// 月が1桁の時は2桁に整形
		if ($m < 10) {
			$m = '0' . $m;
		}
		
		// 日が1桁の時は2桁に整形
		if ($d < 10) {
			$d = '0' . $d;
		}
		
		// 日付指定オプションの時
		if ($opt === 'timely') {
			$opt = $d;
			$d = 'timely';
		}
		
		// オプションが存在する時は/を追記
		if ($opt) {
			$opt .= '/';
		}
		
		// 年月日を設定
		$ymd = $y . '/' . $m . '/' . $d;
		
		// リンクを整形
		$link = $ymd . '/' . $opt . '" title="' . $ymd;
		
		// リンクを返す
		return $link;
		
	}
	
	//---------------------------------------------------------
	//  月の最終日の取得
	//---------------------------------------------------------
	
	public static function monthDays($y, $m) {
		
		// 日付をintにキャスト
		$y = (int) $y;
		$m = (int) $m;
		
		// 2月以外の時
		if ($m !== 2) {
			
			// 配列を設定
			$days = [
				1 => 31,
				2 => 28,
				3 => 31,
				4 => 30,
				5 => 31,
				6 => 30,
				7 => 31,
				8 => 31,
				9 => 30,
				10 => 31,
				11 => 30,
				12 => 31
			];
			
			// そのまま値を返す
			return $days[$m];
			
		}
		
		// 2月の時
		else {
			
			// 閏年フラグを取得
			$yu = $y % 4;
			
			// 閏年の時は29を返す
			if ($yu === 0) {
				return 29;
			}
			
			// 閏年以外の時は28を返す
			else {
				return 28;
			}
			
		}
		
	}
	
}

