<?php

class Canvas {
	
	//---------------------------------------------------------
	//  indexクラス用グラフを取得（canvas & JavaScript）
	//---------------------------------------------------------
	
	public static function index($width, $height, $length, $left, $px, $tdCounts, $ydCounts, $h) {
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// 変数を初期化
		$count = '';
		$canvas = '';
		$uuLine = '';
		$pvLine = '';
		$uuRect = '';
		$pvRect = '';
		
		// 連結点の色補正を設定
		$plus = 40;
		
		// インデントタブを設定
		$tab = "\n\t\t\t\t\t\t";
		
		// 折れ線グラフの色を取得
		$uuLineColor = $GLOBALS['config']['content']['uuLineColor'];
		$pvLineColor = $GLOBALS['config']['content']['pvLineColor'];
		
		// 連結点の色を設定
		$uuRectColor = self::toRectColor($uuLineColor, $plus);
		$pvRectColor = self::toRectColor($pvLineColor, $plus);
		
		// 不透明度を設定
		$alpha = 0.5;
		
		// 時間をintにキャスト
		$h = (int) $h;
		
		////////////////////////////////////////////////////////////
		
		// テンプレートファイルを読み込み
		[$head, $body1, $body2, $body3, $body4, $foot] = $tmpl->read('canvas.htm');
		
		// 時間別グラフ出力ループ
		for ($i = 0; $i < 24; ++$i) {
			
			// 桁補正
			$zero = ($i < 10) ? 0 : '';
			
			// カラム名を設定
			$uuName = 'h' . $zero . $i . '_uu';
			$pvName = 'h' . $zero . $i . '_pv';
			
			// 現在時間以下の時
			if ($i <= $h) {
				
				// 今日のカウントを取得
				$uu = $tdCounts[$uuName];
				$pv = $tdCounts[$pvName];
				
			}
			
			// 現在時間より大きい時
			else {
				
				// 現在時間 + 1時間の時
				if ($i === $h + 1) {
					
					// テンプレートを変更
					$body3 = $body4;
					
					// 左座標を一時的に戻す
					$left -= $length;
					
					// 不透明度を設定
					$uuLine .= $body1 . "ctx.moveTo($left, $uuTop);";
					$uuLine .= $tab . "ctx.strokeStyle = 'rgba($uuLineColor, $alpha)';";
					$pvLine .= $body1 . "ctx.moveTo($left, $pvTop);";
					$pvLine .= $tab . "ctx.strokeStyle = 'rgba($pvLineColor, $alpha)';";
					$uuRect .= $body1 . "ctx.fillStyle = 'rgba($uuRectColor, $alpha)';";
					$pvRect .= $body1 . "ctx.fillStyle = 'rgba($pvRectColor, $alpha)';";
					
					// 左座標を進める
					$left += $length;
					
				}
				
				// 昨日のカウントを取得
				$uu = $ydCounts[$uuName];
				$pv = $ydCounts[$pvName];
				
			}
			
			// 高さを算出
			$uuTop = $height - round($uu * $px);
			$pvTop = $height - round($pv * $px);
			
			// 一番最初の時
			if ($i === 0) {
				
				// 移動メソッドを追記
				$uuLine .= $tab . "ctx.moveTo($left, $uuTop);";
				$pvLine .= $tab . "ctx.moveTo($left, $pvTop);";
				
			}
			
			// 二番目以降の時
			else {
				
				// 折れ線描画メソッドを追記
				$uuLine .= $tab . "ctx.lineTo($left, $uuTop);";
				$pvLine .= $tab . "ctx.lineTo($left, $pvTop);";
				
			}
			
			// 矩形の座標を調整
			$rectLeft = $left - 4;
			$uuRectTop = $uuTop - 4;
			$pvRectTop = $pvTop - 4;
			
			// 矩形描画メソッドを追記
			$uuRect .= $tab . "ctx.fillRect($rectLeft, $uuRectTop, 7, 7);";
			$pvRect .= $tab . "ctx.fillRect($rectLeft, $pvRectTop, 7, 7);";
			
			// テンプレート変数にセット
			$vars['uu'] = $uu;
			$vars['pv'] = $pv;
			$vars['left'] = $left - 6 . 'px';
			$vars['uuTop'] = $uuTop - 4 . 'px';
			$vars['pvTop'] = $pvTop - 4 . 'px';
			
			// 連結点を追記
			$count .= $tmpl->res($body3, $vars);
			
			// 左座標を増やす
			$left += $length;
			
		}
		
		// キャンバスサイズを設定
		$vars['width'] = $width;
		
		// canvasを整形
		$canvas .= $tmpl->res($head, $vars) . "ctx.strokeStyle = 'rgb($uuLineColor)';";
		$canvas .= $uuLine . $body1 . "ctx.strokeStyle = 'rgb($pvLineColor)';";
		$canvas .= $pvLine . $body1 . "ctx.fillStyle = 'rgb($uuRectColor)';";
		$canvas .= $uuRect . $body1 . "ctx.fillStyle = 'rgb($pvRectColor)';";
		$canvas .= $pvRect . $body2 . $count . $foot;
		
		// canvasを返す
		return $canvas;
		
	}
	
	//---------------------------------------------------------
	//  graphクラス用グラフを取得（canvas & JavaScript）
	//---------------------------------------------------------
	
	public static function graph($width, $height, $length, $left, $px, $uuCounts, $pvCounts, $begin, $end) {
		
		// テンプレートクラスを取得
		$tmpl = $GLOBALS['object']['tmpl'];
		
		// 変数を初期化
		$count = '';
		$canvas = '';
		$uuLine = '';
		$pvLine = '';
		$uuRect = '';
		$pvRect = '';
		
		// 連結点の色補正を設定
		$plus = 40;
		
		// インデントタブを設定
		$tab = "\n\t\t\t\t\t\t";
		
		// 折れ線グラフの色を取得
		$uuLineColor = $GLOBALS['config']['content']['uuLineColor'];
		$pvLineColor = $GLOBALS['config']['content']['pvLineColor'];
		
		// 連結点の色を設定
		$uuRectColor = self::toRectColor($uuLineColor, $plus);
		$pvRectColor = self::toRectColor($pvLineColor, $plus);
		
		// 不透明度を設定
		$alpha = 0.5;
		
		////////////////////////////////////////////////////////////
		
		// テンプレートファイルを読み込み
		[$head, $body1, $body2, $body3, $body4, $foot] = $tmpl->read('canvas.htm');
		
		// データ取得ループ
		for ($i = $begin; $i < $end; ++$i) {
			
			// データが無い時
			if (!isset($pvCounts[$i])) {
				
				// 左位置を追加
				$left += $length;
				
				// 次へ
				continue;
				
			}
			
			// カウントを取得
			$uu = $uuCounts[$i];
			$pv = $pvCounts[$i];
			
			// 高さを算出
			$uuTop = $height - round($uu * $px);
			$pvTop = $height - round($pv * $px);
			
			// 一番最初の時
			if (!$pvLine) {
				
				// 移動メソッドを追記
				$uuLine .= $tab . "ctx.moveTo($left, $uuTop);";
				$pvLine .= $tab . "ctx.moveTo($left, $pvTop);";
				
			}
			
			// 二番目以降の時
			else {
				
				// 折れ線描画メソッドを追記
				$uuLine .= $tab . "ctx.lineTo($left, $uuTop);";
				$pvLine .= $tab . "ctx.lineTo($left, $pvTop);";
				
			}
			
			// 矩形の座標を調整
			$rectLeft = $left - 4;
			$uuRectTop = $uuTop - 4;
			$pvRectTop = $pvTop - 4;
			
			// 矩形描画メソッドを追記
			$uuRect .= $tab . "ctx.fillRect($rectLeft, $uuRectTop, 7, 7);";
			$pvRect .= $tab . "ctx.fillRect($rectLeft, $pvRectTop, 7, 7);";
			
			// テンプレート変数にセット
			$vars['uu'] = $uu;
			$vars['pv'] = $pv;
			$vars['left'] = $left - 6 . 'px';
			$vars['uuTop'] = $uuTop - 4 . 'px';
			$vars['pvTop'] = $pvTop - 4 . 'px';
			
			// 連結点を追記
			$count .= $tmpl->res($body3, $vars);
			
			// 左座標を増やす
			$left += $length;
			
		}
		
		// キャンバスサイズを設定
		$vars['width'] = $width;
		
		// canvasを整形
		$canvas .= $tmpl->res($head, $vars) . "ctx.strokeStyle = 'rgb($uuLineColor)';";
		$canvas .= $uuLine . $body1 . "ctx.strokeStyle = 'rgb($pvLineColor)';";
		$canvas .= $pvLine . $body1 . "ctx.fillStyle = 'rgb($uuRectColor)';";
		$canvas .= $uuRect . $body1 . "ctx.fillStyle = 'rgb($pvRectColor)';";
		$canvas .= $pvRect . $body2 . $count . $foot;
		
		// canvasを返す
		return $canvas;
		
	}
	
	//---------------------------------------------------------
	//  色補正
	//---------------------------------------------------------
	
	private static function toRectColor($color, $plus) {
		
		// RGBに分割
		[$r, $g, $b] = explode(',', $color);
		
		// 補正値を足す
		$r += $plus;
		$g += $plus;
		$b += $plus;
		
		// 補正後の色を返す
		return "$r, $g, $b";
		
	}
	
}

