<?php

class Logging {
	
	//---------------------------------------------------------
	//  メイン処理
	//---------------------------------------------------------
	
	public static function execute($baseDir, $workDir) {
		
		// config.iniのパスを設定
		$config = $workDir . '/config.ini';
		
		// config.iniが存在する時
		if (file_exists($config)) {
			
			// システム設定ファイルを解析
			$GLOBALS['config'] = parse_ini_file($config, true);
			
		}
		
		// config.iniが存在しない時
		else {
			
			// default.iniのパスを設定
			$default = $workDir . '/default.ini';
			
			// システム設定ファイルを解析
			$GLOBALS['config'] = parse_ini_file($default, true);
			
			// default.iniをconfig.iniにコピー
			copy($default, $config);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 自分のアクセスをカウントしない時は終了
		if (!$GLOBALS['config']['content']['allowCountAdmin'] && (isset($_COOKIE['lunalysLogin']))) {
			return;
		}
		
		// UA情報が存在する時は取得する
		$uaFull = (isset($_SERVER['HTTP_USER_AGENT'])) ? strip_tags($_SERVER['HTTP_USER_AGENT']) : '';
		
		// ページタイトル取得の時は終了
		if ($uaFull === 'Lunalys') {
			return;
		}
		
		// 同じIPのUAチェック可否設定を取得
		$allowCountAnotherUA = $GLOBALS['config']['content']['allowCountAnotherUA'];
		
		// 別アクセスと判断する空白時間を取得
		$visitBlankTime = $GLOBALS['config']['content']['visitBlankTime'];
		
		// 詳細ログローテーション件数を取得
		$visitSaveLimit = $GLOBALS['config']['content']['visitSaveLimit'];
		
		// アクセスルートの最大保存件数を取得
		$routeSaveLimit = $GLOBALS['config']['content']['routeSaveLimit'];
		
		////////////////////////////////////////////////////////////
		
		// ディレクトリ＆ファイルパスを設定
		$fwDir = $baseDir . '/framework';
		$dataDir = $baseDir . '/data';
		$errorLog = $dataDir . '/' . $GLOBALS['config']['framework']['errorLog'];
		
		// エラー関連のオプションを設定
		ini_set('error_reporting', E_ALL);
		ini_set('display_errors', $GLOBALS['config']['framework']['displayErrors']);
		ini_set('log_errors', $GLOBALS['config']['framework']['logErrors']);
		ini_set('error_log', $errorLog);
		
		// タイムゾーンを設定
		ini_set('date.timezone', $GLOBALS['config']['framework']['timeZone']);
		
		// 時差設定を取得
		$gmtDiff = $GLOBALS['config']['framework']['gmtDiff'];
		
		////////////////////////////////////////////////////////////
		
		// サイト番号を取得
		$site = (isset($_GET['site']) && is_numeric($_GET['site'])) ? (int) $_GET['site'] : 1;
		
		// DBのサフィックスを設定（接尾辞）
		$suffix = '_' . $site . '.sqlite';
		
		// メインDB名を設定
		$mainDB = 'access_main' . $suffix;
		
		////////////////////////////////////////////////////////////
		
		// アップデートクラスを読み込み
		include($workDir . '/modules/Update.php');
		
		// ver 2.5.0 で拡張子変更（将来的に削除予定）
		Update::update250($dataDir);
		
		////////////////////////////////////////////////////////////
		
		// DBクラスを読み込み
		include($fwDir . '/modules/core/DB.php');
		
		// インスタンスを生成
		$db = new DB($dataDir);
		
		////////////////////////////////////////////////////////////
		
		// フラグ変数を初期化
		$doSaveUA = false;
		$doVacuum = false;
		$doUpdateUU = false;
		$doUpdateDay = false;
		$doUpdateMonth = false;
		
		// 変数を初期化
		$params = [];
		$refId = '';
		$referrer = '';
		$updateCountUU = '';
		$updateTotalUU = '';
		
		// Cookie用変数を初期化
		$visitorId = '';
		$visitCount = 0;
		
		////////////////////////////////////////////////////////////
		
		// アクション名を取得
		$act = (isset($_GET['act'])) ? $_GET['act'] : '';
		
		// Cookieが存在する時
		if (isset($_COOKIE['lunalysVisit'])) {
			
			// Cookieをパース
			parse_str($_COOKIE['lunalysVisit'], $result);
			
			// IDを取得
			$visitorId = $result['visitorId'];
			
			// 訪問回数を取得
			$visitCount = $result['visitCount'];
			
		}
		
		// Cookieが存在しない時はIDを生成
		else {
			$visitorId = uniqid(rand(100, 999) . '_');
		}
		
		// クラスを読み込み
		include($workDir . '/modules/UserAgent.php');
		include($workDir . '/modules/RemoteHost.php');
		
		// IPアドレスを取得
		$ipAddress = RemoteHost::parseIP();
		
		// UA、OS、UA種別、端末種別、キャリア、UA-CH、モバイル判定を取得
		[$ua, $os, $uaType, $device, $carrier, $chUA, $chMobile] = UserAgent::parseUA($uaFull);
		
		// 現在日付を取得する
		[$y, $m, $d, $hh, $mm, $ss, $wday] = explode(' ', gmdate('Y m d H i s D', $_SERVER['REQUEST_TIME'] + $gmtDiff * 60 * 60));
		
		// 月間DB名を設定
		$tmDB = 'access_' . $y . '_' . $m . $suffix;
		
		// 年-月-日を設定
		$currentDate = $y . '-' . $m . '-' . $d;
		
		// 年-月を設定
		$currentMonth = $y . '-' . $m;
		
		// 時:分:秒を設定
		$currentTime = $hh . ':' . $mm . ':' . $ss;
		
		// 曜日を設定
		$currentWday = $wday;
		
		// 時間（intにキャスト）を設定
		$currentHour = (int) $hh;
		
		// 現在の日時と曜日を保持
		$GLOBALS['args']['date'] = $currentDate;
		$GLOBALS['args']['wday'] = $currentWday;
		
		////////////////////////////////////////////////////////////
		
		// メインDBが存在しない時は作成
		if (!$db->exists($mainDB)) {
			self::createMainTable($workDir, $mainDB, $db);
		}
		
		// 月間DBに接続
		$db->connect($tmDB);
		
		// メインDBに追加接続
		$db->attach($mainDB);
		
		////////////////////////////////////////////////////////////
		
		// 検索条件を設定
		$where = "where date = '$currentDate'";
		$order = 'order by last_time desc limit 1';
		
		// UAもチェックする時はwhere句に追記
		if ($allowCountAnotherUA) {
			$where .= ' and ua = ' . $db->escape($ua);
		}
		
		// include以外の時
		if ($act) {
			
			// where句を設定
			$whereOrderBy = $where . " and ip_address = '$ipAddress' " . $order;
			
			// js 呼び出しの時
			if ($act === 'js') {
				header('Content-Type: text/javascript');
			}
			
			// img 呼び出しの時
			elseif ($act === 'img') {
				
				// 透明gif画像を出力
				self::showClearImg();
				
				// 呼び出し元をURLに設定
				if (!isset($_GET['url']) && isset($_SERVER['HTTP_REFERER'])) {
					$_GET['url'] = $_SERVER['HTTP_REFERER'];
				}
				
			}
			
			// click 呼び出しの時
			elseif ($act === 'click') {
				
				// クラスを読み込み
				include($workDir . '/modules/Click.php');
				
				// クリック情報を追記
				exit(Click::addClick($db, $whereOrderBy));
				
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 各種テーブル名を取得
		$visitTable = $GLOBALS['config']['table']['visit'];
		$paramTable = $GLOBALS['config']['table']['param'];
		$dailyTable = $GLOBALS['config']['table']['daily'];
		$countTable = $GLOBALS['config']['table']['count'];
		$totalTable = $GLOBALS['config']['table']['total'];
		
		////////////////////////////////////////////////////////////
		
		// ver 2.6.0 でカラムを追加（将来的に削除予定）
		Update::update260($db, $visitTable);
		
		////////////////////////////////////////////////////////////
		
		// クラスを読み込み
		include($workDir . '/modules/Page.php');
		
		// ページIDを取得
		$pageId = Page::parseId($db);
		
		////////////////////////////////////////////////////////////
		
		// リンク元を取得（JavaScript） 
		if (isset($_GET['referrer'])) {
			$referrer = $_GET['referrer'];
		}
		
		// リンク元を取得
		elseif (isset($_SERVER['HTTP_REFERER'])) {
			$referrer = $_SERVER['HTTP_REFERER'];
		}
		
		// リンク元が存在する時
		if ($referrer) {
			
			// 許可するドメインを取得
			$allowDomain = $GLOBALS['config']['content']['allowDomain'];
			
			// 設定がない場合はサーバーのドメインを取得
			if (!$allowDomain) {
				$allowDomain = $_SERVER['HTTP_HOST'];
			}
			
			// リンク元が同じドメインの時
			if (preg_match("/(http|https):\/\/[^\/]*($allowDomain)/Ai", $referrer)) {
				$referrer = '';
			}
			
			// リンク元が違うドメインの時
			else {
				
				// クラスを読み込み
				include($workDir . '/modules/Referrer.php');
				
				// リンク元No,検索語を取得
				[$refId, $searchWords] = Referrer::parseWords($db, $referrer);
				
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// トランザクション開始
		$db->begin();
		
		// クエリーを作成
		$selectTdDateQuery = "select td_date from $totalTable;";
		
		// クエリーを実行
		$tdDate = $db->queryFetch($selectTdDateQuery, 'td_date');
		
		// ログの日付と現在の日付が違う時
		if ($tdDate !== $currentDate) {
			
			// 日更新フラグを設定
			$doUpdateDay = true;
			
			// 月更新フラグを設定
			if (!preg_match("/$currentMonth/A", $tdDate)) {
				$doUpdateMonth = true;
			}
			
		}
		
		// 日付が同じ時はトランザクション終了
		else {
			$db->commit();
		}
		
		////////////////////////////////////////////////////////////
		
		// 日の更新がある時
		if ($doUpdateDay) {
			
			// UU更新フラグを設定
			$doUpdateUU = true;
			
			// 日曜日の時はフラグを設定
			if ($currentWday === 'Sun') {
				$doVacuum = true;
			}
			
			// 月の更新がある時
			if ($doUpdateMonth) {
				
				// クラスを読み込み
				include_once($workDir . '/modules/Sql.php');
				
				// テーブルを生成
				$db->query(Sql::createCountTable($countTable));
				$db->query(Sql::createVisitTable($visitTable));
				$db->query(Sql::createParamTable($paramTable));
				$db->query(Sql::createDailyTable($dailyTable));
				
			}
			
			// クエリーを作成
			$insertCountQuery = "insert into $countTable (date, wday) values ('$currentDate', '$currentWday');";
			
			// クエリーを実行
			$db->query($insertCountQuery);
			
			// 先日日付を取得
			$ydDate = gmdate('Y-m-d', strtotime('-1 days') + $gmtDiff * 60 * 60);
			
			// 累計カウントテーブルが存在する時
			if ($tdDate) {
				$query = "update $totalTable set td_date = '$currentDate', yd_date = '$ydDate';";
			}
			
			// 累計カウントテーブルが存在しない時
			else {
				$query = "insert into $totalTable (td_date, yd_date) values ('$currentDate', '$ydDate');";
			}
			
			// クエリーを実行
			$db->query($query);
			
			// トランザクション終了
			$db->commit();
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 日付が同じ場合
		else {
			
			// IP検索条件
			$whereIP = '';
			
			// PCの時
			if ($uaType === 'PC') {
				$whereIP = "ip_address = '$ipAddress'";
			}
			
			// PCではない時
			else {
				
				// IPアドレスを分割
				$ips = explode('.', $ipAddress);
				
				// IPアドレスの末尾を削除
				array_pop($ips);
				
				// 検索条件を設定
				$whereIP = "ip_address like '" . implode('.', $ips) . "%'";
				
			}
			
			// 条件にID一致を追記
			$where .= " and (visitor_id = '$visitorId' or $whereIP)";
			
			// 現在時刻からブランクと判定する時刻を算出
			$expiresHour = $currentHour - $visitBlankTime;
			
			// ロボットでないかつ1時以降の時
			if ($uaType !== 'Robot' && $expiresHour > 0) {
				
				// 時間が10時未満の時は2桁に整形
				if ($expiresHour < 10) {
					$expiresHour = '0' . $expiresHour;
				}
				
				// ブランク時刻を取得
				$expiresTime = $expiresHour . ':' . $mm . ':' . $ss;
				
				// 検索条件に有効期限を追記
				$where .= " and last_time > '$expiresTime'";
				
			}
			
			////////////////////////////////////////////////////////////
			
			// クエリーを作成
			$selectVititQuery = "select seq, last_time, route, stay_time from $visitTable $where $order;";
			
			// クエリーを実行
			$rec = $db->queryFetch($selectVititQuery);
			
			// IP or IDが存在しない時はUU更新フラグを設定
			if (!$rec) {
				$doUpdateUU = true;
			}
			
			// IP or IDが既に存在している場合
			else {
				
				// ルートを取得
				$route = $rec['route'];
				
				// ルートを分割
				$ids = explode('-', $rec['route']);
				
				// PV総数を取得
				$routeCount = count($ids);
				
				// 配列の最終indexを算出
				$lastIndex = $routeCount - 1;
				
				// 最終ページIDを取得
				$lastId = (int) $ids[$lastIndex];
				
				// リロードの時は終了
				if ($pageId === $lastId) {
					return $db->close();
				}
				
				// ロボットかつ10PV以上の時は終了
				elseif ($uaType === 'Robot' && $routeCount >= 10) {
					return $db->close();
				}
				
				// PVが最大保存件数以上の時は終了
				elseif ($routeCount >= $routeSaveLimit) {
					return $db->close();
				}
				
				// ルートに追記
				$route .= '-' . $pageId;
				
				// 滞在時間を取得
				$stayTime = $rec['stay_time'];
				
				// 最終訪問時間を取得
				$lastTime = $rec['last_time'];
				
				// 経過時間を取得
				[$stayHH, $stayMM, $staySS] = self::parseStayTime($lastTime, $currentTime);
				
				// 時間を2桁に調整
				if ($stayHH > 0) {
					$stayMM = 60;
					$staySS = '00';
				}
				
				// 滞在時間が存在する時
				if ($stayTime) {
					$stayTime .= '-';
				}
				
				// 滞在時間に追記
				$stayTime .= $stayMM . ':' . $staySS;
				
				// シークエンスを取得
				$seq = $rec['seq'];
				
				// クエリーを作成
				$updateVisitQuery = "update $visitTable set last_time = '$currentTime', route = '$route', stay_time = '$stayTime' where seq = $seq;";
				
				// クエリーを実行
				$db->query($updateVisitQuery);
				
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// ユニークフラグがtrueの時
		if ($doUpdateUU) {
			
			// 変数を初期化
			$city = '';
			$searchWords = '';
			$displaySize = '';
			$clientSize = '';
			
			// 訪問回数を1増やす
			$visitCount++;
			
			//////////////////////////////////////////////////////////////////////
			
			// js呼び出しの時
			if ($act === 'js') {
				
				// クラスを読み込み
				include($workDir . '/modules/ClientSize.php');
				
				// ディスプレイ解像度,ブラウザ表示領域を取得
				[$displaySize, $clientSize] = ClientSize::parseSize();
				
			}
			
			//////////////////////////////////////////////////////////////////////
			
			// IP/ホスト/ドメイン を取得
			[$remoteHost, $hostDomain] = RemoteHost::parseDomain($ipAddress);
			
			// ロボットの時
			if ($uaType === 'Robot') {
				
				// 保存フラグを設定
				$doSaveUA = true;
				
			}
			
			// ロボット以外の時
			else {
				
				// ロボット判定
				$isRobot = false;
				
				// リモートホストとディスプレイサイズが空欄の時
				if ($remoteHost === 'unknown' && $displaySize === '') {
					$isRobot = true;
				}
				
				// リモートホストに特定の文字列が含まれる時
				elseif (preg_match('/(amazonaws.com|.com.ua|.net.ua|.org.ua|trendnet.org|trendmicro.com|.sjdc|.iad1|clients.your-server.de|google.com|googleusercontent.com|search.line-apps.com)$/', $remoteHost)) {
					$isRobot = true;
				}
				
				// UAが特定バージョンかつディスプレイサイズが空欄の時
				elseif (strpos($ua, '114.0') !== false && $displaySize === '') {
					$isRobot = true;
				}
				
				// ロボットの時はUA種別を上書き
				if ($isRobot) {
					$uaType = 'Robot';
				}
				
			}
			
			// 日本のドメインの時は都道府県を取得
			if (preg_match('/\.(jp|net|com|org)$/i', $hostDomain)) {
				$city = RemoteHost::parseCity($remoteHost, $workDir);
			}
			
			//////////////////////////////////////////////////////////////////////
			
			// 詳細ログパラメータ配列を設定
			$params['host_domain']= $hostDomain;
			$params['search_words'] = $searchWords;
			$params['ua_full'] = $uaFull;
			$params['ua'] = $ua;
			$params['os'] = $os;
			$params['display_size'] = $displaySize;
			$params['client_size'] = $clientSize;
			$params['device'] = $device;
			$params['carrier'] = $carrier;
			$params['city'] = $city;
			$params['ch_ua'] = $chUA;
			$params['ch_mobile'] = $chMobile;
			
			// 新規詳細ログデータを整形
			$values = "null, '$currentDate', '$currentWday', '$currentTime', '$currentTime', $visitCount, '$visitorId', ";
			$values .= "'$ipAddress', '$remoteHost', :host_domain, '$refId', '$pageId', '', '', :search_words, ";
			$values .= "'$uaType', :ua_full, :ua, :os, :display_size, :client_size, :device, :carrier, :city, :ch_ua, :ch_mobile";
			
			// クエリーを作成
			$insertVisitQuery = "insert into $visitTable values ($values);";
			
			// クエリーを実行
			$db->prepare($insertVisitQuery, $params);
			
			// 時間別UUのカラム名を設定
			$uuCol = 'h' . $hh . '_uu';
			
			// クエリーに追記
			$updateCountUU = ", sum_uu = sum_uu + 1, $uuCol = $uuCol + 1";
			$updateTotalUU = ", total_uu = total_uu + 1";
			
			// UA全文は保存しない
			$params['ua_full'] = '';
			
			// リンク元を追加
			$params['referrer'] = $refId;
			
			// JavaScript呼び出しか初回時は訪問回数を追加
			if ($act || $visitCount === 1) {
				$params['visit_count'] = $visitCount;
			}
			
		}
		
		////////////////////////////////////////////////////////////
		
		// ロボットではない時は更新
		if ($uaType !== 'Robot') {
			
			// 月間アクセスページログを更新
			if ($pageId) {
				$params['page'] = $pageId;
			}
			
			// PVカラム名を設定
			$pvCol = 'h' . $hh . '_pv';
			
			// クエリーを作成
			$updateCountQuery = "update $countTable set sum_pv = sum_pv + 1, $pvCol = $pvCol + 1 $updateCountUU where date = '$currentDate';";
			
			// クエリーを実行
			$db->query($updateCountQuery);
			
			// クエリーを作成
			$updateTotalQuery = "update $totalTable set total_pv = total_pv + 1 $updateTotalUU;";
			
			// クエリーを実行
			$db->query($updateTotalQuery);
			
		}
		
		////////////////////////////////////////////////////////////
		
		// 月間ログテーブルの更新がない時は終了
		if (!$params) {
			return $db->close();
		}
		
		// トランザクション開始
		$db->begin();
		
		// 
		// 月間ログテーブルを更新（ロボットでない時）
		if ($uaType !== 'Robot') {
			self::updateParams($db, $params, $uaType);
		}
		
		// 月間ログテーブルを更新（ロボットの時）
		elseif ($ua && $doSaveUA) {
			self::updateParamTable($db, 'robot', $ua);
		}
		
		// トランザクション終了
		$db->commit();
		
		// 詳細ログをローテーション
		if ($doUpdateUU && $visitSaveLimit) {
			self::rotationVisitTable($db, $visitSaveLimit);
		}
		
		// データ領域開放
		if ($doVacuum) {
			$db->query('vacuum;');
		}
		
		// DB接続終了
		$db->close();
		
		// js 呼び出しの時はCookieを発行
		if ($act === 'js') {
			self::saveCookie('lunalysVisit', "visitorId=$visitorId&visitCount=$visitCount");
		}
		
	}
	
	//---------------------------------------------------------
	//  メインテーブル作成
	//---------------------------------------------------------
	
	private static function createMainTable($workDir, $mainDB, $db) {
		
		// クラスを読み込み
		include($workDir . '/modules/Sql.php');
		
		// メインDBに接続
		$db->connect($mainDB);
		
		// テーブルを生成
		$db->query(Sql::createTotalTable($GLOBALS['config']['table']['total']));
		$db->query(Sql::createPageTable($GLOBALS['config']['table']['page']));
		$db->query(Sql::createPageTable($GLOBALS['config']['table']['click']));
		$db->query(Sql::createPageTable($GLOBALS['config']['table']['referrer']));
		
		// メインDBを切断
		$db->close();
		
	}
	
	//---------------------------------------------------------
	//  月間項目ログの更新
	//---------------------------------------------------------
	
	private static function updateParams($db, $params, $uaType) {
		
		// 月間パラメータの更新ループ
		foreach ($params as $key => $val) {
			
			// データが無い時は次へ
			if (!$val) {
				continue;
			}
			
			// PC以外のUAの時
			if ($key === 'ua' && $uaType !== 'PC') {
				
				// 携帯の時
				if ($uaType === 'Mobile') {
					$key = 'mobile';
				}
				
				// ゲーム機の時
				elseif ($uaType === 'Game') {
					$key = 'game';
				}
				
			}
			
			// 検索語の時
			if ($key === 'search_words') {
				
				// スペースで単語に分割
				$words = explode(' ', $val);
				
				// 単語取得ループ
				foreach ($words as $word) {
					
					// データがある場合は単語で保存
					if ($word) {
						self::updateParamTable($db, 'search_word', $word);
					}
					
				}
				
				// 2語以上ある時
				if (count($words) > 1) {
					
					// 月間フレーズログを更新
					self::updateParamTable($db, 'search_words', $val);
					
				}
				
			}
			
			// 検索語以外の時
			else {
				self::updateParamTable($db, $key, $val);
			}
			
		}
		
	}
	
	//---------------------------------------------------------
	//  月間項目ログの更新
	//---------------------------------------------------------
	
	public static function updateParamTable($db, $type, $value) {
		
		// テーブル名を取得
		$paramTable = $GLOBALS['config']['table']['param'];
		
		// 値をエスケープ
		$value = $db->escape($value);
		
		// クエリーを作成
		$selectCountQuery = "select count(type) as count from $paramTable where type = '$type' and value = $value;";
		
		// クエリーを実行
		$count = $db->queryFetch($selectCountQuery, 'count');
		
		// 更新 or 挿入用SQL
		$query = '';
		
		// データ更新用クエリーを作成
		if ($count) {
			$query = "update $paramTable set count = count + 1 where type = '$type' and value = $value;";
		}
		
		// データ挿入用クエリーを作成
		else {
			$query = "insert into $paramTable values ('$type', $value, 1);";
		}
		
		// クエリーを実行
		$db->query($query);
		
		// 日間項目ログを更新
		self::updateDailyTable($db, $type, $value);
		
	}
	
	//---------------------------------------------------------
	//  日間項目ログの更新
	//---------------------------------------------------------
	
	public static function updateDailyTable($db, $type, $value) {
		
		// テーブル名を取得
		$dailyTable = $GLOBALS['config']['table']['daily'];
		
		// 現在の日時と曜日を取得
		$date = $GLOBALS['args']['date'];
		$wday = $GLOBALS['args']['wday'];
		
		// クエリーを作成
		$selectCountQuery = "select count(type) as count from $dailyTable where date = '$date' and type = '$type' and value = $value;";
		
		// クエリーを実行
		$count = $db->queryFetch($selectCountQuery, 'count');
		
		// 更新 or 挿入用SQL
		$query = '';
		
		// データ更新用クエリーを作成
		if ($count) {
			$query = "update $dailyTable set count = count + 1 where date = '$date' and type = '$type' and value = $value;";
		}
		
		// データ挿入用クエリーを作成
		else {
			$query = "insert into $dailyTable values ('$date', '$wday', '$type', $value, 1);";
		}
		
		// クエリーを実行
		$db->query($query);
		
	}
	
	//---------------------------------------------------------
	//  詳細ログローテーション
	//---------------------------------------------------------
	
	private static function rotationVisitTable($db, $visitSaveLimit) {
		
		// テーブル名を取得
		$visitTable = $GLOBALS['config']['table']['visit'];
		
		// クエリーを作成
		$selectCountQuery = "select count(seq) as count from $visitTable;";
		
		// クエリーを実行
		$count = $db->queryFetch($selectCountQuery, 'count');
		
		// 最大保存件数以下の時は何もしない
		if ($count <= $visitSaveLimit) {
			return;
		}
		
		// クエリーを作成
		$selectSeqQuery = "select seq from $visitTable order by seq limit 1;";
		
		// クエリーを実行
		$seq = $db->queryFetch($selectSeqQuery, 'seq');
		
		// クエリーを作成
		$deleteQuery = "delete from $visitTable where seq = $seq;";
		
		// クエリーを実行
		$db->query($deleteQuery);
		
	}
	
	//---------------------------------------------------------
	//  Cookieを発行
	//---------------------------------------------------------
	
	public static function saveCookie($name, $value) {
		
		// PHPエラーが出力済の時は終了
		if (headers_sent()) {
			return;
		}
		
		// 有効期限を設定（1年間）(60 * 60 * 24 * 365)
		$expires = $_SERVER['REQUEST_TIME'] + 31536000;
		
		// 対象ドメイン
		$domain = $_SERVER['HTTP_HOST'];
		
		// PHP 7.3 以上の時
		if (version_compare(PHP_VERSION, '7.3.0') >= 0) {
			
			// Cookieを発行（PHP 7.3以上）
			setcookie($name, $value, [
				'expires' => $expires,
				'path' => '/',
				'domain' => $domain,
				'samesite' => 'Lax',
				'secure' => false,
				'httponly' => false,
			]);
			
		}
		
		// PHP 7.3 未満の時
		else {
			setcookie($name, $value, $expires, '/', $domain);
		}
		
	}
	
	//---------------------------------------------------------
	//  経過時間の算出
	//---------------------------------------------------------
	
	private static function parseStayTime($begin, $end, $del = ':') {
		
		// デリミタで分割
		[$sh, $sm, $ss] = explode($del, $begin);
		[$eh, $em, $es] = explode($del, $end);
		
		// 秒数を算出
		$xss = ($sh * 60 * 60) + ($sm * 60) + $ss;
		$xes = ($eh * 60 * 60) + ($em * 60) + $es;
		
		// 総経過時間を算出
		$diff = $xes - $xss;
		
		// 経過分を算出
		$dm = floor($diff / 60);
		
		// 経過秒を算出
		$ds = $diff % 60;
		
		// 一時間以上の時
		if ($diff >= 3600) {
			
			// 経過時を算出
			$dh = floor($dm / 60);
			
			// 経過分を算出
			$dm = $dm - ($dh * 60);
			
		}
		
		// 一時間未満の時
		else {
			$dh = 0;
		}
		
		// 桁数を補正（時）
		if ($dh < 10) {
			$dh = '0' . $dh;
		}
		
		// 桁数を補正（分）
		if ($dm < 10) {
			$dm = '0' . $dm;
		}
		
		// 桁数を補正（秒）
		if ($ds < 10) {
			$ds = '0' . $ds;
		}
		
		// 経過時間を返す
		return [$dh, $dm, $ds];
		
	}
	
	//---------------------------------------------------------
	//  透明gif画像出力
	//---------------------------------------------------------
	
	private static function showClearImg() {
		
		// PHPエラーが出力済の時は終了
		if (headers_sent()) {
			return;
		}
		
		// 更新日付ヘッダーを出力
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
		
		// キャッシュコントロールヘッダーを出力
		header('Cache-Control: no-store, no-cache, must-revalidate');
		header('Cache-Control: post-check=0, pre-check=0', false);
		
		// Pragmaを出力
		header('Pragma: no-cache');
		
		// imgヘッダーを出力
		header('Content-Type: image/gif');
		
		// 透明gifバイナリを出力
		echo base64_decode('R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==');
		
	}
	
	//---------------------------------------------------------
	//  URL整形
	//---------------------------------------------------------
	
	public static function rewriteUrl($url, $isExtUrl = false) {
		
		// カット文字列を取得
		$cutUrl = ($isExtUrl) ? $GLOBALS['config']['content']['cutExternalUrl'] : $GLOBALS['config']['content']['cutInternalUrl'];
		
		// カット文字列が存在する時
		if ($cutUrl) {
			
			// URLカット文字列を整形
			$cutUrl = addcslashes($cutUrl, '/?-=^');
			
			// カット文字列以降を削除
			$url = preg_replace("/(.*$cutUrl).*/", "$1", $url);
			
		}
		
		// 「index.～」を削除
		$url = preg_replace('/index\.[\w]{2,4}$/', '', $url);
		
		// 外部URLの時はURLを返す
		if ($isExtUrl) {
			return $url;
		}
		
		// 「www.」「:～」「#～」を削除
		$url = preg_replace('/(www\.|:[\d]+|#.*)/', '', $url);
		
		// URL引数は削除
		if ($GLOBALS['config']['content']['allowCutArgs']) {
			$url = preg_replace('/[\?&%].*/', '', $url);
		}
		
		// ファイル名がない時は「/」を補正
		if ($GLOBALS['config']['content']['allowAddSlash']) {
			$url = preg_replace('/(\/[^\.\/]+$|:\/\/[^\/]+$)/', "$1/", $url);
		}
		
		// 「/」の連続は「/」に統一
		$url = preg_replace('/[\/]{2,}/', '/', $url);
		
		// http:を元に戻す
		$url = preg_replace('/(http|https):/', "$1:/", $url);
		
		// URLを返す
		return $url;
		
	}
	
	//---------------------------------------------------------
	//  Web用の文字列に変換
	//---------------------------------------------------------
	
	public static function toWebStr($value, $encoding = 'UTF-8') {
		
		// 文字コードが違う時は変換
		if (mb_detect_encoding($value) !== $encoding) {
			$value = mb_convert_encoding($value, $encoding, 'ASCII, JIS, UTF-8, EUC-JP, SJIS');
		}
		
		// 半角カナを全角、全角英数字を半角に変換
		$value = mb_convert_kana($value, 'KVa', $encoding);
		
		// 参照文字列に変換
		$value = htmlspecialchars($value);
		
		// 参照文字列に変換（特殊文字）
		$value = str_replace(',', '&#44;', $value);
		
		// &amp;は&に戻す
		$value = str_replace('&amp;', '&', $value);
		
		// 改行コードを削除
		$value = preg_replace('/\r|\n/', '' , $value);
		
		// エンコードデータを返す
		return $value;
		
	}
	
	//---------------------------------------------------------
	//  HTMLからページタイトルを取得
	//---------------------------------------------------------
	
	public static function parseTitle($url) {
		
		// HTMLを読み込み
		$html = self::getHtml($url);
		
		// HTMLが取得出来なかった時はURLを設定
		if (!$html) {
			$title = $url;
		}
		
		// ページタイトルを切り出し
		else {
			$title = (preg_match('/<title>([^<>]+)</i', $html, $m)) ? $m[1] : $url;
		}
		
		// Web用にエンコードしたタイトルを返す
		return self::toWebStr(trim($title));
		
	}
	
	//---------------------------------------------------------
	//  外部WebサイトのHTMLを取得
	//---------------------------------------------------------
	
	public static function getHtml($url) {
		
		// 基本情報を設定
		$stream['http'] = ['method' => 'GET', 'header' => "User-Agent: Lunalys\r\n"];
		
		// HTTPS用追加設定
		$stream['ssl'] = ['verify_peer' => false, 'verify_peer_name' => false];
		
		// ストリームコンテキストを生成
		$context = stream_context_create($stream);
		
		// HTMLを読み込み
		return @file_get_contents($url, false, $context);
		
	}
	
}

