<?php

class UserAgent {
	
	//---------------------------------------------------------
	//  使用OS・ブラウザ解析 関数
	//---------------------------------------------------------
	
	public static function parseUA($uaFull) {
		
		// UA情報が存在しない時は空白を返す
		if (!$uaFull) {
			return ['', '', 'Robot', '', '', '', ''];
		}
		
		// 変数を初期化
		$ua = '';
		$os = '';
		$uaType = 'PC';
		$device = '';
		$carrier = '';
		$smart = false;
		$chUA = '';
		$chMobile = '';
		
		////////////////////////////////////////////////////////////////////////////////
		// [Robot] //
		
		// Googlebotの時
		if (strpos($uaFull, 'Googlebot') !== false) {
			$ua = 'Googlebot';
			$uaType = 'Robot';
		}
		
		// Google系の時
		elseif (preg_match('/([\w]+-Google)/', $uaFull, $m)) {
			$ua = $m[1];
			$uaType = 'Robot';
		}
		
		// Bingbotの時
		elseif (preg_match('/bingbot|BingPreview/', $uaFull)) {
			$ua = 'Bingbot';
			$uaType = 'Robot';
		}
		
		// MSNBotの時
		elseif (strpos($uaFull, 'msnbot') !== false) {
			$ua = 'MSNBot';
			$uaType = 'Robot';
		}
		
		// Yahoo! Slurpの時
		elseif (strpos($uaFull, 'Yahoo! Slurp') !== false) {
			$ua = 'Yahoo! Slurp';
			$uaType = 'Robot';
		}
		
		// Y!J-AGENTの時
		elseif (strpos($uaFull, 'Y!J-AGENT') !== false) {
			$ua = 'Y!J-AGENT';
			$uaType = 'Robot';
		}
		
		// 百度の時
		elseif (strpos($uaFull, 'Baiduspider') !== false) {
			$ua = 'Baiduspider';
			$uaType = 'Robot';
		}
		
		// Ask Jeevesの時
		elseif (strpos($uaFull, 'Ask Jeeves') !== false) {
			$ua = 'Ask Jeeves';
			$uaType = 'Robot';
		}
		
		// CFNetwork の時
		elseif (strpos($uaFull, 'CFNetwork') !== false) {
			$ua = 'CFNetwork';
			$uaType = 'Robot';
		}
		
		// WinHttpRequest の時
		elseif (strpos($uaFull, 'WinHttpRequest') !== false) {
			$ua = 'WinHttpRequest';
			$uaType = 'Robot';
		}
		
		// libwww-perlの時
		elseif (strpos($uaFull, 'libwww-perl') !== false) {
			$ua = 'libwww-perl';
			$uaType = 'Robot';
		}
		
		// DiffBrowser の時
		elseif (strpos($uaFull, 'DiffBrowser') !== false) {
			$ua = 'DiffBrowser';
			$uaType = 'Robot';
		}
		
		// Facebook Crawler の時
		elseif (strpos($uaFull, 'facebookexternalhit') !== false) {
			$ua = 'Facebook Crawler';
			$uaType = 'Robot';
		}
		
		// wkhtmltoimage の時
		elseif (strpos($uaFull, 'wkhtmltoimage') !== false) {
			$ua = 'wkhtmltoimage';
			$uaType = 'Robot';
		}
		
		// 特定の用語が含まれる時
		elseif (preg_match('/http:|https:|bot|spider|crawler|checker|reader|antenna|capture|screenshot/i', $uaFull)) {
			$ua = strip_tags($uaFull);
			$uaType = 'Robot';
		}
		
		////////////////////////////////////////////////////////////////////////////////
		// [フィーチャーフォン] //
		
		// jig browserの時
		elseif (preg_match('/jig browser[^\)]*\d; (\w*)/', $uaFull, $m)) {
			
			// OSとUA種別を設定
			$os = 'Symbian OS';
			$ua = 'jig browser';
			$uaType = 'Mobile';
			$device = $m[1];
			
			// docomo1
			if (preg_match('/([A-Z]{1,2})(\d{2}[A-Z])/', $device, $m)) {
				$carrier = 'docomo';
				$device = $m[1] . '-' . $m[2];
			}
			
			// docomo2
			elseif (preg_match('/[A-Z]{1,2}\d{3}i/', $device)) {
				$carrier = 'docomo';
			}
			
			// au1
			elseif (preg_match('/[A-Z]{2}\d{1}[A-Z]/', $device)) {
				$carrier = 'au';
			}
			
			// au2
			elseif (preg_match('/[A-Z]{2}\d{2}$/', $device)) {
				$carrier = 'au';
			}
			
			// UA、OS、UA種別、端末種別、キャリア を返す
			return [$ua, $os, $uaType, $device, $carrier, '', ''];
			
		}
		
		// ドコモの時
		elseif (preg_match('/DoCoMo\/\d\.\d ([A-Z]{1,2})(\d{2}[A-Z])/', $uaFull, $m)) {
			$carrier = 'docomo';
			$device = $m[1] . '-' . $m[2];
		}
		
		// ドコモの時2
		elseif (preg_match('/DoCoMo\/\d\.\d[\/ ]([A-Z]{1,2}\d{3}\w*)/', $uaFull, $m)) {
			$carrier = 'docomo';
			$device = $m[1];
		}
		
		// ドコモの時3
		elseif (preg_match('/\(([A-Z]{1,2})(\d{2}[A-Z]);FOMA/', $uaFull, $m)) {
			$carrier = 'docomo';
			$device = $m[1] . '-' . $m[2];
		}
		
		// ドコモの時4
		elseif (preg_match('/BlackBerry/', $uaFull)) {
			$carrier = 'docomo';
			$device = 'BlackBerry';
		}
		
		// ソフトバンクの時
		elseif (preg_match('/(SoftBank|Vodafone|J-PHONE)\/\d\.\d\/([^\/_]*)/', $uaFull, $m)) {
			$carrier = 'SoftBank';
			$device = $m[2];
		}
		
		// ソフトバンクの時2
		elseif (preg_match('/\((.*);SoftBank/', $uaFull, $m)) {
			$carrier = 'SoftBank';
			$device = $m[1];
		}
		
		// ソフトバンクの時3
		elseif (preg_match('/SoftBank ([^\s]*)/', $uaFull, $m)) {
			$carrier = 'SoftBank';
			$device = $m[1];
		}
		
		// au の時
		elseif (preg_match('/KDDI-(\w*)/', $uaFull, $m)) {
			$carrier = 'au';
			$device = $m[1];
		}
		
		////////////////////////////////////////////////////////////////////////////////
		// [Game] //
		
		// PS系の時
		elseif (preg_match('/PlayStation/i', $uaFull)) {
			
			// OSとUA種別を設定（統一）
			$os = 'PS OS';
			$ua = 'PS Browser';
			$uaType = 'Game';
			
			// PSPの時
			if (strpos($uaFull, 'PlayStation Portable') !== false) {
				$device = 'PSP';
			}
			
			// PS Vitaの時
			elseif (strpos($uaFull, 'PlayStation Vita') !== false) {
				$device = 'PS Vita';
			}
			
			// PS3～の時
			elseif (preg_match('/PlayStation (\d{1,})/i', $uaFull, $m)) {
				$device = 'PS' . $m[1];
			}
			
			// UA、OS、UA種別、端末種別、キャリア を返す
			return [$ua, $os, $uaType, $device, $carrier, '', ''];
			
		}
		
		// 任天堂系の時
		elseif (strpos($uaFull, 'Nintendo') !== false) {
			
			// OSとUA種別を設定（統一）
			$os = 'Nintendo OS';
			$ua = 'Nintendo Browser';
			$uaType = 'Game';
			
			// New Nintendo 3DSの時
			if (strpos($uaFull, 'New Nintendo 3DS') !== false) {
				$device = 'New Nintendo 3DS';
			}
			
			// その他ハードの時
			elseif (preg_match('/(Nintendo [^;\)]+)[;\)]/', $uaFull, $m)) {
				$device = $m[1];
			}
			
			// UA、OS、UA種別、端末種別、キャリア を返す
			return [$ua, $os, $uaType, $device, $carrier, '', ''];
			
		}
		
		////////////////////////////////////////////////////////////////////////////////
		// [PC] //
		
		// Samsung Internetの時
		elseif (preg_match('/SamsungBrowser\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Samsung Internet ' . $m[1];
		}
		
		// Yandexの時
		elseif (preg_match('/YaBrowser\/(\d{1,}\.\d{1,})/', $uaFull, $m)) {
			$ua = 'Yandex ' . $m[1];
		}
		
		// Whaleの時
		elseif (preg_match('/Whale\/(\d{1,}\.\d{1,})/', $uaFull, $m)) {
			$ua = 'Whale ' . $m[1];
		}
		
		// Vivaldiの時
		elseif (preg_match('/Vivaldi\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Vivaldi ' . $m[1];
		}
		
		// Operaの時
		elseif (preg_match('/OPR\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Opera ' . $m[1];
		}
		
		// Lunascape の時
		elseif (preg_match('/Lunascape[ \/](\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Lunascape ' . $m[1];
		}
		
		// Sleipnir の時
		elseif (preg_match('/Sleipnir( Version |\/)(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Sleipnir ' . $m[2];
		}
		
		// Edgeの時（Chromium or EdgeHTML）
		elseif (preg_match('/(Edg|Edge)\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Edge ' . $m[2];
		}
		
		// IE11以上の時
		elseif (preg_match('/Trident.+ rv:(\d{1,})\.\d/', $uaFull, $m)) {
			$ua = 'IE ' . $m[1];
		}
		
		// IE10以下の時
		elseif (preg_match('/MSIE (\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'IE ' . $m[1];
		}
		
		// Chromeの時
		elseif (preg_match('/Chrome\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Chrome ' . $m[1];
		}
		
		// Safariの時
		elseif (preg_match('/Version\/(\d{1,}\.\d).* Safari/', $uaFull, $m)) {
			$ua = 'Safari ' . $m[1];
		}
		
		// Firefoxの時1
		elseif (preg_match('/Firefox\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Firefox ' . $m[1];
		}
		
		// Firefoxの時2
		elseif (preg_match('/rv:(\d{1,}\.\d).*Firefox/', $uaFull, $m)) {
			$ua = 'Firefox ' . $m[1];
		}
		
		// Classillaの時
		elseif (preg_match('/rv:(\d{1,}\.\d).*Classilla/', $uaFull, $m)) {
			$ua = 'Classilla ' . $m[1];
		}
		
		// Lynxの時
		elseif (preg_match('/Lynx\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Lynx ' . $m[1];
		}
		
		// Konquerorの時
		elseif (preg_match('/Konqueror\/(\d{1,}\.\d)/', $uaFull, $m)) {
			$ua = 'Konqueror ' . $m[1];
		}
		
		////////////////////////////////////////////////////////////////////////////////
		// [etc] //
		
		// Blue Coat Proxyの時
		elseif ($uaFull === 'Mozilla/4.0 (compatible;)') {
			$ua = 'Blue Coat Proxy';
		}
		
		// その他特定の書式の場合もロボットとみなす
		elseif (preg_match('/compatible; ([^;]+)\/(\d\.\d)/', $uaFull, $m)) {
			$ua = $m[1] . ' ' . $m[2];
			$uaType = 'Robot';
		}
		
		// 不明の時はフルUAをそのままUAにセット
		else {
			$ua = $uaFull;
		}
		
		// ロボットの時は終了
		if ($uaType === 'Robot') {
			return [$ua, $os, $uaType, $device, $carrier, '', ''];
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// [OSの判定] //
		
		// Windowsの時
		if (strpos($uaFull, 'Windows') !== false) {
			
			// Windows Phone の時
			if (preg_match('/; ([^;]*); Windows Phone (\d\.\d)/', $uaFull, $m)) {
				$os = 'Windows Phone ' . $m[2];
				$uaType = 'Mobile';
			}
			
			// Windows 10 Mobile の時
			elseif (strpos($uaFull, 'Windows Phone 10.0') !== false) {
				$os = 'Windows 10 Mobile';
				$uaType = 'Mobile';
			}
			
			// Windows Mobileの時
			elseif (strpos($uaFull, 'Windows CE') !== false) {
				$os = 'Windows Mobile';
				$uaType = 'Mobile';
			}
			
			// その他 Windows の時
			elseif (strpos($uaFull, 'Windows NT 10.0') !== false) {
				$os = 'Windows 10';
			} elseif (strpos($uaFull, 'Windows NT 6.3') !== false) {
				$os = 'Windows 8.1';
			} elseif (strpos($uaFull, 'Windows NT 6.2') !== false) {
				$os = 'Windows 8';
			} elseif (strpos($uaFull, 'Windows NT 6.1') !== false) {
				$os = 'Windows 7';
			} elseif (strpos($uaFull, 'Windows NT 6.0') !== false) {
				$os = 'Windows Vista';
			} elseif (strpos($uaFull, 'NT 5.2') !== false) {
				$os = 'Windows Server 2003';
			} elseif (preg_match('/Windows (NT 5.1|XP)/', $uaFull)) {
				$os = 'Windows XP';
			} elseif (preg_match('/Windows (NT 5.0|2000)/', $uaFull)) {
				$os = 'Windows 2000';
			} elseif (preg_match('/Windows ME|Win 9x/', $uaFull)) {
				$os = 'Windows Me';
			} elseif (preg_match('/Windows 98|Win98/', $uaFull)) {
				$os = 'Windows 98';
			} elseif (preg_match('/Windows 95|Win95/', $uaFull)) {
				$os = 'Windows 95';
			} elseif (preg_match('/NT(\d\.\d)/', $uaFull, $m)) {
				$os = 'Windows NT' . $m[1];
			}
			
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// iOS の時
		elseif (preg_match('/(iPhone|iPad|iPod touch).* OS (\d{1,})_(\d)/' , $uaFull, $m)) {
			
			// OSとUA種別を設定（統一）
			$version = $m[2] . '.' . $m[3];
			$os = 'iOS ' . $version;
			$device = $m[1];
			$smart = true;
			
			// UAを再判定
			
			// Opera Mini の時
			if (preg_match('/OPiOS\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Opera Mini ' . $m[1];
			}
			
			// Opera Coast の時
			elseif (preg_match('/Coast\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Opera Coast ' . $m[1];
			}
			
			// Firefox Mobile の時
			elseif (preg_match('/FxiOS\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Firefox Mobile ' . $m[1];
			}
			
			// Edge Mobile の時
			elseif (preg_match('/EdgiOS\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Edge Mobile ' . $m[1];
			}
			
			// Chrome Mobile の時
			elseif (preg_match('/CriOS\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Chrome Mobile ' . $m[1];
			}
			
			// iLunascape の時
			elseif (strpos($uaFull, 'iLunascape') !== false) {
				$ua = 'iLunascape';
			}
			
			// Mobile Safari の時
			elseif (preg_match('/Version\/(\d{1,}\.\d).* Mobile.* Safari/', $uaFull, $m)) {
				$ua = 'Safari Mobile ' . $m[1];
			}
			
			// その他は標準ブラウザと判定
			else {
				$ua = 'Safari Mobile ' . $version;
			}
			
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// Androidの時
		elseif (strpos($uaFull, 'Android') !== false) {
			
			// OSを仮設定
			$os = 'Android';
			$smart = true;
			
			// 端末機種が取れる時1（Android x.x Build）
			if (preg_match('/Android (\d{1,}\.\d).*; (.+) Build/', $uaFull, $m)) {
				$os = 'Android ' . $m[1];
				$device = $m[2];
			}
			
			// 端末機種が取れる時2（Android x.x）
			elseif (preg_match('/Android (\d{1,}\.\d).*; (.+)\) AppleWebKit/', $uaFull, $m)) {
				$os = 'Android ' . $m[1];
				$device = $m[2];
			}
			
			// 端末機種が取れる時3（Android xx Build）
			elseif (preg_match('/Android (\d{1,}); ([^\)]+) Build/', $uaFull, $m)) {
				$os = 'Android ' . $m[1];
				$device = $m[2];
			}
			
			// 端末機種が取れる時4（Android xx）
			elseif (preg_match('/Android (\d{1,}); (.+)\) AppleWebKit/', $uaFull, $m)) {
				$os = 'Android ' . $m[1];
				$device = $m[2];
			}
			
			// 端末機種が取れない時1（Android x.x）
			elseif (preg_match('/Android (\d{1,}\.\d)[^;]*;/', $uaFull, $m)) {
				$os = 'Android ' . $m[1];
			}
			
			// 端末機種が取れない時2（Android xx）
			elseif (preg_match('/Android (\d{1,})[^0-9]/', $uaFull, $m)) {
				$os = 'Android ' . $m[1];
			}
			
			// デバイスを再判定
			if (preg_match('/SAMSUNG ([^\/]+)/', $device, $m)) {
				$device = $m[1];
			}
			
			////////////////////////////////////////////////////////////////////////////////
			
			// UAを再判定
			
			// Opera Mini の時
			if (preg_match('/Opera Mini\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Opera Mini ' . $m[1];
			}
			
			// Opera Mobile の時
			elseif (preg_match('/OPR\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Opera Mobile ' . $m[1];
			}
			
			// iLunascape の時
			elseif (preg_match('/iLunascape\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'iLunascape ' . $m[1];
			}
			
			// Sleipnir Mobile の時
			elseif (preg_match('/Sleipnir\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Sleipnir Mobile ' . $m[1];
			}
			
			// Firefox Mobile の時
			elseif (preg_match('/Firefox\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Firefox Mobile ' . $m[1];
			}
			
			// Edge Mobile の時
			elseif (preg_match('/EdgA\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Edge Mobile ' . $m[1];
			}
			
			// Samsung Internetの時
			elseif (preg_match('/SamsungBrowser\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Samsung Internet Mobile ' . $m[1];
			}
			
			// Chrome Mobile の時
			elseif (preg_match('/Chrome\/(\d{1,}\.\d)/', $uaFull, $m)) {
				$ua = 'Chrome Mobile ' . $m[1];
			}
			
			// その他は標準ブラウザと判定
			else {
				$ua = 'Chrome Mobile';
			}
			
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// Mac OS、OS X、macOS の時
		elseif (preg_match('/Mac OS X (\d{2})[_\.](\d{1,})/', $uaFull, $m)) {
			$os = 'Mac OS X ' . $m[1] . '.' . $m[2];
		} elseif (strpos($uaFull, 'Mac OS X') !== false) {
			$os = 'Mac OS X';
		} elseif (strpos($uaFull, 'Mac') !== false) {
			$os = 'Mac OS';
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// Linux の時
		elseif (strpos($uaFull, 'TurboLinux') !== false) {
			$os = 'Turbo Linux';
		} elseif (strpos($uaFull, 'VineLinux') !== false) {
			$os = 'Vine Linux';
		} elseif (strpos($uaFull, 'Red Hat') !== false) {
			$os = 'RedHat Linux';
		} elseif (strpos($uaFull, 'Debian') !== false) {
			$os = 'Debian Linux';
		} elseif (strpos($uaFull, 'Fedora') !== false) {
			$os = 'Fedora';
		} elseif (strpos($uaFull, 'Ubuntu') !== false) {
			$os = 'Ubuntu';
		} elseif (strpos($uaFull, 'CentOS') !== false) {
			$os = 'CentOS';
		} elseif (strpos($uaFull, 'Linux') !== false) {
			$os = 'Linux';
		}
		
		// Solaris の時
		elseif (strpos($uaFull, 'SunOS') !== false) {
			$os = 'Solaris';
		}
		
		// Chrome OS の時
		elseif (strpos($uaFull, 'CrOS') !== false) {
			$os = 'Chrome OS';
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// [スマートフォンの判定] //
		
		// フィーチャーフォンの時
		if ($carrier !== '') {
			
			// OSとUA種別を設定（統一）
			$os = 'Symbian OS';
			$uaType = 'Mobile';
			
			// docomoの時
			if ($carrier === 'docomo') {
				$ua = 'i-mode Browser';
			}
			
			// SoftBankの時
			elseif ($carrier === 'SoftBank') {
				$ua = 'NetFront Browser';
			}
			
			// auの時
			elseif ($carrier === 'au') {
				$ua = 'UP.Browser';
			}
			
		}
		
		// スマートフォンの時
		elseif ($smart) {
			
			// UA種別を設定
			$uaType = 'Mobile';
			
			// IPアドレスをリモートホストに変換
			$remoteHost = @gethostbyaddr($_SERVER['REMOTE_ADDR']);
			
			// 特定の文字列を削除
			$device = preg_replace('/HTC|SonyEricsson/', '', $device);
			$device = preg_replace('/SonySO/', 'SO', $device);
			
			// docomoの時
			if (preg_match('/spmode|mopera/', $remoteHost)) {
				$carrier = 'docomo';
			}
			
			// auの時1
			elseif (preg_match('/au-net|ezweb/', $remoteHost)) {
				$carrier = 'au';
			}
			
			// SoftBankの時1
			elseif (preg_match('/openmobile|pcsitebrowser|panda-world|bbtec/', $remoteHost)) {
				$carrier = 'SoftBank';
			}
			
			// キャリアを判定（端末機種から）
			elseif (preg_match('/[A-Z]{1,2}-\d{2}[A-Z]|FOMA|Docomo/', $device)) {
				$carrier = 'docomo';
			}
			
			// auの時2
			elseif (preg_match('/IS/A', $device)) {
				$carrier = 'au';
			}
			
			// SoftBankの時2
			elseif (preg_match('/SBM|DM|[0-9]{3}/A', $device)) {
				$carrier = 'SoftBank';
			}
			
			// auの時3
			elseif (preg_match('/[A-Z]{3}\d{2}/A', $device)) {
				$carrier = 'au';
			}
			
			// 判別出来ない時はWi-Fiと判定
			else {
				$carrier = 'Wi-Fi';
			}
			
			// 特定の文字列を削除
			$device = preg_replace('/_| |FOMA |Docomo |SBM/A', '', $device);
			
		}
		
		////////////////////////////////////////////////////////////////////////////////
		
		// リクエスト引数にUA-CHが存在する時はUAを上書き
		if (isset($_GET['ua']) && !empty($_GET['ua'])) {
			$ua = $_GET['ua'];
			$uaType = (isset($_GET['ua_type']) && !empty($_GET['ua_type'])) ? $_GET['ua_type'] : 'PC';
			$chUA = (isset($_GET['ch_ua'])) ? $_GET['ch_ua'] : '';
			$chMobile = (isset($_GET['ch_mobile'])) ? $_GET['ch_mobile'] : '';
		}
		
		// リクエストヘッダーにUA-CHが存在する時はUAを上書き
		elseif (isset($_SERVER['HTTP_SEC_CH_UA']) && !empty($_SERVER['HTTP_SEC_CH_UA'])) {
			[$ua, $uaType, $chUA, $chMobile] = self::parseClientHints($ua, $uaType, $_SERVER['HTTP_SEC_CH_UA'], $_SERVER['HTTP_SEC_CH_UA_MOBILE']);
		}
		
		// リクエスト引数にUA-CHが存在する時はOSを上書き
		if (isset($_GET['os']) && !empty($_GET['os'])) {
			$os = $_GET['os'];
		}
		
		// リクエスト引数にUA-CHが存在する時は端末機種を上書き
		if (isset($_GET['model']) && !empty($_GET['model'])) {
			$device = $_GET['model'];
		}
		
		// UA、OS、UA種別、端末種別、キャリア、UA-CH、モバイル判定 を返す
		return [$ua, $os, $uaType, $device, $carrier, $chUA, $chMobile];
		
	}
	
	//---------------------------------------------------------
	//  ブラウザ解析 関数（Client-Hints）
	//---------------------------------------------------------
	
	public static function parseClientHints($orgUA, $orgUAType, $orgCHUA, $chMobile = '?0') {
		
		// Client-Hints UA（例）
		//$chUA = '" Not A;Brand";v="99", "Chromium";v="101", "Google Chrome";v="101"';
		//$chUA = '"Opera";v="81", " Not;A Brand";v="99", "Chromium";v="95"';
		//$chUA = '"Not/A)Brand";v="8", "Chromium";v="126", "Microsoft Edge";v="126"';
		
		// UA取得フラグ
		$isGetUA = false;
		
		// UA-CH
		$chUA0 = '';
		$chUA1 = '';
		$chUA2 = '';
		
		// UA
		$ua = '';
		
		// UA追記文字列
		$mobile = '';
		
		// UA種別
		$uaType = 'PC';
		
		// モバイルの時
		if ($chMobile === '?1') {
			
			// UA追記文字列を設定
			$mobile = 'Mobile ';
			
			// UA種別を設定
			$uaType = 'Mobile';
			
		}
		
		// 「"」を削除
		$chUA0 = str_replace('"', '', $orgCHUA);
		
		// 「, 」で分割
		$array1 = explode(', ', $chUA0);
		
		// ループ
		foreach ($array1 as $val1) {
			
			// 「;v=」で分割
			$array2 = explode(';v=', $val1);
			
			// ブランド名を取得
			$brand = $array2[0];
			
			// バージョンを取得
			$version = str_replace('.0', '', $array2[1]);
			
			// ダミーデータの時は次へ
			if (strpos($brand, 'Not') !== false) {
				continue;
			}
			
			// UA未取得の時
			elseif (!$isGetUA) {
				
				// UAを取得
				if (strpos($brand, 'HeadlessChrome') !== false) {
					$ua = 'Headless Chrome ';
				} elseif (strpos($brand, 'Chrome') !== false) {
					$ua = 'Chrome ';
				} elseif (strpos($brand, 'Edge') !== false) {
					$ua = 'Edge ';
				} elseif (strpos($brand, 'Opera') !== false) {
					$ua = 'Opera ';
				} elseif (strpos($brand, 'Safari') !== false) {
					$ua = 'Safari ';
				} elseif (strpos($brand, 'Firefox') !== false) {
					$ua = 'Firefox ';
				} elseif (strpos($brand, 'Brave') !== false) {
					$ua = 'Brave ';
				} elseif (strpos($brand, 'Whale') !== false) {
					$ua = 'Whale ';
				} elseif (strpos($brand, 'Yandex') !== false) {
					$ua = 'Yandex ';
				} elseif (strpos($brand, 'YaBrowser') !== false) {
					$ua = 'Yandex ';
				} elseif (strpos($brand, 'Samsung Internet') !== false) {
					$ua = 'Samsung Internet ';
				} elseif (strpos($brand, 'Android WebView') !== false) {
					$ua = 'Android WebView ';
				} elseif (strpos($brand, 'Avast Secure Browser') !== false) {
					$ua = 'Avast Secure Browser ';
				}
				
				// UAを取得した時
				if (!empty($ua)) {
					
					// UA取得フラグを設定
					$isGetUA = true;
					
					// バージョンを追記
					$ua .= $mobile . $version . '.0';
					
					// 値を保持
					$chUA2 = $val1;
					
				}
				
				// 未知のUA、UA以外の情報の時（Chromium等）
				else {
					
					// 2個目以降の時は「, 」を追加
					if (!empty($chUA1)) {
						$chUA1 .= ', ';
					}
					
					// 値を追加
					$chUA1 .= $val1;
					
				}
				
			}
			
			// UA以外の情報の時（Chromium等）
			else {
				$chUA1 = $val1;
			}
			
		}
		
		// 保存用UA-CHを設定
		$chUA = (!empty($chUA2)) ? $chUA1 . ', ' . $chUA2 : $chUA1;
		
		// UAが取得出来た時はUAとUA種別を返す
		if ($ua) {
			return [$ua, $uaType, $chUA, $chMobile];
		}
		
		// UAが取得出来なかった時は元データを返す
		else {
			return [$orgUA, $orgUAType, $chUA, $chMobile];
		}
		
	}
	
}

