
// 呼び出し元のscript要素を取得
var element = document.getElementById('lunalys');

// JavaScriptのパスをPHPのパスに変換
var phpPath = element.src.replace(/tracker.js/, 'write.php');

// ドメイン一致の正規表現
var regExp = new RegExp('^(http|https)://' + document.domain);

// リンク元を取得
var referrer = parent.document.referrer;

// 同一ドメインの場合はリンク元を削除
if (referrer.match(regExp)) {
	referrer = '';
}

// サイト番号が存在する時は取得
var site = (element.dataset.site) ? '&site=' + element.dataset.site : '';

//////////////////////////////////////////////////////////////

// URL引数
var args = '';

// ディスプレイの幅を追記
args += '&display_width=' + screen.width;

// ディスプレイの高さを追記
args += '&display_height=' + screen.height;

//////////////////////////////////////////////////////////////

// documentElement が使える時
if (document.documentElement) {
	
	// ブラウザ表示領域の幅を追記
	args += '&client_width=' + parent.document.documentElement.clientWidth;
	
	// ブラウザ表示領域の高さを追記
	args += '&client_height=' + parent.document.documentElement.clientHeight;
	
}

//////////////////////////////////////////////////////////////

// encodeURIComponent が使える時
if (window.encodeURIComponent) {
	
	// ページURLを追記
	args += '&url=' + encodeURIComponent(document.URL);
	
	// ページタイトルを追記
	args += '&title=' + encodeURIComponent(document.title);
	
	// リンク元を追記
	if (referrer){
		args += '&referrer=' + encodeURIComponent(referrer);
	}
	
}

//////////////////////////////////////////////////////////////

// encodeURIComponent が使えない時
else {
	
	// ページURLを追記
	args += '&url=' + encodeStr(document.URL);
	
	// ページタイトルを追記
	args += '&title=' + encodeStr(document.title);
	
	// リンク元を追記
	if (referrer){
		args += '&referrer=' + encodeStr(referrer);
	}
	
}

//////////////////////////////////////////////////////////////

// リクエストURLを設定
var url = phpPath + '?act=js' + args + site + '&cache=' + (new Date()).getTime();

// UA-CHを取得後にリクエスト送信
if (navigator.userAgentData) {
	addClientHints();
}

// 今すぐリクエスト送信
else {
	addElement(url);
}

// ページ読み込み完了後にイベントを設定
if (window.addEventListener) {
	window.addEventListener('load', setEvent, {passive: false});
}

//////////////////////////////////////////////////////////////

// 文字列をURLエンコード
function encodeStr(str) {
	
	// 特定の文字をエンコード
	str = str.replace(/\?/g,'%3F');
	str = str.replace(/&/g,'%26');
	str = str.replace(/#/g,'%23');
	str = str.replace(/=/g,'%3D');
	str = str.replace(/ /g,'%20');
	
	// 変換後の文字列を返す
	return str;
	
}

//////////////////////////////////////////////////////////////

// リンククリック時のイベントを設定
function setEvent() {
	
	// 正規表現を設定（対象ドメイン）
	var regExp = new RegExp('^(http|https)://' + document.domain);
	
	// 正規表現を設定（既知の拡張子）
	var pageExt = /\.(htm|php|cgi|jsp|asp)/i;
	
	// 正規表現を設定（未知の拡張子）
	var fileExt = /\.\w{2,4}$/i;
	
	// イベント種別を設定
	var eventType = (window.ontouchstart === undefined) ? 'mousedown' : 'touchstart';
	
	// リンクの総数を取得
	var length = document.getElementsByTagName('a').length;
	
	// リンクの数だけループ
	for (i = 0; i < length; i++) {
		
		// a要素を取得
		var node = document.getElementsByTagName('a')[i];
		
		// href属性を取得
		var href = node.href;
		
		// JavaScript実行の時は次へ
		if (href.match('javascript')) {
			continue;
		}
		
		// 違うドメイン、ファイルダウンロードの時はイベントを設定
		else if (!href.match(regExp) || (!href.match(pageExt) && href.match(fileExt))) {
			node.addEventListener(eventType, clickLink, {passive: true});
		}
		
	}
	
}

//////////////////////////////////////////////////////////////

// リンククリック時にリクエスト送信
function clickLink() {
	
	// URL引数
	var args = '';
	
	// encodeURIComponent が使える時
	if (window.encodeURIComponent) {
		
		// URLを追記
		args += '&url=' + encodeURIComponent(this.href);
		
		// タイトルを追記
		args += '&title=' + encodeURIComponent(this.innerHTML);
		
	}
	
	// encodeURIComponent が使えない時
	else {
		
		// URLを追記
		args += '&url=' + encodeStr(this.href);
		
		// タイトルを追記
		args += '&title=' + encodeStr(this.innerHTML);
		
	}
	
	// リクエストURLを設定
	var url = phpPath + '?act=click' + args + site;
	
	// リクエスト送信
	addElement(url);
	
}

//////////////////////////////////////////////////////////////

// リクエスト送信
function addElement(url) {
	
	// テスト用出力
	//console.log(url);
	
	// 要素を作成
	var script = document.createElement('script');
	
	// 属性を設定
	script.setAttribute('src', url);
	script.setAttribute('type', 'text/javascript');
	script.setAttribute('async', true);
	
	// head要素を取得
	var head = document.getElementsByTagName('head')[0];
	
	// head要素の最後に追記
	head.appendChild(script);
	
}

//////////////////////////////////////////////////////////////

// UA-CHからUAとOSを取得
function addClientHints() {
	
	// UA取得フラグ
	var isGetUA = false;
	
	// UA-CH
	var chUA0 = '';
	var chUA1 = '';
	var chUA2 = '';
	
	// モバイル判定
	var chMobile = '%3F0';
	
	// UA追記文字列
	var mobile = '';
	
	// UA種別
	var uaType = 'PC';
	
	// モバイルの時
	if (navigator.userAgentData.mobile) {
		
		// UA追記文字列を設定
		mobile = 'Mobile ';
		
		// UA種別を設定
		uaType = 'Mobile';
		
		// モバイル判定
		chMobile = '%3F1';
		
	}
	
	// ブランド情報を取得
	var brands = navigator.userAgentData.brands;
	
	// 配列数を取得
	var length = brands.length;
	
	// 子要素の分だけループ
	for (var i = 0; i < length; i++) {
		
		// ブランドを取得
		var brand = brands[i].brand;
		
		// バージョンを取得
		var version = brands[i].version;
		
		// ダミーデータの時は次へ
		if (brand.match(/Not/i)) {
			continue;
		}
		
		// UA未取得の時
		else if (!isGetUA) {
			
			// UAを初期化
			var ua = '';
			
			// UAを取得
			if (brand.match(/HeadlessChrome/i)) {
				ua = 'Headless Chrome ';
			} else if (brand.match(/Chrome/i)) {
				ua = 'Chrome ';
			} else if (brand.match(/Edge/i)) {
				ua = 'Edge ';
			} else if (brand.match(/Opera/i)) {
				ua = 'Opera ';
			} else if (brand.match(/Safari/i)) {
				ua = 'Safari ';
			} else if (brand.match(/Firefox/i)) {
				ua = 'Firefox ';
			} else if (brand.match(/Brave/i)) {
				ua = 'Brave ';
			} else if (brand.match(/Whale/i)) {
				ua = 'Whale ';
			} else if (brand.match(/YaBrowser|Yandex/i)) {
				ua = 'Yandex ';
			} else if (brand.match(/Samsung Internet/i)) {
				ua = 'Samsung Internet ';
			} else if (brand.match(/Android WebView/i)) {
				ua = 'Android WebView ';
			} else if (brand.match(/Avast Secure Browser/i)) {
				ua = 'Avast Secure Browser ';
			}
			
			// UAを取得した時
			if (ua) {
				
				// UA取得フラグを設定
				isGetUA = true;
				
				// UAの引数を追加
				url += '&ua=' + ua + mobile + version + '.0&ua_type=' + uaType;
				
				// 値を保持
				chUA2 = brand + '%3Bv%3D' + version;
				
			}
			
			// 未知のUA、UA以外の情報の時（Chromium等）
			else {
				
				// 2回目以降の時は「, 」を追加
				if (chUA1) {
					chUA1 += '%2C%20';
				}
				
				// 値を追加
				chUA1 += brand + '%3Bv%3D' + version;
				
			}
			
		}
		
		// UA以外の情報の時（Chromium等）
		else {
			chUA1 = brand + '%3Bv%3D' + version;
		}
		
	}
	
	// UA-CHを設定
	var chUA = (chUA2) ? chUA1 + '%2C%20' + chUA2 : chUA1;
	
	// UA-CHとモバイル判定を追加
	url += '&ch_ua=' + chUA + '&ch_mobile=' + chMobile;
	
	//////////////////////////////////////////////////////////////
	
	// 非同期で最新OSの情報を取得（User-Agent Client Hints）
	navigator.userAgentData.getHighEntropyValues([
		'platform', 
		'platformVersion', 
		'model'
	]).then(ua => {
		
		// OS
		var os = '';
		
		// バージョンを「.」で分割して配列で取得
		var versions = ua.platformVersion.split('.');
		
		// メジャーバージョンを取得
		var majorVersion = parseInt(versions[0]);
		
		// Windows の時
		if (navigator.userAgentData.platform === 'Windows') {
			
			// Windows 11 以降の時
			if (majorVersion >= 13) {
				os = 'Windows 11';
			}
			
		}
		
		// macOS の時
		else if (navigator.userAgentData.platform === 'macOS') {
			
			// macOS Big Sur 以降の時
			if (majorVersion >= 11) {
				os = 'Mac OS X ' + majorVersion + '.0';
			}
			
		}
		
		// 最新のOSを取得した時はURLに追記
		if (os) {
			url += '&os=' + os;
		}
		
		// モバイルで機種名を取得出来た時はURLに追記
		if (navigator.userAgentData.mobile && ua.model) {
			url += '&model=' + ua.model;
		}
		
		// リクエスト送信
		addElement(url);
		
	}).catch((e) => {
		
		// リクエスト送信
		addElement(url);
		
	});
	
}

